(* --------------------------------------------------------------------------

:Program.    Disp.mod
:Contents.   Program to display a CPic - File on the Screen.
:Author.     Thomas Zipproth
:Address.    Dr. Jochner Weg 10, 8948 Mindelheim, Germany
:Phone.      08261/4838
:Copyright.  PD
:Language.   Modula-2
:Translator. M2-Amiga A+l V3.3d
:Imports.    CPicSupport [thz]
:History.    V1.1 [thz] 26.7.90
:Usage.      Disp <file>
:Usage.      file  : Name of a CPic File

---------------------------------------------------------------------------*)

(* $S- $F- $V- $R- *)
MODULE Disp;

FROM Arts      IMPORT TermProcedure,wbStarted;
FROM Arguments IMPORT NumArgs,GetArg;
FROM SYSTEM    IMPORT ADR,SETREG;
FROM Dos       IMPORT FileHandlePtr,Close,Output,Write;
FROM Exec      IMPORT WaitPort,GetMsg,ReplyMsg;
FROM Graphics  IMPORT BitMapPtr,FreeSprite;
FROM Intuition IMPORT CloseScreen,ScreenPtr,customScreen,ScreenToFront,
                      MoveScreen,WindowFlags,WindowFlagSet,
                      NewWindow,OpenWindow,CloseWindow,WindowPtr,ShowTitle,
                      IDCMPFlags,IDCMPFlagSet,IntuiMessage,IntuiMessagePtr;
FROM CPicSupport IMPORT ReadInfo,OpenCPicScreen,LoadBitplanes,ci,Cle;


VAR fp : FileHandlePtr;
    sp : ScreenPtr;
    bp : BitMapPtr;
    nw : NewWindow;
    wp : WindowPtr;
    len : INTEGER;
    Name : ARRAY[1..79] OF CHAR;
    OutHandle: FileHandlePtr;


PROCEDURE CleanUp;
BEGIN
  (* $Z- *)
  IF fp # NIL THEN Close(fp) END;
  IF wp # NIL THEN CloseWindow(wp) END;
  IF sp # NIL THEN CloseScreen(sp) END;
  (* $Z+ *)
END CleanUp;


PROCEDURE WriteString(str: ARRAY OF CHAR);
VAR length,l: INTEGER;
BEGIN
  length:=0; l:=HIGH(str);
  WHILE (length<=l) AND (str[length] # 00C) DO
    INC(length)
  END;
  SETREG(0,Write(OutHandle,ADR(str),length))
END WriteString;


PROCEDURE WriteLn;
VAR ch : CHAR;
BEGIN
  ch := 12C;
  SETREG(0,Write(OutHandle,ADR(ch),1));
END WriteLn;


PROCEDURE Wait;
VAR toggle : BOOLEAN;
    code   : INTEGER;
    mp     : IntuiMessagePtr;
BEGIN;
  toggle:=TRUE;
  REPEAT
    WaitPort(wp^.userPort);
    mp:=GetMsg(wp^.userPort);
    code:=mp^.code;
    ReplyMsg(mp);
    IF mp^.class=IDCMPFlagSet{mouseButtons} THEN
      IF mp^.mouseX#0 THEN ShowTitle(sp,toggle);toggle:=NOT toggle;
        ELSE code:=69
      END;
    END;
  UNTIL code=69;
END Wait;


PROCEDURE CPicLoad() : BOOLEAN;

BEGIN
  IF NOT ReadInfo(Name,ci,fp) THEN RETURN FALSE END;

  IF NOT OpenCPicScreen(ci,sp,bp,0,600,
                        ADR("Esc or click left side to quit"),FALSE)
    THEN RETURN FALSE
  END;

  ShowTitle(sp,FALSE);

  WITH nw DO
    width:=ci.Width;height:=ci.Height-10;
    idcmpFlags:=IDCMPFlagSet{rawKey,mouseButtons};
    flags:=WindowFlagSet{borderless,activate,backDrop};
    screen:=sp;type:=customScreen;
  END;   (* All other 0 because nw global variable *)
  wp:=OpenWindow(nw);
  IF wp=NIL THEN RETURN FALSE END;

  IF NOT LoadBitplanes(ci,fp,bp,ci.PicX0,ci.PicY0,0) THEN RETURN FALSE END;

  MoveScreen(sp,0,-600); FreeSprite(0);
  ScreenToFront(sp); Close(fp); fp:=NIL;
  RETURN TRUE
END CPicLoad;


BEGIN

TermProcedure(CleanUp);

IF NOT wbStarted THEN OutHandle:=Output() END;

IF NumArgs()=1 THEN
  GetArg(1,Name,len);
  IF CPicLoad() THEN
    Wait;
  ELSE
    IF NOT wbStarted THEN
      IF Cle=10 THEN WriteString("Couldn't open File !");
        ELSIF Cle=11 THEN WriteString("File is not a CPic-File !");
        ELSIF Cle=12 THEN WriteString("Not enough Chip-Mem !");
        ELSIF Cle=15 THEN WriteString("Not enough Memory !");
        ELSIF Cle=16 THEN WriteString("Read - Error !");
        ELSIF ((Cle=13) OR (Cle=17)) THEN
          WriteString("File contains wrong data !");
        ELSE WriteString("Couldn't load File !");
      END;
      WriteLn;WriteLn;
    END;
  END;
ELSE
  IF NOT wbStarted THEN
    WriteLn;
    WriteString("Disp V1.1 26.7.90 by Thomas Zipproth");
    WriteLn;WriteLn;
    WriteString("Usage:  Disp <CPic - File>");
    WriteLn;WriteLn;
  END;
END;

END Disp.
