(* --------------------------------------------------------------------------

:Program.    Show.mod
:Contents.   Program to display CPic - Files on the Screen.
:Author.     Thomas Zipproth
:Address.    Dr. Jochner Weg 10, 8948 Mindelheim, Germany
:Phone.      08261/4838
:Copyright.  PD
:Language.   Modula-2
:Translator. M2-Amiga A+l V3.3d
:Imports.    CPicSupport [thz]
:Imports.    ToolSupport [thz]
:History.    V1.1 [thz] 26.7.90
:Usage.      Disp <Skript> <mode> <time> <count>
:Usage.      Skript : Text - file with one picture - name in every line
:Usage.      mode   : 0 for simple change Screens, 1 for scrolling
:Usage.      time   : Time in seconds to display every picture
:Usage.      count  : How often to display all Pictures

--------------------------------------------------------------------------- *)

(* $S- $F- $R- $V- *)
MODULE Show;

FROM Arts        IMPORT Assert,TermProcedure;
FROM SYSTEM      IMPORT ADR;
FROM Dos         IMPORT Open,Close,Seek,Read,FileHandlePtr,oldFile,Delay;
FROM Intuition   IMPORT ScreenPtr,CloseScreen,ScreenToFront,MoveScreen;
FROM Graphics    IMPORT ViewModes,ViewModeSet,FreeSprite;
FROM ToolSupport IMPORT GetArg,NumArgs,WriteString,WriteLn,StrToInt;
FROM CPicSupport IMPORT LoadCPicScreen,ci;


TYPE PicName = ARRAY[1..24] OF CHAR;

VAR sp :      ARRAY[1..2] OF ScreenPtr;
    Names :   ARRAY[1..120] OF PicName;
    Skript :  ARRAY[1..1400] OF CHAR;
    Wechsel : ARRAY[1..2] OF INTEGER;
    Name :    ARRAY[1..32] OF CHAR;
    i,j,Anz,Nr,len,mode,time,count : INTEGER;
    err : BOOLEAN;


PROCEDURE Ausstieg;
BEGIN
  IF sp[1]#NIL THEN CloseScreen(sp[1]) END;
  IF sp[2]#NIL THEN CloseScreen(sp[2]) END;
END Ausstieg;


PROCEDURE CPicLoad(n,p:INTEGER);

VAR i,m : INTEGER;

BEGIN

  IF sp[n]#NIL THEN CloseScreen(sp[n]);sp[n]:=NIL END;
  err:=LoadCPicScreen(Names[p],sp[n],0,600,FALSE);
  Assert(err,ADR("Couldn't load picture"));
  IF mode = 0 THEN
    MoveScreen(sp[n],0,-600); FreeSprite(0); ScreenToFront(sp[n]);
  ELSE
    IF (lace IN ci.Mode) THEN m := -4; MoveScreen(sp[n],0,-40)
                         ELSE m := -2; MoveScreen(sp[n],0,-320)
    END;
    FreeSprite(0); ScreenToFront(sp[n]);
    FOR i:=1 TO 140 DO MoveScreen(sp[n],0,m) END;
  END;
  Delay(time);
END CPicLoad;


PROCEDURE ScanSkript;
VAR size,dummy : LONGINT;
    i,j,k : INTEGER;
    fp : FileHandlePtr;
BEGIN
  fp:=Open(ADR(Name),oldFile);
  Assert(fp#NIL,ADR("Konnte Skript nicht finden"));
  size:=Seek(fp,0,1);size:=Seek(fp,0,-1);
  Assert(size<=1400,ADR("Skriptfile > 1.4 KB"));
  dummy:=Read(fp,ADR(Skript[1]),size);
  Close(fp);
  IF Skript[size]#12C THEN INC(size);Skript[size]:=12C END;
  i:=1;j:=1;
  FOR k:=1 TO size DO
    IF Skript[k]#12C THEN
      Names[i,j]:=Skript[k];INC(j);
    ELSE
      Assert(j<25,ADR("Picname >24 letters"));
      IF j>1 THEN Names[i,j]:=00C;INC(i);j:=1;INC(Anz) END;
    END;
  END;
END ScanSkript;

BEGIN

TermProcedure(Ausstieg);
IF NumArgs()=4 THEN
  Wechsel[1]:=2;Wechsel[2]:=1;Nr:=1;

  GetArg(1,Name,len);ScanSkript;
  Assert(Anz<=120,ADR("Max 120 Pictures"));

  GetArg(2,Name,len);StrToInt(Name,mode);
  IF ((mode#0)AND(mode#1)) THEN mode:=0 END;

  GetArg(3,Name,len);StrToInt(Name,time);
  IF ((time<0) OR err) THEN time:=2 END;
  time:=time*50;

  GetArg(4,Name,len);StrToInt(Name,count);
  IF err THEN count:=1 END;

  FOR j:=1 TO count DO
    FOR i:=1 TO Anz DO
      CPicLoad(Nr,i);Nr:=Wechsel[Nr];
    END;
  END;
  IF time=0 THEN Delay(50) END;
ELSE
  WriteLn;
  WriteString("Show V1.1 26.7.90 by Thomas Zipproth");
  WriteLn;WriteLn;
  WriteString("Usage : Show <Skript> <mode> <time> <count>");
  WriteLn;WriteLn;
  WriteString("Skript : Text - file with one CPic - name in every line");
  WriteLn;
  WriteString("mode   : 0 for simple change Screens, 1 for scrolling");
  WriteLn;
  WriteString("time   : Time in seconds to display every picture");
  WriteLn;
  WriteString("count  : How often to display all Pictures");
  WriteLn;WriteLn;
END;
END Show.
