(* --------------------------------------------------------------------------

:Program.    CPicSupport.mod
:Contents.   Procedures to handle the CPic - Format.
:Author.     Thomas Zipproth
:Address.    Dr. Jochner Weg 10, 8948 Mindelheim, Germany
:Phone.      08261/4838
:Copyright.  Shareware (refer to CPic.doc)
:Language.   Modula2 + Assembler
:Translator. M2-Amiga A+L V3.3d
:History.    V1.0 [thz] 10.7.90
:History.    V1.1 [thz] 12.8.90  Crunch - procedure in Assembler

-------------------------------------------------------------------------- *)

(* $S- $V- $R- $F- $N- *)
IMPLEMENTATION MODULE CPicSupport;

FROM Arts      IMPORT TermProcedure,DetectCtrlC;
FROM SYSTEM    IMPORT ADR,ADDRESS,INLINE,SETREG;
FROM Dos       IMPORT Open,Close,Read,Write,Seek,FileHandlePtr,newFile,oldFile,
                      beginning,DeleteFile;
FROM Exec      IMPORT CopyMem,UByte,AllocMem,FreeMem,MemReqSet;
FROM Graphics  IMPORT BitMapPtr,RastPortPtr,ViewModes,ViewModeSet,LoadRGB4,
                      ViewPortPtr,RectFill,SetAPen;
FROM Intuition IMPORT OpenScreen,CloseScreen,customScreen,NewScreen,ScreenPtr,
                      ScreenToBack,ScreenFlagSet,ScreenFlags,MoveScreen;


CONST RecLen = 132; (* Length of CPicInfo - Record *)

VAR fp1,fp2 : FileHandlePtr;      (* for saving and loading *)
    Buffer1,Buffer2 : ADDRESS;    (* Buffer for crunching and decrunching *)
    BufSize1,BufSize2 : LONGINT;  (* = one Bitplane of rectangle + 800 *)


PROCEDURE CleanUp;
BEGIN
  (* $Z- *)
  IF fp1 # NIL THEN Close(fp1) END;
  IF fp2 # NIL THEN Close(fp2) END;
  IF Buffer1 # NIL THEN FreeMem(Buffer1,BufSize1) END;
  IF Buffer2 # NIL THEN FreeMem(Buffer2,BufSize2) END;
  (* $Z+ *)
END CleanUp;


(* $E- *)
PROCEDURE RectToString(a{8},b{9} : ADDRESS; c{0},d{1},e{2} : LONGINT);
BEGIN
  INLINE(05380H,05381H,02601H,02449H,010D2H,0D5C2H,
         051CBH,0FFFAH,05289H,051C8H,0FFF0H,04E75H);
END RectToString;


(* $E- *)
PROCEDURE Crunch(S{8},D{9} : ADDRESS; l{0} : LONGINT) : LONGINT;
BEGIN
INLINE(
(*0000*) 02C48H,05380H,0DDC0H,02649H,02448H,07401H,07803H,07A60H,
(*0010*) 07C01H,07E21H,04A80H,06606H,07601H,06172H,06036H,01218H,
(*0020*) 0B210H,06612H,05282H,0B484H,06616H,02608H,0968AH,05583H,
(*0030*) 0630EH,0615AH,0600AH,0B484H,06504H,02448H,0611CH,07401H,
(*0040*) 0BDC8H,062DAH,0B484H,06504H,06110H,06008H,02608H,0968AH,
(*0050*) 05283H,0613AH,02009H,0908BH,04E75H,0B485H,0630AH,012FCH,
(*0060*) 000FFH,012C1H,09485H,060F2H,0B487H,06418H,04A01H,06606H,
(*0070*) 05382H,012C2H,04E75H,00C01H,000FFH,06608H,00602H,0007FH,
(*0080*) 012C2H,04E75H,00602H,0009FH,012C2H,012C1H,04E75H,0B686H,
(*0090*) 06616H,01012H,04A00H,06604H,04219H,04E75H,00C00H,000FFH,
(*00A0*) 06606H,012FCH,00080H,04E75H,0B685H,06316H,012FCH,0007FH,
(*00B0*) 07017H,012DAH,012DAH,012DAH,012DAH,051C8H,0FFF6H,09685H,
(*00C0*) 060E6H,0701FH,0D003H,012C0H,05303H,012DAH,051CBH,0FFFCH,
(*00D0*) 04E75H);
END Crunch;


PROCEDURE SaveRect(Name : ARRAY OF CHAR;
                   bp : BitMapPtr; vp : ViewPortPtr;
                   x0,y0,x,y : LONGINT) : LONGINT;

VAR i : INTEGER;
    pos,len,l,PicSize1,Rows1,BpR1 : LONGINT;
    dn : ARRAY[0..80] OF CHAR;
    ad1,ad2 : ADDRESS;
    Error : BOOLEAN;

BEGIN
  Cse:=0; Rows1:=bp^.rows; BpR1:=bp^.bytesPerRow; PicSize1:=x*y;
  IF ((x0<0) OR (y0<0) OR (x<1) OR (y<1)) THEN Cse:=10; RETURN 0 END;
  IF ((x0+x > BpR1) OR (y0+y > Rows1)) THEN Cse:=11; RETURN 0 END;

  ci.Ident  := 1129343331;
  ci.Width  := BpR1*8;
  ci.Height := Rows1;
  ci.PicX0  := x0;
  ci.PicY0  := y0;
  ci.PicX   := x;
  ci.PicY   := y;
  ci.Depth  := bp^.depth;
  ci.Mode   := vp^.modes;
  CopyMem(vp^.colorMap^.colorTable,ADR(ci.Colors),64);

  l := HIGH(Name);      (* trouble with open array parameters *)
  FOR i:=0 TO l DO dn[i] := Name[i] END; dn[l+1]:=00C;
  fp1:=NIL; fp1:=Open(ADR(dn),newFile);
  IF fp1=NIL THEN Cse:=12; RETURN 0 END;
  len:=Write(fp1,ADR(ci),RecLen);
  IF len # RecLen THEN
    (* $Z- *) Close(fp1);(* $Z+ *) fp1:=NIL; Error:=DeleteFile(ADR(Name));
    Cse:=13;RETURN 0
  END;

  Buffer1:=NIL;BufSize1:=PicSize1+800;
  (* $Z- *) Buffer1:=AllocMem(BufSize1,MemReqSet{}); (* $Z+ *)
  IF Buffer1=NIL THEN Cse:=14;RETURN 0 END;

  pos:=RecLen;
  FOR i:=0 TO ci.Depth-1 DO
    ad1:=bp^.planes[i]+y0*BpR1+x0; ad2:=Buffer1+780;
    RectToString(ad2,ad1,x,y,BpR1);
    l:=Crunch(ad2,Buffer1,PicSize1);
    len := Write(fp1,Buffer1,l);
    IF len#l THEN
      (* $Z- *) Close(fp1); (* $Z+ *) fp1:=NIL;Error:=DeleteFile(ADR(Name));
      (* $Z- *) FreeMem(Buffer1,BufSize1);(* $Z+ *) Buffer1:=NIL;
      Cse:=13;RETURN 0
    END;
    ci.Planes[i]:=pos;ci.Length[i]:=l;INC(pos,l);
  END;
  len := Seek(fp1,0,beginning);
  len := Write(fp1,ADR(ci),RecLen);
  (* $Z- *) Close(fp1);(* $Z+ *) fp1:=NIL;
  (* $Z- *) FreeMem(Buffer1,BufSize1); (* $Z+ *) Buffer1:=NIL;
  RETURN pos;

END SaveRect;


PROCEDURE SaveBitMap(Name : ARRAY OF CHAR;
                     bp : BitMapPtr; vp : ViewPortPtr) : LONGINT;

VAR j,k,start,BpR1,Rows1,X,Y,X0,X1,Y0,Y1 : LONGINT;
    m,depth1 : INTEGER;
    bpr : POINTER TO ARRAY[1..1200000] OF UByte;

  PROCEDURE CheckLine(n:LONGINT) : BOOLEAN;  (* Checks every byte for 0 *)
  BEGIN
    start:=n*BpR1+1;
    FOR m:=0 TO depth1-1 DO
      bpr:=bp^.planes[m];
      FOR j:=start TO start+BpR1-1 DO
        IF bpr^[j]#0 THEN RETURN TRUE END;
      END;
    END;RETURN FALSE;
  END CheckLine;

  PROCEDURE CheckColumn(n:LONGINT) : BOOLEAN;  (* Checks every byte for 0 *)
  BEGIN
    start:=n+1;
    FOR m:=0 TO depth1-1 DO
      bpr:=bp^.planes[m];k:=start;
      FOR j:=0 TO Rows1-1 DO
        IF bpr^[k]#0 THEN RETURN TRUE END; INC(k,BpR1);
      END;
    END; RETURN FALSE;
  END CheckColumn;

BEGIN
  depth1 := bp^.depth;
  BpR1   := bp^.bytesPerRow;
  Rows1  := bp^.rows;

  Y0:=-1;   REPEAT INC(Y0); UNTIL (CheckLine(Y0)   OR (Y0=Rows1-1));
  Y1:=Rows1;REPEAT DEC(Y1); UNTIL (CheckLine(Y1)   OR (Y1=0));
  X0:=-1;   REPEAT INC(X0); UNTIL (CheckColumn(X0) OR (X0=BpR1-1));
  X1:=BpR1; REPEAT DEC(X1); UNTIL (CheckColumn(X1) OR (X1=0));
  X:=X1-X0+1; Y:=Y1-Y0+1;
  IF X < 1 THEN X0:=0; Y0:=0; X:=BpR1; Y:=Rows1 END;

  RETURN SaveRect(Name,bp,vp,X0,Y0,X,Y);
END SaveBitMap;


PROCEDURE SaveScreen(Name : ARRAY OF CHAR; sp : ScreenPtr) : LONGINT;
VAR rp : RastPortPtr;
    bp : BitMapPtr;
    vp : ViewPortPtr;
BEGIN
  Cse := 0; IF sp = NIL THEN Cse:=15; RETURN 0 END;
  rp := ADR(sp^.rastPort);
  bp := rp^.bitMap;
  vp := ADR(sp^.viewPort);
  RETURN SaveBitMap(Name,bp,vp);
END SaveScreen;


(* ----------------------------------------------------------------------*)


(* $E- *)
PROCEDURE StringToRect(a{8},b{9} : ADDRESS; c{0},d{1},e{2} : LONGINT);
BEGIN
  INLINE(05380H,05381H,02601H,02449H,01498H,0D5C2H,
         051CBH,0FFFAH,05289H,051C8H,0FFF0H,04E75H);
END StringToRect;


(* $E- *)
PROCEDURE Decrunch(a{8} : ADDRESS; b{9} : ADDRESS; c{0} : LONGINT) : LONGINT;
BEGIN
  INLINE(02E08H,02449H,0D5C0H,07200H,07400H,07620H,0183CH,00080H,
         01A3CH,000A0H,01C3CH,000FFH,07000H,01018H,0B003H,06408H,
         012C2H,051C8H,0FFFCH,06026H,0B004H,0640AH,09003H,012D8H,
         051C8H,0FFFCH,06018H,0B005H,0640AH,09004H,012C6H,051C8H,
         0FFFCH,0600AH,01218H,09005H,012C1H,051C8H,0FFFCH,0B5C9H,
         06EC6H,02008H,09087H,04E75H);
END Decrunch;


(* $E- *)
PROCEDURE SetRect(b{9} : ADDRESS; c{0},d{1},e{2} : LONGINT; f{3} : UByte);
BEGIN
  INLINE(05380H,05381H,02801H,02449H,01483H,0D5C2H,
         051CCH,0FFFAH,05289H,051C8H,0FFF0H,04E75H);
END SetRect;


PROCEDURE ReadInfo(Name   : ARRAY OF CHAR;
                   VAR ci : CPicInfo;
                   VAR fp : FileHandlePtr) : BOOLEAN;
VAR l  : LONGINT;
    i  : INTEGER;
    dn : ARRAY[0..80] OF CHAR;
BEGIN
  l := HIGH(Name);      (* trouble with open array parameters *)
  FOR i:=0 TO l DO dn[i] := Name[i] END; dn[l+1]:=00C;
  Cle:=0; fp:=NIL; fp:=Open(ADR(dn),oldFile);
  IF fp=NIL THEN Cle:=10; RETURN FALSE END;
  l:=Seek(fp,0,-1);
  l:=Read(fp,ADR(ci),RecLen);
  IF ci.Ident # 1129343331 THEN
    (* $Z- *) Close(fp);(* $Z+ *) fp:=NIL; Cle:=11; RETURN FALSE;
  END;
  RETURN TRUE;
END ReadInfo;


PROCEDURE OpenCPicScreen(VAR ci : CPicInfo;
                         VAR sp : ScreenPtr; VAR bp : BitMapPtr;
                         x0,y0 : INTEGER;
                         title : ADDRESS; Front : BOOLEAN) : BOOLEAN;
VAR ns : NewScreen;
    rp : RastPortPtr;
BEGIN
  Cle:=0; sp:=NIL; bp:=NIL;
  WITH ns DO
    leftEdge:=x0;topEdge:=y0;width:=ci.Width;height:=ci.Height;
    depth:=ci.Depth;detailPen:=0;
    IF title=NIL THEN
      blockPen:=0;type:=customScreen+ScreenFlagSet{screenQuiet};
    ELSE
      blockPen:=1;type:=customScreen;
    END;
    viewModes:=ci.Mode;
    font:=NIL;defaultTitle:=title;
    gadgets:=NIL;customBitMap:=NIL;
  END;
  IF NOT Front THEN ns.topEdge:=600 END;

  DetectCtrlC(FALSE);
  sp:=OpenScreen(ns);
  IF sp=NIL THEN Cle:=12; DetectCtrlC(TRUE); RETURN FALSE; END;
  rp := ADR(sp^.rastPort); bp := rp^.bitMap;
  DetectCtrlC(TRUE);

  IF NOT Front THEN ScreenToBack(sp); MoveScreen(sp,0,y0-600) END;
  LoadRGB4(ADR(sp^.viewPort),ADR(ci.Colors),32);
  IF title = NIL THEN
    SetAPen(rp,0);RectFill(rp,0,0,ci.Width-1,9)
  END;
  RETURN TRUE;
END OpenCPicScreen;


PROCEDURE LoadBitplanes(VAR ci : CPicInfo;
                        fp : FileHandlePtr; bp : BitMapPtr;
                        x0,y0,mode : INTEGER) : BOOLEAN;
VAR i,depth,BpR,rows : INTEGER;
    PicSize,PicStart,dummy : LONGINT;
    ad1,ad2,ad3 : ADDRESS;
    num : UByte;

BEGIN
  Cle:=0;
  depth := bp^.depth;
  BpR   := bp^.bytesPerRow;
  rows  := bp^.rows;
  IF (x0<0) OR (y0<0) THEN Cle:=13; RETURN FALSE END;
  IF ((x0+ci.PicX>BpR) OR (y0+ci.PicY>rows)) THEN Cle:=13; RETURN FALSE END;
  IF ci.Depth>depth THEN Cle:=14; RETURN FALSE END;

  PicSize := LONGINT(ci.PicX) * LONGINT(ci.PicY);
  BufSize2 := PicSize + 800;
  Buffer2:=NIL; (* $Z- *) Buffer2:=AllocMem(BufSize2,MemReqSet{}); (* $Z+ *)
  IF Buffer2=NIL THEN Cle:=15; RETURN FALSE END;
  ad1:=Buffer2 + BufSize2 - 20;
  PicStart := LONGINT(y0) * LONGINT(BpR) + LONGINT(x0);

  FOR i:=0 TO ci.Depth-1 DO
    dummy:=Seek(fp,ci.Planes[i],-1);
    ad2:=ad1-ci.Length[i];
    ad3:=bp^.planes[i]+PicStart;
    dummy:=Read(fp,ad2,ci.Length[i]);
    IF dummy # ci.Length[i] THEN
      (* $Z- *) FreeMem(Buffer2,BufSize2); (* $Z+ *)
      Buffer2:=NIL; Cle:=16; RETURN FALSE;
    END;
    IF Decrunch(ad2,Buffer2,PicSize) # dummy THEN
      (* $Z- *) FreeMem(Buffer2,BufSize2); (* $Z+ *)
      Buffer2:=NIL; Cle:=17; RETURN FALSE;
    END;
    StringToRect(Buffer2,ad3,ci.PicX,ci.PicY,BpR);
  END;

  IF mode # 0 THEN
    IF ci.Depth<depth THEN
      IF mode=2 THEN num:=255 ELSE num:=0 END;
      FOR i:=ci.Depth TO depth-1 DO
        ad3:=bp^.planes[i]+PicStart;
        SetRect(ad3,ci.PicX,ci.PicY,BpR,num);
      END;
    END;
  END;

  (* $Z- *) FreeMem(Buffer2,BufSize2); (* $Z+ *)
  Buffer2:=NIL;RETURN TRUE;
END LoadBitplanes;


PROCEDURE LoadCPicScreen(Name : ARRAY OF CHAR;
                         VAR sp : ScreenPtr;
                         x0,y0 : INTEGER; Front : BOOLEAN) : BOOLEAN;
VAR bp : BitMapPtr;

BEGIN
  (* $Z- *)
  IF NOT ReadInfo(Name,ci,fp2) THEN RETURN FALSE END;
  IF NOT OpenCPicScreen(ci,sp,bp,x0,y0,NIL,Front) THEN
    Close(fp2); fp2:=NIL; RETURN FALSE;
  END;
  IF NOT LoadBitplanes(ci,fp2,bp,ci.PicX0,ci.PicY0,0) THEN
    Close(fp2); fp2:=NIL; CloseScreen(sp); sp:=NIL; RETURN FALSE;
  END;
  Close(fp2); fp2:=NIL; RETURN TRUE;
  (* $Z+ *)
END LoadCPicScreen;


PROCEDURE LoadBitplanesMem(VAR ci : CPicInfo;
                           Pic,Buffer : ADDRESS; bp : BitMapPtr;
                           x0,y0,mode : INTEGER);
VAR i,depth,BpR : INTEGER;
    PicSize,PicStart : LONGINT;
    ad2,ad3 : ADDRESS;
    num : UByte;

BEGIN
  CopyMem(Pic,ADR(ci),RecLen);
  BpR   := bp^.bytesPerRow;
  depth := bp^.depth;

  PicSize := LONGINT(ci.PicX) * LONGINT(ci.PicY);
  PicStart := LONGINT(y0) * LONGINT(BpR) + LONGINT(x0);

  FOR i:=0 TO ci.Depth-1 DO
    ad2:=Pic+ci.Planes[i];
    ad3:=bp^.planes[i]+PicStart;
    SETREG(0,Decrunch(ad2,Buffer,PicSize));
    StringToRect(Buffer,ad3,ci.PicX,ci.PicY,BpR);
  END;

  IF mode # 0 THEN
    IF ci.Depth<depth THEN
      IF mode=2 THEN num:=255 ELSE num:=0 END;
      FOR i:=ci.Depth TO depth-1 DO
        ad3:=bp^.planes[i]+PicStart;
        SetRect(ad3,ci.PicX,ci.PicY,BpR,num);
      END;
    END;
  END;

END LoadBitplanesMem;


PROCEDURE LoadColors(VAR ci : CPicInfo;
                     vp : ViewPortPtr; bp : BitMapPtr;
                     mode : INTEGER);

VAR col : ARRAY[0..31] OF INTEGER;
    num : ARRAY[1..5] OF INTEGER;
    depth,n : INTEGER;

BEGIN
  IF mode=0 THEN
    LoadRGB4(vp,ADR(ci.Colors),32);
  ELSE
    depth :=bp^.depth; IF depth>5 THEN depth:=5 END;
    IF ci.Depth>=depth THEN
      LoadRGB4(vp,ADR(ci.Colors),32);
    ELSE
      num[1]:=2;num[2]:=4;num[3]:=8;num[4]:=16;num[5]:=32;
      IF mode=1 THEN
        LoadRGB4(vp,ADR(ci.Colors),num[ci.Depth]);
      ELSE
        CopyMem(vp^.colorMap^.colorTable,ADR(col),64);
        n:=num[depth]-num[ci.Depth];
        CopyMem(ADR(ci.Colors),ADR(col[n]),num[ci.Depth]*2);
        LoadRGB4(vp,ADR(col),32);
      END;
    END;
  END;
END LoadColors;


BEGIN

TermProcedure(CleanUp);

END CPicSupport.
