(*---------------------------------------------------------------------------
  :Program.       PrinterSupport.mod
  :Contents.      Hilfsroutinen fr Druckerprogrammierung
  :Author.        !!! Original von Fritz Cizmarov & AMIGA DOS (Ausgabe 7'90)
  :Author.        Thomas Clever
  :Address.       Brhausen 9, D-5883 Kierspe 3
  :Phone.         02269/7084
  :ShortCut.      [tcl]
  :Version.       1.1
  :Date.          08. Oktober 1990
  :Copyright.     PD
  :Language.      MODULA-II
  :Translator.    M2Amiga V3.3
  :History.       V1.0 AmigaDOS 7'90
  :Update.        V1.1 [tcl] 01.Okt.90 (Erweiterungen, nderungen,
  :Update.             Optimierungen, bessere Fehlerbehandlung)
  :Remark.        Ich hoffe, da ich nicht gegen ein Copyright verstoe!
---------------------------------------------------------------------------*)

(* $R- *)
IMPLEMENTATION MODULE PrinterSupport;

FROM Arts        IMPORT Requester, TermProcedure, Error;
FROM Conversions IMPORT ValToStr;
FROM Exec        IMPORT Byte, UByte, write, IOStdReq, MsgPortPtr,
                        IOStdReqPtr, CloseDevice, DoIO, OpenDevice;
FROM ExecSupport IMPORT CreatePort, DeletePort, CreateExtIO, DeleteExtIO;
FROM Graphics    IMPORT ColorMapPtr, RastPortPtr, ViewModeSet;
FROM Intuition   IMPORT WindowPtr, ScreenPtr;
FROM Parallel    IMPORT pSel, paperOut, pBusy;
FROM Printer     IMPORT printerName, rawWrite, prtCommand, dumpRPort, query,
                        IOPrinterPtr, Special, SpecialSet, IODRPReq,
                        IODRPReqPtr;
FROM Serial      IMPORT busy, select;
FROM Str         IMPORT Length;
FROM SYSTEM      IMPORT ADDRESS, BYTE, LONGSET, ADR;
IMPORT Parallel, Serial;

TYPE
  prtStatus = RECORD
    CASE :CARDINAL OF
       1 : parStatus : Parallel.StatusSet
      |2 : serStatus : Serial.StatusSet
    END;
  END;

VAR
  ErrorProc : PROC;
  prtPtr : IOStdReqPtr;
  prtPort: MsgPortPtr;


PROCEDURE PrinterReady(): BOOLEAN;
VAR
  status : prtStatus;
  erg    : BOOLEAN;
BEGIN
  IF prtPtr # NIL THEN
    WITH prtPtr^ DO
      command := query;
      length  := 0;
      data    := ADR(status);
    END;
    DoIO(prtPtr);
    WITH status DO
      CASE prtPtr^.actual OF
         1 : erg := NOT (pBusy IN parStatus) AND (pSel IN parStatus)
        |2 : erg := NOT (busy IN serStatus) AND (select IN serStatus)
      END;
    END;
  ELSE
    erg := FALSE;
  END;
  RETURN erg
END PrinterReady;


PROCEDURE OpenPrinter(): BOOLEAN;
VAR
  erg : BOOLEAN;

  PROCEDURE PrinterRequest( body: ADDRESS; retry: BOOLEAN ): BOOLEAN;
  VAR
    adr : ADDRESS;
  BEGIN
    IF retry THEN
      adr := ADR(" Retry ");
    ELSE
      adr := NIL;
    END;
    RETURN Requester( ADR(" PrinterSupport:"), body, adr, ADR(" Cancel ") );
  END PrinterRequest;

BEGIN
  prtPort := CreatePort( NIL, 0 );
  IF prtPort # NIL THEN
    prtPtr := CreateExtIO( prtPort, SIZE(IODRPReq) );
    IF prtPtr # NIL THEN
      REPEAT
        OpenDevice( ADR(printerName), 0, prtPtr, LONGSET{0} );
        (* In prtPtr^.error steht, ob es geklappt hat oder nicht! *)
      UNTIL (prtPtr^.error = 0) OR
            (NOT PrinterRequest(ADR(" Can't open PrinterDevice!"),TRUE));
      IF prtPtr^.error = 0 THEN
        REPEAT
          erg := PrinterReady();
        UNTIL erg OR NOT PrinterRequest( ADR(" Printer NOT ready"), TRUE );
        IF NOT erg THEN
          CloseDevice(prtPtr);
        END;
      END;
    ELSE
      erg := PrinterRequest( ADR(" Can't create IORequest!"), FALSE );
    END;
  ELSE
    erg := PrinterRequest( ADR(" Can't create Port!"), FALSE );
  END;
  IF NOT erg THEN
    IF prtPtr # NIL THEN DeleteExtIO(prtPtr) END;
    IF prtPort # NIL THEN DeletePort(prtPort) END;
    prtPtr := NIL;
    prtPort := NIL;
  END;
  RETURN erg
END OpenPrinter;


PROCEDURE ClosePrinter;
BEGIN
  IF prtPtr # NIL THEN
    IF prtPtr^.device # NIL THEN
      CloseDevice(prtPtr);
    END;
    DeleteExtIO(prtPtr);
    prtPtr := NIL;
  END;
  IF prtPort # NIL THEN
    DeletePort(prtPort);
    prtPort := NIL;
  END;
END ClosePrinter;


PROCEDURE PrinterError(): Byte;
BEGIN
  IF prtPtr # NIL THEN
    RETURN prtPtr^.error
  ELSE
    RETURN openFail
  END
END PrinterError;


PROCEDURE CheckError;
BEGIN
  IF (ErrorProc # NIL) AND (PrinterError() # noErr) THEN
    ErrorProc;  (* benutzereigene Fehlerprozedur *)
  END;
END CheckError;


PROCEDURE DoPrtCommand( com: CARDINAL; adr: ADDRESS; len: LONGCARD );
BEGIN
  IF prtPtr # NIL THEN
    WITH prtPtr^ DO
      command := com;
      length  := len;
      data    := adr;
    END;
    DoIO(prtPtr);
  END;
  CheckError;
END DoPrtCommand;


PROCEDURE PWrite( ch: CHAR );
BEGIN
  DoPrtCommand( write, ADR(ch), 1 );
END PWrite;


PROCEDURE PWriteString( str: ARRAY OF CHAR );
BEGIN
  DoPrtCommand( write, ADR(str), Length(str) );
END PWriteString;


PROCEDURE PWriteLn;
VAR
  ch : CHAR;
BEGIN
  ch := 12C;
  DoPrtCommand( write, ADR(ch), 1 );
END PWriteLn;


PROCEDURE PWriteCard( val: LONGCARD; width: INTEGER );
VAR
  str : ARRAY [0..31] OF CHAR;
  err : BOOLEAN;
BEGIN
  ValToStr( val, FALSE, str, 10, width, " ", err );
  PWriteString( str );
END PWriteCard;


PROCEDURE PWriteInt( val: LONGINT; width: INTEGER );
VAR
  str : ARRAY [0..31] OF CHAR;
  err : BOOLEAN;
BEGIN
  ValToStr( val, TRUE, str, 10, width, " ", err );
  PWriteString( str );
END PWriteInt;


PROCEDURE PRawWrite( str: ARRAY OF BYTE; len: INTEGER );
BEGIN
  DoPrtCommand( rawWrite, ADR(str), len );
END PRawWrite;


PROCEDURE PCommand( com: CARDINAL; p0, p1, p2, p3: UByte );
VAR
  comPtr : IOPrinterPtr;
BEGIN
  IF prtPtr # NIL THEN
    comPtr := ADDRESS(prtPtr);
    comPtr^.command := prtCommand;
    comPtr^.prtCommand := com;
    comPtr^.parm0 := p0;
    comPtr^.parm1 := p1;
    comPtr^.parm2 := p2;
    comPtr^.parm3 := p3;
    DoIO( comPtr );
  END;
  CheckError;
END PCommand;


PROCEDURE DumpRPort( rPort: RastPortPtr; cMap: ColorMapPtr;
                     mod: ViewModeSet; x, y, w, h: CARDINAL;
                     destW, destH: LONGINT; spec: SpecialSet );
VAR
  ioDRPReqPtr : IODRPReqPtr;
BEGIN
  IF prtPtr # NIL THEN
    ioDRPReqPtr := ADDRESS(prtPtr);
    WITH ioDRPReqPtr^ DO
      command := dumpRPort;
      rastPort := rPort;
      colorMap := cMap;
      modesHi  := 0;
      modes    := mod;
      srcX     := x;
      srcY     := y;
      srcWidth := w;
      srcHeight:= h;
      destCols := destW;
      destRows := destH;
      special  := spec;
    END;
    DoIO(prtPtr);
  END;
  CheckError;
END DumpRPort;


PROCEDURE DumpScreen( screen: ScreenPtr );
BEGIN
  WITH screen^ DO
    DumpRPort( ADR(rastPort), viewPort.colorMap, viewPort.modes, 0,
               0, width, height, 0, 0, SpecialSet{fullCols,aspect} );
  END;
END DumpScreen;


PROCEDURE DumpWindow( window: WindowPtr );
BEGIN
  WITH window^ DO
    WITH window^.wScreen^.viewPort DO
      DumpRPort( rPort, colorMap, modes, 0, 0, width, height,
                 0, 0, SpecialSet{fullCols,aspect} )
    END
  END
END DumpWindow;


(* ---------- F e h l e r b e h a n d l u n g ----------- *)

PROCEDURE InstallErrorProc( p: PROC );
BEGIN
  ErrorProc := p;
END InstallErrorProc;


PROCEDURE RemoveErrorProc;
BEGIN
  ErrorProc := NIL;
END RemoveErrorProc;


PROCEDURE BreakOnError;
VAR
  b : BOOLEAN;
BEGIN
  b := Requester( ADR(" Modula-2 error message:"), ADR(" Printer-trouble"),
                  NIL, ADR(" Abort ") );
END BreakOnError;


BEGIN  (* main *)
  prtPtr  := NIL;
  prtPort := NIL;
  TermProcedure( ClosePrinter );
  ErrorProc := NIL;
END PrinterSupport.mod

