(*-------------------------------------------------------------------------
:Program.       AudioDemo.mod
:Contents.      Demo for `AudioSupport'
:Author.        Christian Stiens
:Address.       Heustiege 2, D-4710 Ldinghausen
:Phone.         02591/6299
:Copyright.     PD
:Language.      Modula-2
:Translator.    M2Amiga V3.3d,  AMSoft
:Imports.       AudioSupport V1.0, IntuiSupport V1.1
-------------------------------------------------------------------------*)

MODULE AudioDemo;

  FROM Arts         IMPORT TermProcedure,Assert;
  FROM Exec         IMPORT Byte;
  FROM Heap         IMPORT AllocMem;
  FROM SYSTEM       IMPORT ADR,ADDRESS,CAST,BITSET;
  FROM AudioSupport IMPORT PlaySound,Channel,ChannelSet,OpenChannel,Filter;
  FROM IntuiSupport IMPORT CreateWindow,stdWin,GetIMes;
  FROM Intuition    IMPORT WindowPtr,CloseWindow,IDCMPFlags,IDCMPFlagSet,
                           SetWindowTitles,IntuiMessage;

  CONST
    bufLen = 20000;

  VAR
    win       : WindowPtr;
    mes       : IntuiMessage;
    buffer    : POINTER TO ARRAY [0..bufLen-1] OF Byte;
    key       : ARRAY [0..127] OF Byte;
    per       : ARRAY [0..35] OF INTEGER;
    period    : INTEGER;
    chan      : Channel;


  PROCEDURE SetKeys;
    VAR i:INTEGER;
  BEGIN
    FOR i := 0 TO 127 DO
      key[i] := -1
    END;
    key[49] := 0; key[33] := 1; key[50] := 2; key[34] := 3; key[51] := 4;
    key[52] := 5; key[36] := 6; key[53] := 7; key[37] := 8; key[54] := 9;
    key[38] := 10;key[55] := 11;key[56] := 12;key[40] := 13;key[57] := 14;
    key[41] := 15;key[58] := 16;key[97] := 17;key[43] := 18;key[66] := 12;
    key[1]  := 13;key[16] := 14;key[2]  := 15;key[17] := 16;key[18] := 17;
    key[4]  := 18;key[19] := 19;key[5]  := 20;key[20] := 21;key[6]  := 22;
    key[21] := 23;key[22] := 24;key[8]  := 25;key[23] := 26;key[9]  := 27;
    key[24] := 28;key[25] := 29;key[11] := 30;key[26] := 31;key[12] := 32;
    key[27] := 33;key[13] := 34;key[68] := 35;
  END SetKeys;

  PROCEDURE SetPer;
    VAR i:INTEGER;
        p:REAL;
  BEGIN
    p:=124.0;
    FOR i := 35 TO 0 BY -1 DO
      per[i] := INTEGER(p+0.5);
      p := p * 1.059463094;
    END;
  END SetPer;

  PROCEDURE SetWave;
    CONST div = bufLen DIV 125;
    VAR i: INTEGER;
  BEGIN
    FOR i := 0 TO bufLen-1 DO
      buffer^[i] := (bufLen-i) DIV div;
      IF 3 IN CAST(BITSET,i) THEN buffer^[i] := -buffer^[i] END;
    END;
  END SetWave;

  PROCEDURE Cleanup;
  BEGIN
    IF win # NIL THEN CloseWindow(win) END;
    Filter(TRUE);
  END Cleanup;

BEGIN
  win := NIL;
  TermProcedure(Cleanup);

  chan := OpenChannel(ChannelSet{left0});   (* Open all channels *)
  chan := OpenChannel(ChannelSet{right0});
  chan := OpenChannel(ChannelSet{right1});
  chan := OpenChannel(ChannelSet{left1});

  AllocMem(buffer,bufLen,TRUE);
  Assert(buffer # NIL,ADR("Not enough memory"));

  win := CreateWindow(NIL,50,30,300,100,NIL,stdWin,
                      IDCMPFlagSet{closeWindow,rawKey});
  Assert(win#NIL,ADR("Can't open window"));

  SetWindowTitles(win,ADR("Calculating Sound"),ADR("Audio-Demo"));
  SetKeys; SetPer; SetWave;
  SetWindowTitles(win,ADR("Ready"),ADR("Audio-Demo"));

  Filter(FALSE);

  LOOP
    GetIMes(win,mes,TRUE);

    IF closeWindow IN mes.class THEN EXIT END;

    IF rawKey IN mes.class THEN
      IF mes.code<128 THEN                (* Key Down *)
        IF key[mes.code] >= 0 THEN
          period := per[key[mes.code]];
          PlaySound(chan,buffer,bufLen,period,64,1);
          chan := VAL(Channel,((ORD(chan)+1) MOD 4));
        END
      END
    END
  END
END AudioDemo.
