(*-------------------------------------------------------------------------
:Program.       GhostMan.mod
:Contents.      The good old PAC-MAN game
:Author.        Christian Stiens
:Address.       Heustiege 2, D-4710 Ldinghausen
:Phone.         02591/6299
:Copyright.     PD
:Language.      Modula-2
:Translator.    M2Amiga V3.3d,  AMSoft
:History.       V1.0, 24-Jul-90
:Imports.       GhostManData, Sprites, Sound, Joystick, Memory
-------------------------------------------------------------------------*)

MODULE GhostMan;

  FROM Graphics     IMPORT SetAPen,SetBPen,Move,Draw,WritePixel,RectFill,
                           SetRGB4,SetDrMd,jam2,ViewModes,ViewModeSet,SetRast,
                           RastPortPtr,ViewPortPtr,WaitTOF,Text,TextAttr,
                           TextAttrPtr,normalFont,FontFlags,FontFlagSet;
  FROM Intuition    IMPORT ScreenPtr,CloseScreen,OpenScreen,customScreen,
                           ImagePtr,DrawImage,Image,stdScreenHeight,NewScreen;
  FROM Exec         IMPORT CopyMem;
  FROM RandomNumber IMPORT RND,PutSeed;
  FROM Joystick     IMPORT StickX,StickY,Fire;
  FROM Sprites      IMPORT CreateSprite,ClearSprite,PutSprite,SwitchSprite,
                           SetSpriteColors;
  FROM Sound        IMPORT PlaySound;
  FROM Memory       IMPORT ToChipMem;
  FROM Conversions  IMPORT ValToStr;
  FROM Arts         IMPORT TermProcedure,Assert,Terminate;
  FROM SYSTEM       IMPORT INLINE,ADR,ADDRESS,SHIFT,BITSET;
  FROM GhostManData IMPORT Ghost1,Ghost2,Ghost3,Pac,PacImage,Laugh,Yuppie,
                           Fruit1,Fruit2,Fruit3,Logo,Null,Eyes,Bell,
                           bellSize,fruitSize,laughSize,yuppieSize;

  CONST
    yOffs      = 20;
    esc        = 165C;
    P          = 315C;
    lives      = 4;
    bonus      = 50000;
    fruitTime  = 400;
    fruitDelay = 1000;
    fruitScore = 5000;
    catchScore = 500;
    flashTime  = 100;
    blinkRate  = 12;
    strongTime = 1000;
    strongSub  = 100;
    numPts     = 236;

  TYPE
    Direction   = (l,r,o,u,h);
    GhostStatus = (alive,dead);
    PacStatus   = (weak,strong,hit);
    Field       = ARRAY [0..21],[0..31] OF CHAR;

  VAR
    scr: ScreenPtr;
    rp:  RastPortPtr;
    vp:  ViewPortPtr;
    topaz80: TextAttr;
    parcour: ARRAY [0..1] OF Field;
    wayback: Field;
    dirs,goodDirs: ARRAY [0..3] OF Direction;
    numDirs,numGoodDirs: CARDINAL;
    dfc: ARRAY [0..1] OF INTEGER;
    px,py,pn: INTEGER;
    pd: Direction;
    ps: PacStatus;
    gx,gy: ARRAY [0..3] OF INTEGER;
    gd: ARRAY [0..3] OF Direction;
    gs: ARRAY [0..3] OF GhostStatus;
    fruit,pacIm: Image;
    fruitDat,pacImDat: ADDRESS;
    live: ARRAY [0..1] OF CARDINAL;
    n,nr,bp,fp: CARDINAL;
    t,ts,tf: CARDINAL;
    plr,numPlr: CARDINAL;
    pts:  ARRAY [0..1] OF CARDINAL;
    bon: ARRAY [0..1] OF LONGINT;
    hsc: LONGINT;
    sc: ARRAY [0..1] OF LONGINT;
    scOld: LONGINT;
    bell,yuppie,laugh: ADDRESS;
    key[0BFEC01H]: CHAR;

  PROCEDURE CreateScreen(wid,hei,dep:INTEGER; mode:ViewModeSet;
                                              fontPtr:TextAttrPtr): ScreenPtr;
    VAR nScr: NewScreen;
  BEGIN
    WITH nScr DO
      leftEdge:=0; topEdge:=0; width:=wid; height:=hei;
      depth:=dep; detailPen:=0; blockPen:=1; defaultTitle:=NIL;
      viewModes:=mode; type:=customScreen; font:=fontPtr;
      gadgets:=NIL; customBitMap:=NIL;
    END;
    RETURN OpenScreen(nScr);
  END CreateScreen;

  PROCEDURE SetParcour(plr: CARDINAL);
  BEGIN
    pts[plr] := 0;
    parcour[plr][00] := "HAAAAAAAAAAAAAAOPAAAAAAAAAAAAAAG";
    parcour[plr][01] := "B..............FF..............B";
    parcour[plr][02] := "B.KEEEM.KEEEEM.FF.KEEEEM.KEEEM.B";
    parcour[plr][03] := "B*LEEEN.LEEEEN.LN.LEEEEN.LEEEN*B";
    parcour[plr][04] := "B..............................B";
    parcour[plr][05] := "B.DAAAC.KM.DAAAOPAAAC.KM.DAAAC.B";
    parcour[plr][06] := "B.......FF.....FF.....FF.......B";
    parcour[plr][07] := "IAAAAAG.FUAAAC LN DAAAVF.HAAAAAJ";
    parcour[plr][08] := "      B.FF            FF.B      ";
    parcour[plr][09] := "  AAAAJ.LN KEEEYYEEEM LN.IAAAA  ";
    parcour[plr][10] := "  Z    .   F        F   .    Z  ";
    parcour[plr][11] := "  AAAAG.KM LEEEEEEEEN KM.HAAAA  ";
    parcour[plr][12] := "      B.FF            FF.B      ";
    parcour[plr][13] := "HAAAAAJ.LN DAAAOPAAAC LN.IAAAAAG";
    parcour[plr][14] := "B..............FF..............B";
    parcour[plr][15] := "B.DAAOW.DAAAAC.LN.DAAAAC.XPAAC.B";
    parcour[plr][16] := "B*...FF........  ........FF...*B";
    parcour[plr][17] := "QAAC.LN.KM.DAAAOPAAAC.KM.LN.DAAR";
    parcour[plr][18] := "B.......FF.....FF.....FF.......B";
    parcour[plr][19] := "B.DAAAAASTAAAC.LN.DAAASTAAAAAC.B";
    parcour[plr][20] := "B..............................B";
    parcour[plr][21] := "IAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAJ";
    wayback[00] := "################################";
    wayback[01] := "#>>>>>>v<<<>>>v##v<<<>>>v<<<<<<#";
    wayback[02] := "#v#####v######v##v######v#####v#";
    wayback[03] := "#v#####v######v##v######v#####v#";
    wayback[04] := "#>>>>>>>>>v<<<<<>>>>>v<<<<<<<<<#";
    wayback[05] := "#^#####^##v##########v##^#####^#";
    wayback[06] := "#>>>>>>^##>>>>v##v<<<<##^<<<<<<#";
    wayback[07] := "#######^######v##v######^#######";
    wayback[08] := "      #v##>>>>>vv<<<<<##v#      ";
    wayback[09] := " ######v##^####vv####^##v###### ";
    wayback[10] := " >>>>>>>>>^#==<<>>==#^<<<<<<<<< ";
    wayback[11] := " ######^##^##########^##^###### ";
    wayback[12] := "      #^##^<<<<<>>>>>^##^#      ";
    wayback[13] := "#######^##^##########^##^#######";
    wayback[14] := "#>>>>>>^>>^<<<<##>>>>^<<^<<<<<<#";
    wayback[15] := "#^#####^######^##^######^#####^#";
    wayback[16] := "#^<<<##^<<>>>>^<>^<<<<>>^##>>>^#";
    wayback[17] := "####^##^##^##########^##^##^####";
    wayback[18] := "#>>>>>>^##^<<<<##>>>>^##^<<<<<<#";
    wayback[19] := "#^############^##^############^#";
    wayback[20] := "#^<<<<<>>>>>>>^<>^<<<<>>>>>>>>^#";
    wayback[21] := "################################";
  END SetParcour;

  PROCEDURE PrintNumber(n: LONGINT; x,y: INTEGER);
    VAR err: BOOLEAN;
        str: ARRAY [0..9] OF CHAR;
  BEGIN
    ValToStr(n,FALSE,str,10,9,' ',err);
    IF n=0 THEN str[7]:='0' END;
    Move(rp,x,y);
    Text(rp,ADR(str),9);
  END PrintNumber;

  PROCEDURE ShowScores;
  BEGIN
    IF sc[plr] # scOld THEN
      SetAPen(rp,1);
      PrintNumber(sc[plr],plr*27*8+11,17);
      PrintNumber(hsc,107,17);
      scOld := sc[plr];
    END;
  END ShowScores;

  PROCEDURE DrawParcour;
    VAR z,s: INTEGER;

    PROCEDURE B(x1,y1,x2,y2: INTEGER);
    BEGIN
      RectFill(rp,s*10+x1,yOffs+z*10+y1,s*10+x2,yOffs+z*10+y2)
    END B;

  BEGIN
    FOR z:=0 TO 21 DO
      FOR s:=0 TO 31 DO
        SetAPen(rp,2);
        CASE parcour[plr][z,s] OF
        | ' ': SetAPen(rp,0);B(3,3,6,6)
        | '.': SetAPen(rp,1);B(4,4,5,5)
        | '*': SetAPen(rp,3);B(3,4,6,5);B(4,3,5,6);
        | 'A': B(0,2,9,3);B(0,6,9,7)
        | 'B': B(2,0,3,9);B(6,0,7,9)
        | 'C': B(0,2,4,3);B(0,6,4,7);B(4,3,5,6)
        | 'D': B(5,2,9,3);B(5,6,9,7);B(4,3,5,6)
        | 'E': B(0,4,9,5)
        | 'F': B(4,0,5,9)
        | 'G': B(0,2,6,3);B(6,3,7,9);B(0,6,2,7);B(2,7,3,9)
        | 'H': B(3,2,9,3);B(2,3,3,9);B(7,6,9,7);B(6,7,7,9)
        | 'I': B(2,0,3,6);B(3,6,9,7);B(6,0,7,2);B(7,2,9,3)
        | 'J': B(2,0,3,2);B(0,2,2,3);B(6,0,7,6);B(0,6,6,7)
        | 'K': B(5,4,9,5);B(4,5,5,9);B(6,6,6,6)
        | 'L': B(4,0,5,4);B(5,4,9,5);B(6,3,6,3)
        | 'M': B(0,4,4,5);B(4,5,5,9);B(3,6,3,6)
        | 'N': B(0,4,4,5);B(4,0,5,4);B(3,3,3,3)
        | 'O': B(0,2,9,3);B(0,6,4,7);B(4,7,5,9)
        | 'P': B(0,2,9,3);B(5,6,9,7);B(4,7,5,9)
        | 'Q': B(2,0,3,9);B(6,0,7,2);B(7,2,9,3);B(7,6,9,7);B(6,7,7,9)
        | 'R': B(6,0,7,9);B(0,2,2,3);B(2,0,3,2);B(0,6,2,7);B(2,7,3,9)
        | 'S': B(0,6,9,7);B(0,2,4,3);B(4,0,5,2)
        | 'T': B(0,6,9,7);B(4,0,5,2);B(5,2,9,3)
        | 'U': B(4,0,5,2);B(5,2,9,3);B(4,7,5,9);B(5,6,9,7)
        | 'V': B(4,0,5,2);B(0,2,4,3);B(0,6,4,7);B(4,7,5,9)
        | 'W': B(4,3,5,9);B(0,2,4,3)
        | 'X': B(4,3,5,9);B(5,2,9,3)
        | 'Y': SetAPen(rp,1);B(0,4,9,4)
        | 'Z':
        END
      END
    END;
    SetAPen(rp,1); Move(rp,123,7); Text(rp,ADR("HIGH SCORE"),10);
    SetAPen(rp,3-plr*2); Move(rp,43,7);  Text(rp,ADR("1UP"),3);
    SetAPen(rp,1+plr*2); Move(rp,259,7); Text(rp,ADR("2UP"),3);
    SetAPen(rp,1);
    PrintNumber(sc[0],11,17); PrintNumber(sc[1],227,17);
    PrintNumber(hsc,107,17);
  END DrawParcour;

  PROCEDURE SetColors;
  BEGIN
    SetRGB4(vp,0,0,0,0);  SetRGB4(vp,1,12,12,12);
    SetRGB4(vp,2,0,4,14); SetRGB4(vp,3,12,12,12);
    SetRGB4(vp,4,0,9,0);  SetRGB4(vp,5,2,12,1);
    SetRGB4(vp,6,4,15,1); SetRGB4(vp,7,8,8,8);
    SetRGB4(vp,8,11,11,0);SetRGB4(vp,9,15,15,0);
    SetRGB4(vp,10,4,2,0); SetRGB4(vp,11,7,4,0);
    SetRGB4(vp,12,9,5,0); SetRGB4(vp,13,15,9,0);
    SetRGB4(vp,14,14,5,0);SetRGB4(vp,15,12,0,0);
  END SetColors;

  PROCEDURE SetBlack;
    VAR i: INTEGER;
  BEGIN
    FOR i := 0 TO 15 DO
      SetRGB4(vp,i,0,0,0)
    END;
  END SetBlack;

  PROCEDURE SetOff;
  BEGIN
    SwitchSprite(1,ADR(Null));
    SwitchSprite(2,ADR(Null));
    SwitchSprite(3,ADR(Null));
    SwitchSprite(4,ADR(Null));
    SwitchSprite(5,ADR(Null));
  END SetOff;

  PROCEDURE SetOn;
  BEGIN
    SwitchSprite(1,ADR(Pac)+240); SetSpriteColors(vp,1,0FC0H,0FFFH,0000H);
    SwitchSprite(2,ADR(Ghost1));
    SwitchSprite(3,ADR(Ghost2)); SetSpriteColors(vp,2,0D00H,00D0H,0FFFH);
    SwitchSprite(4,ADR(Ghost1));
    SwitchSprite(5,ADR(Ghost2)); SetSpriteColors(vp,4,007FH,0F0AH,0FFFH);
  END SetOn;

  PROCEDURE SetBlue;
  BEGIN
    IF gs[0]=alive THEN SwitchSprite(2,ADR(Ghost3)) END;
    IF gs[1]=alive THEN SwitchSprite(3,ADR(Ghost3)) END;
    SetSpriteColors(vp,2,0000H,000FH,0FFFH);
    IF gs[2]=alive THEN SwitchSprite(4,ADR(Ghost3)) END;
    IF gs[3]=alive THEN SwitchSprite(5,ADR(Ghost3)) END;
    SetSpriteColors(vp,4,0000H,000FH,0FFFH);
    fp:=15;
  END SetBlue;

  PROCEDURE SetNormal;
  BEGIN
    IF gs[0]=alive THEN SwitchSprite(2,ADR(Ghost1)) END;
    IF gs[1]=alive THEN SwitchSprite(3,ADR(Ghost2)) END;
    SetSpriteColors(vp,2,0D00H,00D0H,0FFFH);
    IF gs[2]=alive THEN SwitchSprite(4,ADR(Ghost1)) END;
    IF gs[3]=alive THEN SwitchSprite(5,ADR(Ghost2)) END;
    SetSpriteColors(vp,4,007FH,0F0AH,0FFFH);
  END SetNormal;

  PROCEDURE Reset;
  BEGIN
    ts:=0; t:=0;
    px:=151; py:=157; pd:=h; ps:=weak;
    FOR n:=0 TO 3 DO
      gx[n]:=120+n*21; gy[n]:=96; gd[n]:=r; gs[n]:=alive;
      PutSprite(vp,n+2,gx[n],gy[n]+yOffs);
    END;
    PutSprite(vp,1,px,py+yOffs);
  END Reset;

  PROCEDURE Blink;
  BEGIN
    IF bp=12 THEN bp:=0 ELSE bp:=12 END;
    SetRGB4(vp,3,bp,bp,bp)
  END Blink;

  PROCEDURE Flash;
  BEGIN
    IF fp=15 THEN fp:=8 ELSE fp:=15 END;
    SetRGB4(vp,22,0,0,fp);
    SetRGB4(vp,26,0,0,fp);
  END Flash;

  PROCEDURE Tick;
  BEGIN
    WaitTOF;
    IF key=esc THEN Terminate(0) END;
    IF key=P THEN WHILE NOT(Fire(2)) DO WaitTOF END END;
    INC(t); IF t=60000 THEN t:=0 END;
    IF t MOD blinkRate = 0 THEN Blink END;
  END Tick;

  PROCEDURE Wait;
  BEGIN
    WHILE  Fire(2) DO Tick END;
    WHILE ~Fire(2) DO Tick END;
  END Wait;

  PROCEDURE Pause(ticks: CARDINAL);
    VAR t: CARDINAL;
  BEGIN
    FOR t:=1 TO ticks DO
      Tick
    END
  END Pause;

  PROCEDURE SetFruit;
  BEGIN
    CASE RND(3) OF
    | 0: CopyMem(ADR(Fruit1),fruitDat,fruitSize)
    | 1: CopyMem(ADR(Fruit2),fruitDat,fruitSize)
    | 2: CopyMem(ADR(Fruit3),fruitDat,fruitSize)
    END;
    parcour[plr][12,15]:='f';
    DrawImage(rp,ADR(fruit),148,116+yOffs);
    tf:=fruitTime;
  END SetFruit;

  PROCEDURE FruitOff;
  BEGIN
    parcour[plr][12,15]:=' ';
    SetAPen(rp,0);
    RectFill(rp,148,116+yOffs,148+15,116+15+yOffs);
    tf:=0;
  END FruitOff;

  PROCEDURE Startup;
    VAR vhposr[0DFF006H]: CARDINAL;
  BEGIN
    WITH topaz80 DO
      name:=ADR("topaz.font");
      ySize:=8;
      style:=normalFont;
      flags:=FontFlagSet{romFont};
    END;
    scr:=CreateScreen(320,stdScreenHeight,4,ViewModeSet{sprites},ADR(topaz80));
    Assert(scr#NIL,ADR("Can't open screen"));
    Assert(scr^.height>=256,ADR("Can't open PAL screen"));
    vp:=ADR(scr^.viewPort); rp:=ADR(scr^.rastPort);
    SetDrMd(rp,jam2); SetBPen(rp,0);
    bell  := ToChipMem(ADR(Bell),bellSize);
    laugh := ToChipMem(ADR(Laugh),laughSize);
    yuppie:= ToChipMem(ADR(Yuppie),yuppieSize);
    Assert((bell#NIL) & (laugh#NIL) & (yuppie#NIL),ADR("Not enough memory"));
    fruitDat:=ToChipMem(ADR(Fruit1),fruitSize);
    pacImDat:=ToChipMem(ADR(PacImage),18);
    Assert((pacImDat#NIL) & (fruitDat#NIL),ADR("Not enough memory"));
    WITH fruit DO
      leftEdge:=0;
      topEdge:=0;
      width:=16;
      height:=16;
      depth:=4;
      imageData:=fruitDat;
      planePick:=15;
      planeOnOff:=0;
      nextImage:=NIL;
    END;
    WITH pacIm DO
      leftEdge:=0;
      topEdge:=0;
      width:=9;
      height:=9;
      depth:=1;
      imageData:=pacImDat;
      planePick:=1;
      planeOnOff:=0;
      nextImage:=NIL;
    END;
    Assert(CreateSprite(1,ADR(Null),15) &
           CreateSprite(2,ADR(Null),17) &
           CreateSprite(3,ADR(Null),17) &
           CreateSprite(4,ADR(Null),17) &
           CreateSprite(5,ADR(Null),17),  ADR("Can't create sprite"));
    PutSeed(vhposr+1);
    hsc:=10000;
  END Startup;

  PROCEDURE Init;
    VAR i: INTEGER;
  BEGIN
    FOR i := 0 TO 1 DO
      dfc[i] := 2;
      live[i] := lives;
      sc[i] := 0;
      bon[i] := bonus;
      SetParcour(i);
    END;
    scOld := 0;
    plr := 0;
    pn := 0;
    Reset;
  END Init;

  PROCEDURE Cleanup;
  BEGIN
    FOR n:=1 TO 5 DO
      ClearSprite(n)
    END;
    IF scr#NIL THEN CloseScreen(scr) END;
  END Cleanup;

  PROCEDURE GetDxDy(d: Direction; VAR dx,dy: INTEGER);
  BEGIN
    CASE d OF
    | l: dx :=-1; dy := 0
    | r: dx := 1; dy := 0
    | o: dx := 0; dy :=-1
    | u: dx := 0; dy := 1
    ELSE dx := 0; dy := 0
    END
  END GetDxDy;

  PROCEDURE BackDir(d1,d2: Direction): BOOLEAN;
  BEGIN
    CASE d1 OF
    | l: RETURN d2 = r
    | r: RETURN d2 = l
    | o: RETURN d2 = u
    | u: RETURN d2 = o
    END;
  END BackDir;

  PROCEDURE In(c,c1,c2: CHAR): BOOLEAN;
  BEGIN
    RETURN (c>=c1) & (c<=c2)
  END In;

  PROCEDURE Sgn(i:INTEGER): INTEGER;
  BEGIN
    IF i>0    THEN RETURN 1
    ELSIF i<0 THEN RETURN -1
    ELSE RETURN 2 END
  END Sgn;

  PROCEDURE AddDir(d: Direction);
  BEGIN
    dirs[numDirs]:=d;
    INC(numDirs)
  END AddDir;

  PROCEDURE AddGoodDir(d: Direction);
  BEGIN
    goodDirs[numGoodDirs]:=d;
    INC(numGoodDirs)
  END AddGoodDir;

  PROCEDURE Legal(d: Direction; z,s: INTEGER): BOOLEAN;
  BEGIN
    IF (s=15) OR (s=16) THEN
      IF (z=8)&(d=u)&(RND(10)<6)  THEN RETURN FALSE END;
      IF (z=9) & ((d=l) OR (d=r)) THEN RETURN FALSE END;
      IF (z=10)&(ps=strong)&(d=o) THEN RETURN FALSE END;
    END;
    RETURN TRUE
  END Legal;

  PROCEDURE NewDir(z,s:INTEGER; oldDir:Direction; wall:BOOLEAN): Direction;
    VAR sgz,sgs: INTEGER;
        zp,sp: INTEGER;
        dz,ds: INTEGER;
        d: Direction;
  BEGIN
    zp:=py DIV 10 + 1; sp:=px DIV 10 + 1;
    sgz:=Sgn(zp-z); sgs:=Sgn(sp-s);
    numDirs:=0; numGoodDirs:=0;
    FOR d:=l TO u DO
      GetDxDy(d,ds,dz);
      IF NOT In(parcour[plr][z+dz,s+ds],'A','X') THEN
        IF NOT((NOT wall) & BackDir(d,oldDir)) THEN
          IF Legal(d,z,s) THEN
            AddDir(d);
            IF ps=weak THEN
              IF (ds=sgs) OR (dz=sgz) THEN AddGoodDir(d) END;
            ELSE
              IF (ds#sgs) AND(dz#sgz) THEN AddGoodDir(d) END;
            END
          END
        END
      END
    END;
    IF (numGoodDirs>0) & (RND(10)<dfc[plr]) THEN
      RETURN goodDirs[RND(numGoodDirs)]
    END;
    RETURN dirs[RND(numDirs)]
  END NewDir;

  PROCEDURE MoveGhost(n: CARDINAL);
    VAR z,s: INTEGER;
        dx,dy: INTEGER;
        x,y: INTEGER;
        rx,ry: INTEGER;
        d,dn: Direction;
  BEGIN
    IF (ps=strong) & ODD(t) & (gs[n]=alive) THEN RETURN END;
    d:=gd[n];
    GetDxDy(d,dx,dy);
    x:=gx[n]+dx; y:=gy[n]+dy;
    z:=y DIV 10 + 1; s:=x DIV 10 + 1;
    rx:=x MOD 10; ry:=y MOD 10;
    IF (rx=8) & (ry=6) THEN
      IF parcour[plr][z,s]='Z' THEN
        x:=306-x;
      ELSE
        CASE gs[n] OF
        | alive:
          IF In(parcour[plr][z+dy,s+dx],'A','X') THEN
            d:=NewDir(z,s,d,TRUE);
          ELSE
            d:=NewDir(z,s,d,FALSE);
          END
        | dead:
          CASE wayback[z,s] OF
          | '<': d:=l
          | '>': d:=r
          | '^': d:=o
          | 'v': d:=u
          | '=': gs[n]:=alive;
                 IF ps#strong THEN
                   CASE ODD(n) OF
                   | FALSE: SwitchSprite(n+2,ADR(Ghost1))
                   | TRUE:  SwitchSprite(n+2,ADR(Ghost2))
                   END
                 ELSE
                   SwitchSprite(n+2,ADR(Ghost3))
                 END
          END
        END
      END
    END;
    IF (ABS(x-px-1)<8) & (ABS(y-py+1)<8) & (gs[n]=alive) THEN
      CASE ps OF
      | weak:   ps:=hit; PlaySound(laugh,laughSize,400,64,1);
      | strong: INC(nr);INC(sc[plr],catchScore*nr); gs[n]:=dead;
                PlaySound(yuppie,yuppieSize,380,64,1); Pause(30);
                SwitchSprite(n+2,ADR(Eyes));
      ELSE
      END
    END;
    PutSprite(vp,n+2,x,y+yOffs);
    gd[n]:=d; gx[n]:=x; gy[n]:=y;
  END MoveGhost;

  PROCEDURE AnimPac;
    VAR offs: INTEGER;
  BEGIN
    IF t MOD 4 = 0 THEN
      INC(pn); IF pn=4 THEN pn:=0 END;
      CASE pn OF
      | 0: offs:=0
      | 1: offs:=60
      | 2: offs:=120
      | 3: offs:=60
      END;
      IF pd<=u THEN SwitchSprite(1,ADR(Pac)+180*ORD(pd)+offs) END
    END
  END AnimPac;

  PROCEDURE MovePac;
    VAR z,s: INTEGER;
        x,y,dx,dy,rx,ry,sx,sy: INTEGER;
        d,dn: Direction;
  BEGIN
    d:=pd;
    GetDxDy(d,dx,dy);
    x:=px+dx; y:=py+dy;
    z:=y DIV 10 + 1; s:=x DIV 10 + 1;
    rx:=x MOD 10; ry:=y MOD 10;
    sx:=StickX(2); sy:=StickY(2);
    dn:=h;
    IF sx#0 THEN IF sx=-1 THEN dn:=l ELSE dn:=r END END;
    IF sy#0 THEN IF sy=-1 THEN dn:=o ELSE dn:=u END END;
    IF (rx=6) & (ry=7) THEN
      CASE parcour[plr][z,s] OF
      | '.':
        INC(sc[plr],10); INC(pts[plr]);
        SetAPen(rp,0); RectFill(rp,x+8,y+7+yOffs,x+9,y+8+yOffs);
        parcour[plr][z,s]:=' ';
        PlaySound(bell,bellSize,150,30,1);
      | '*':
        INC(sc[plr],50); INC(pts[plr]); nr:=0;
        SetAPen(rp,0); RectFill(rp,x+7,y+6+yOffs,x+10,y+9+yOffs);
        parcour[plr][z,s]:=' ';
        ts:=strongTime-dfc[plr]*strongSub;
        ps:=strong; SetBlue;
        PlaySound(bell,bellSize,200,40,1);
      | 'f':
        PlaySound(yuppie,yuppieSize,380,64,1);
        INC(sc[plr],fruitScore);
        FruitOff;
      ELSE
      END;
      IF parcour[plr][z,s]='Z' THEN
        x:=302-x;
      ELSE
        IF dn<=u THEN
          GetDxDy(dn,dx,dy);
          IF NOT In(parcour[plr][z+dy,s+dx],'A','Y') THEN
            d:=dn
          END;
        END;
        GetDxDy(d,dx,dy);
        IF In(parcour[plr][z+dy,s+dx],'A','Y') THEN
          d:=h
        END
      END
    ELSE
      IF dn<=u THEN
        CASE d OF
        | l..u: IF BackDir(dn,d) THEN d:=dn END
        | h   : IF (dn=l) OR (dn=r) THEN d:=dn END
        END
      END
    END;
    AnimPac;
    PutSprite(vp,1,x,y+yOffs);
    pd:=d; px:=x; py:=y;
  END MovePac;

  PROCEDURE ShowLives;
    VAR n:CARDINAL;
  BEGIN
    IF live[plr]>1 THEN
      FOR n:=0 TO live[plr]-2 DO
        DrawImage(rp,ADR(pacIm),n*11+3,242)
      END;
    END;
    SetAPen(rp,0);
    FOR n:=live[plr]-1 TO 10 DO
      RectFill(rp,n*11+3,242,n*11+13,252);
    END;
  END ShowLives;

  PROCEDURE RotatePac;
    VAR i: CARDINAL;
        d: Direction;
  BEGIN
    FOR i:=1 TO 16 DO
      CASE i MOD 4 OF
      | 0: d:=r
      | 1: d:=u
      | 2: d:=l
      | 3: d:=o
      END;
      SwitchSprite(1,ADR(Pac)+ORD(d)*180+120);
      Pause(4)
    END;
  END RotatePac;

  PROCEDURE Frame(x,y,a,b: INTEGER);
  BEGIN
    Move(rp,x,y);
    Draw(rp,x+a,y);
    Draw(rp,x+a,y+b);
    Draw(rp,x,y+b);
    Draw(rp,x,y);
  END Frame;

  PROCEDURE Titel;
    VAR logoPtr: POINTER TO CARDINAL;
        wert: CARDINAL;
        i,j,k,x,y: INTEGER;
        w: CARDINAL;
        s,f: CARDINAL;
        r,g,b: CARDINAL;
        col: ARRAY [0..7],[0..2] OF CARDINAL;
        pause: CARDINAL;
        xx,xxold: CARDINAL;
        flag: BOOLEAN;
  BEGIN
    SetBlack; SetRast(rp,0);
    FOR i := 0 TO 7 DO
      col[i,0] := 0;
      col[i,1] := 0;
      col[i,2] := 0;
    END;
    y:=90;
    logoPtr:=ADR(Logo);
    FOR j:=0 TO 7 DO
      x:=10;
      FOR i:=0 TO 3 DO
        wert:=logoPtr^;
        FOR k:=0 TO 15 DO
          IF wert>=32768 THEN
            SetAPen(rp,8 + j);
            RectFill(rp,x,y+1,x+3,y+2); RectFill(rp,x+1,y,x+2,y+3);
          END;
          (* $R- *) wert:=SHIFT(wert,1); (* $R= *)
          INC(x,5);
        END;
        INC(logoPtr,2);
      END;
      INC(y,5);
    END;
    SetRGB4(vp,1,15,15,15); SetAPen(rp,1);
    Move(rp,88,20);   Text(rp,ADR("   A PD GAME BY   "),18);
    Move(rp,88,35);   Text(rp,ADR(" CHRISTIAN STIENS "),18);
    Move(rp,84,50);   Text(rp,ADR("WRITTEN IN MODULA-2"),19);
    Move(rp,28,190);  Text(rp,ADR("1 PLAYER"),8);
    Move(rp,125,190); Text(rp,ADR("2 PLAYERS"),9);
    Move(rp,233,190); Text(rp,ADR("BYE BYE"),7);
    xx := 0; xxold := 1;
    flag := TRUE;
    w:=0;
    WHILE NOT Fire(2) DO
      IF key=esc THEN Terminate(0) END;
      IF xx#xxold THEN
        SetAPen(rp,0); Frame(xxold*100+20,168,82,40);
        SetAPen(rp,1); Frame(   xx*100+20,168,82,40);
      END;
      xxold := xx;
      CASE StickX(2) OF
      | +1: IF (xx<2) & flag THEN INC(xx); flag:=FALSE END
      | -1: IF (xx>0) & flag THEN DEC(xx); flag:=FALSE END
      ELSE
        flag:=TRUE;
      END;
      FOR i := 7 TO 1 BY -1 DO
        col[i,0] := col[i-1,0];
        col[i,1] := col[i-1,1];
        col[i,2] := col[i-1,2];
      END;
      s := w DIV 16; f := w MOD 16;
      CASE s OF
      | 0: r := 15;   g := f;    b := 0
      | 1: r := 15-f; g := 15;   b := 0
      | 2: r := 0;    g := 15;   b := f
      | 3: r := 0;    g := 15-f; b := 15
      | 4: r := f;    g := 0;    b := 15
      | 5: r := 15;   g := 0;    b := 15-f
      END;
      col[0,0] := r; col[0,1] := g; col[0,2] := b;
      w := (w+2) MOD 96;
      FOR i:=0 TO 7 DO
        SetRGB4(vp, 8 + i, col[i,0],col[i,1],col[i,2])
      END;
      FOR pause:=1 TO 4 DO WaitTOF END;
    END;
    CASE xx OF
    | 0: numPlr := 1
    | 1: numPlr := 2
    | 2: Terminate(0)
    END;
    SetBlack; SetRast(rp,0);
  END Titel;

  PROCEDURE NewPoints;
  BEGIN
    IF dfc[plr]<8 THEN INC(dfc[plr],2) END;
    Pause(50); SetOff; FruitOff; Reset; SetParcour(plr);
    DrawParcour; SetOn; Wait
  END NewPoints;

  PROCEDURE Message;
  BEGIN
    Move(rp,130,88+yOffs);
    CASE plr OF
      0: Text(rp,ADR("PLAYER 1"),8) |
      1: Text(rp,ADR("PLAYER 2"),8)
    END;
    Move(rp,124,108+yOffs); Text(rp,ADR("GAME OVER"),9);
  END Message;

  PROCEDURE NextLive;
  BEGIN
    RotatePac; Pause(50); SetOff; FruitOff;
    DEC(live[plr]);
    IF (live[plr]=0) THEN
      IF (numPlr=1) OR ( (numPlr=2) & (live[1-plr]=0) ) THEN
        SetAPen(rp,1); Move(rp,124,108+yOffs);
        Text(rp,ADR("GAME OVER"),9);
        Wait; Titel; Init; DrawParcour; ShowLives; SetColors; SetOn; Wait;
        RETURN
      ELSE
        IF numPlr=2 THEN
          SetAPen(rp,1); Message;
          Wait;
          SetAPen(rp,0); Message;
        END;
      END;
    END;
    CASE numPlr OF
    |1: Reset; ShowLives; SetOn; Wait
    |2: Reset;
        IF live[1-plr]>0 THEN plr := 1-plr; DrawParcour END;
        ShowLives; SetOn; Wait
    END;
  END NextLive;

BEGIN
  scr:=NIL;
  TermProcedure(Cleanup);
  Startup;
  Titel;
  Init;
  DrawParcour; ShowLives;
  SetColors;
  SetOn; Wait;
  LOOP
    Tick;
    FOR n:=0 TO 3 DO
      MoveGhost(n);
    END;
    MovePac;
    IF sc[plr]>hsc THEN hsc:=sc[plr] END;
    IF sc[plr]>=bon[plr] THEN INC(bon[plr],bonus); INC(live[plr]); ShowLives END;
    ShowScores;
    IF ts#0 THEN
      DEC(ts);
      IF (ts<flashTime) & (t MOD 6 = 3) THEN Flash END;
      IF ts=0 THEN SetNormal; ps:=weak END;
    END;
    IF tf#0 THEN
      DEC(tf);
      IF tf=0 THEN FruitOff END;
    END;
    IF t MOD fruitDelay = 0 THEN SetFruit END;
    IF ps=hit THEN NextLive END;
    IF pts[plr]=numPts THEN NewPoints END;
  END
END GhostMan.
