(*-------------------------------------------------------------------------
:Program.       Sprites.mod
:Contents.      Prozeduren zur einfachen Programmierung der Hardware-Sprites
:Author.        Christian Stiens
:Address.       Heustiege 2, D-4710 Ldinghausen
:Phone.         02591/6299
:Copyright.     PD
:Language.      Modula-2
:Translator.    M2Amiga V3.3d,  AMSoft
:History.       V1.0, 17-Jul-90
-------------------------------------------------------------------------*)

IMPLEMENTATION MODULE Sprites;

  FROM Graphics IMPORT GetSprite,FreeSprite,SetRGB4,MoveSprite,
                       SimpleSprite,SimpleSpritePtr,ViewPortPtr;
  FROM Exec     IMPORT CopyMem,AllocMem,FreeMem,MemReqs,MemReqSet;
  FROM SYSTEM   IMPORT ADR,ADDRESS;

  VAR
    dat:    ARRAY [0..7] OF ADDRESS;
    size:   ARRAY [0..7] OF CARDINAL;
    sprite: ARRAY [0..7] OF SimpleSprite;
    free:   ARRAY [0..7] OF BOOLEAN;

  PROCEDURE CreateSprite(num: CARDINAL; data: ADDRESS;
                                        height: CARDINAL): BOOLEAN;
    VAR spritePtr: SimpleSpritePtr;
  BEGIN
    IF NOT free[num] THEN RETURN FALSE END;
    size[num] := (height*2 + 4) * 2;
    dat[num] := AllocMem(size[num],MemReqSet{chip,memClear});
    IF dat[num] = NIL THEN RETURN FALSE END;
    CopyMem(data,dat[num]+4,size[num]-8);
    sprite[num].posctldata := dat[num];
    sprite[num].height := height;
    spritePtr := ADR(sprite[num]);
    IF GetSprite(spritePtr,num) < 0 THEN
      FreeMem(dat[num],size[num]);
      RETURN FALSE
    END;
    free[num] := FALSE;
    RETURN TRUE
  END CreateSprite;

  PROCEDURE ClearSprite(num: CARDINAL);
  BEGIN
    IF (NOT free[num]) & (num#0) THEN
      FreeSprite(num);
      FreeMem(dat[num],size[num]);
      free[num] := TRUE
    END
  END ClearSprite;

  PROCEDURE PutSprite(vp: ViewPortPtr; num: CARDINAL; x,y: INTEGER);
  BEGIN
    MoveSprite(vp,ADR(sprite[num]),x,y)
  END PutSprite;

  PROCEDURE SwitchSprite(num: CARDINAL; newData:ADDRESS);
  BEGIN
    IF NOT free[num] THEN
      CopyMem(newData,dat[num]+4,size[num]-8)
    END
  END SwitchSprite;

  PROCEDURE SetSpriteColors(vp: ViewPortPtr; num: CARDINAL;
                            col1,col2,col3: CARDINAL);
    PROCEDURE R(c: CARDINAL): CARDINAL; BEGIN RETURN (c DIV 256) MOD 16 END R;
    PROCEDURE G(c: CARDINAL): CARDINAL; BEGIN RETURN (c DIV  16) MOD 16 END G;
    PROCEDURE B(c: CARDINAL): CARDINAL; BEGIN RETURN (c DIV   1) MOD 16 END B;
    VAR nr: CARDINAL;
  BEGIN
    nr := 16+(num DIV 2)*4;
    SetRGB4(vp,nr+1,R(col1),G(col1),B(col1));
    SetRGB4(vp,nr+2,R(col2),G(col2),B(col2));
    SetRGB4(vp,nr+3,R(col3),G(col3),B(col3));
  END SetSpriteColors;

  VAR nr: CARDINAL;

BEGIN
  free[0] := FALSE;
  FOR nr := 1 TO 7 DO free[nr] := TRUE END
END Sprites.
