(*-------------------------------------------------------------------------
:Program.       IntuiSupport.mod
:Contents.      Easy programming of Intuition
:Author.        Christian Stiens
:Address.       Heustiege 2, D-4710 Ldinghausen
:Phone.         02591/6299
:Copyright.     PD
:Language.      Modula-2
:Translator.    M2Amiga V3.3d,  AMSoft
:History.       V1.0, 18-Jul-90: First release
:History        V1.1, 26-Aug-90: Added new procs, bugs fixed, cosmetics
-------------------------------------------------------------------------*)

IMPLEMENTATION MODULE IntuiSupport; (* $R-$V-$S-$F- *)

  FROM Intuition IMPORT
    NewScreen,NewWindow,customScreen,IntuiMessagePtr,WindowFlags,ScreenPtr,
    WindowFlagSet,ScreenFlags,ScreenFlagSet,IDCMPFlags,IDCMPFlagSet,
    WindowPtr,StringInfo,Gadget,GadgetFlags,GadgetFlagSet,IntuiText,
    IntuiTextPtr,MenuItem,AddGadget,OnGadget,RemoveGadget,boolGadget,
    strGadget,Border,Image,maxPot,ActivationFlags,ActivationFlagSet,PropInfo,
    propGadget,PropInfoPtr,maxBody,PropInfoFlags,PropInfoFlagSet,ImagePtr,
    MenuPtr,MenuItemPtr,Menu,OpenWindow,MenuItemFlagSet,OpenScreen,BorderPtr,
    menuEnabled,SetMenuStrip,GadgetPtr,MenuItemFlags,StringInfoPtr,reqGadget,
    Requester,RequesterPtr,IntuiMessage,RequesterFlagSet,InitRequester;
  FROM Graphics  IMPORT
    ViewModes,ViewModeSet,DrawModeSet,jam1,RastPortPtr,TextAttr,TextAttrPtr,
    FontFlags,FontFlagSet,normalFont;
  FROM Exec IMPORT
    GetMsg,ReplyMsg,WaitPort,TypeOfMem,MemReqs,CopyMem;
  FROM Heap IMPORT
    Allocate,AllocMem;
  FROM SYSTEM IMPORT
    ADR,ADDRESS,LONGSET,WORD,BITSET,CAST;
  FROM Arts IMPORT
    Assert;

  VAR
    topaz80attr: TextAttr;
    firstMenu,lastMenu: MenuPtr;
    lastItem, lastSub : MenuItemPtr;


  PROCEDURE CreateScreen(title:ADDRESS; wi,he,dep:INTEGER;
                         mode:ViewModeSet): ScreenPtr;
    VAR nScr: NewScreen;
  BEGIN
    WITH nScr DO
      leftEdge:=0;
      topEdge:=0;
      width:=wi;
      height:=he;
      depth:=dep;
      detailPen:=0;
      blockPen:=1;
      defaultTitle:=title;
      viewModes:=mode;
      type:=customScreen;
      font:=topaz80;
      gadgets:=NIL;
      customBitMap:=NIL;
    END;
    RETURN OpenScreen(nScr);
  END CreateScreen;

  PROCEDURE CreateWindow(name:ADDRESS; le,to,wi,he:INTEGER; scr:ScreenPtr;
                         winFlags: WindowFlagSet;
                         idcmp:    IDCMPFlagSet): WindowPtr;
    VAR nWdw: NewWindow;
  BEGIN
    WITH nWdw DO
      leftEdge:=le;
      topEdge:=to;
      width:=wi;
      height:=he;
      detailPen:=0;
      blockPen:=1;
      idcmpFlags:=idcmp;
      flags:=winFlags;
      firstGadget:=NIL;
      checkMark:=NIL;
      title:=name;
      screen:=scr;
      bitMap:=NIL;
      minWidth :=90;
      minHeight:=40;
      maxWidth :=-1;
      maxHeight:=-1;
      type:=customScreen;
    END;
    IF scr=NIL THEN
      nWdw.type:=ScreenFlagSet{wbenchScreen}
    END;
    RETURN OpenWindow(nWdw);
  END CreateWindow;

  PROCEDURE GetIMes (win:WindowPtr; VAR iMes:IntuiMessage; wait:BOOLEAN);
    VAR msg: IntuiMessagePtr;
  BEGIN
    IF wait THEN WaitPort(win^.userPort) END;
    msg:=GetMsg(win^.userPort);
    IF msg # NIL THEN
      iMes := msg^;
      ReplyMsg(msg)
    ELSE
      iMes.class := IDCMPFlagSet{}
    END;
  END GetIMes;

  PROCEDURE ALLOCATE(VAR adr:ADDRESS; size:LONGINT);
  BEGIN
    Allocate(adr,size);
    Assert(adr#NIL,ADR("Out of memory"));
  END ALLOCATE;

  PROCEDURE CreateAutoBorder(le,to,wi,he:INTEGER): BorderPtr;
    VAR
      bord: BorderPtr;
      dat: POINTER TO ARRAY [0..8] OF RECORD x,y:INTEGER END;
  BEGIN
    ALLOCATE(bord,SIZE(Border) + SIZE(INTEGER)*2*9);
    dat := CAST(ADDRESS,bord);
    INC(dat,SIZE(Border));
    dat^[0].x:=0;   dat^[0].y:=0;
    dat^[1].x:=wi;  dat^[1].y:=0;
    dat^[2].x:=wi;  dat^[2].y:=he;
    dat^[3].x:=0;   dat^[3].y:=he;
    dat^[4].x:=0;   dat^[4].y:=0;
    dat^[5].x:=1;   dat^[5].y:=0;
    dat^[6].x:=1;   dat^[6].y:=he;
    dat^[7].x:=wi-1;dat^[7].y:=he;
    dat^[8].x:=wi-1;dat^[8].y:=0;
    WITH bord^ DO
      leftEdge:=le;
      topEdge:=to;
      frontPen:=1;
      drawMode:=jam1;
      count:=9;
      xy:=dat;
    END;
    RETURN bord
  END CreateAutoBorder;

  PROCEDURE CreateBorder(le,to: INTEGER; pen:CARDINAL; dm:DrawModeSet;
                         num:CARDINAL; dat:ADDRESS;
                         next:BorderPtr): BorderPtr;
    VAR bord: BorderPtr;
  BEGIN
    ALLOCATE(bord,SIZE(Border));
    WITH bord^ DO
      leftEdge := le;
      topEdge := to;
      frontPen := pen;
      drawMode := dm;
      count := num;
      xy := dat;
      nextBorder := next;
    END;
    RETURN bord;
  END CreateBorder;

  PROCEDURE CreateIntuiText(fp,bp:CARDINAL; dm:DrawModeSet;le,to:INTEGER;
                            font:TextAttrPtr; text:ADDRESS;
                            next:IntuiTextPtr): IntuiTextPtr;
    VAR itex: IntuiTextPtr;
  BEGIN
    ALLOCATE(itex,SIZE(IntuiText));
    WITH itex^ DO
      frontPen:=fp; backPen:=bp;
      drawMode:=dm;
      leftEdge:=le; topEdge:=to;
      iTextFont:=font; iText:=text;
      nextText:=next;
    END;
    RETURN itex
  END CreateIntuiText;

  PROCEDURE CreateImage(le,to,wi,he,dep:INTEGER; dat:ADDRESS;
                        pick,onOff: BITSET): ImagePtr;
    VAR im: ImagePtr;
  BEGIN
    ALLOCATE(im,SIZE(Image));
    WITH im^ DO
      leftEdge := le; topEdge := to;
      width := wi; height := he;
      depth := dep;
      imageData := dat;
      planePick := CAST(CARDINAL,pick);
      planeOnOff := CAST(CARDINAL,onOff);
    END;
    RETURN im
  END CreateImage;

  PROCEDURE ToChipMem(adr:ADDRESS; size:LONGINT; check:BOOLEAN): ADDRESS;
    VAR newAdr: ADDRESS;
  BEGIN
    IF check & (chip IN TypeOfMem(adr)) THEN RETURN adr END;
    AllocMem(newAdr,size,TRUE);
    Assert(newAdr # NIL,ADR("Not enough chip-mem"));
    CopyMem(adr,newAdr,size);
    RETURN newAdr
  END ToChipMem;

  PROCEDURE IsIText(text: ADDRESS): BOOLEAN;
    VAR str: POINTER TO CHAR;
  BEGIN
    IF text=NIL THEN RETURN TRUE END;
    str := text;
    RETURN str^ < " "
  END IsIText;

  PROCEDURE CreateBoolGadget(id, le,to,wi,he: INTEGER; text:ADDRESS;
                            render:    ADDRESS;
                            gadgFlags: GadgetFlagSet;
                            actvFlags: ActivationFlagSet): GadgetPtr;
    VAR len: INTEGER;
        str:  POINTER TO CHAR;
        gad: GadgetPtr;
  BEGIN
    ALLOCATE(gad,SIZE(Gadget));
    WITH gad^ DO
      leftEdge:=le; topEdge:=to; width:= wi; height:=he;
      flags:=gadgFlags;
      activation:=actvFlags;
      IF render=autoBorder THEN
        gadgetRender:=CreateAutoBorder(-2,-1,wi+3,he+1)
      ELSE
        gadgetRender:=render
      END;
      selectRender:=NIL;
      IF IsIText(text) THEN
        gadgetText:=text
      ELSE
        str:=text; len:=0;
        WHILE str^ # 0C DO INC(str);INC(len) END;
        gadgetText:=CreateIntuiText(1,0,jam1,(wi-8*len)DIV 2,(he-8)DIV 2,topaz80,text,NIL)
      END;
      gadgetType:=boolGadget;
      gadgetID:=id;
    END;
    RETURN gad
  END CreateBoolGadget;

  PROCEDURE CreateStrGadget(id, le,to,wi,he, chars: INTEGER;
                            render:    ADDRESS;
                            gadgFlags: GadgetFlagSet;
                            actvFlags: ActivationFlagSet): GadgetPtr;
    VAR gad: GadgetPtr;
        strInf: StringInfoPtr;
  BEGIN
    ALLOCATE(gad,SIZE(Gadget) + SIZE(StringInfo) + 2*chars);
    strInf := CAST(ADDRESS,gad);
    INC(strInf,SIZE(Gadget));
    WITH strInf^ DO
      buffer := CAST(LONGINT,strInf)+SIZE(StringInfo);
      undoBuffer := buffer+chars;
      maxChars:=chars;
    END;
    WITH gad^ DO
      leftEdge:=le; topEdge:=to; width:=wi; height:=he;
      flags:=gadgFlags;
      activation:=actvFlags;
      IF render=autoBorder THEN
        gadgetRender:=CreateAutoBorder(-4,-2,wi+5,he+3)
      ELSE
        gadgetRender:=render
      END;
      gadgetType:=strGadget;
      specialInfo:=strInf;
      gadgetID:=id
    END;
    RETURN gad
  END CreateStrGadget;

  PROCEDURE GadgetText(gadg:GadgetPtr): ADDRESS;
    VAR strI: POINTER TO StringInfo;
  BEGIN
    strI:=gadg^.specialInfo;
    RETURN strI^.buffer;
  END GadgetText;

  PROCEDURE GadgetVal(gadg:GadgetPtr): LONGINT;
    VAR strI: POINTER TO StringInfo;
  BEGIN
    strI:=gadg^.specialInfo;
    RETURN strI^.longInt;
  END GadgetVal;

  PROCEDURE CreatePropGadget(id, le,to,wi,he: INTEGER;
                             horizSteps,vertSteps: CARDINAL;
                             knop: ImagePtr;
                             gadgFlags: GadgetFlagSet;
                             actvFlags: ActivationFlagSet;
                             propInfo:  PropInfoFlagSet): GadgetPtr;
    VAR pi: PropInfoPtr;
        autoknob: ImagePtr;
        gad: GadgetPtr;
  BEGIN
    ALLOCATE(gad,SIZE(Gadget) + SIZE(PropInfo));
    pi := CAST(ADDRESS,gad);
    INC(pi,SIZE(Gadget));
    WITH pi^ DO
     flags:=propInfo;
     IF horizSteps#0 THEN horizBody:=maxBody DIV horizSteps END;
     IF  vertSteps#0 THEN  vertBody:=maxBody DIV  vertSteps END;
    END;
    WITH gad^ DO
      leftEdge:=le; topEdge:=to; width:=wi; height:=he;
      flags:=gadgFlags; activation:=actvFlags; gadgetType:=propGadget;
      IF knop#NIL THEN
        gadgetRender:=knop
      ELSE
        ALLOCATE(autoknob,SIZE(Image));
        gadgetRender:=autoknob;
      END;
      specialInfo:=pi; gadgetID:=id
    END;
    RETURN gad
  END CreatePropGadget;

  PROCEDURE HorizPot(gadg:GadgetPtr; steps:CARDINAL): CARDINAL;
    VAR pi:PropInfoPtr;
  BEGIN
    pi:=gadg^.specialInfo;
    RETURN (LONGCARD(pi^.horizPot)*LONGCARD(steps)) DIV (maxPot+1);
  END HorizPot;

  PROCEDURE VertPot (gadg:GadgetPtr; steps:CARDINAL): CARDINAL;
    VAR pi:PropInfoPtr;
  BEGIN
    pi:=gadg^.specialInfo;
    RETURN (LONGCARD(pi^.vertPot)*LONGCARD(steps)) DIV (maxPot+1);
  END VertPot;

  PROCEDURE Pot(val,steps: CARDINAL): CARDINAL;
  BEGIN
    RETURN LONGCARD(maxPot) * LONGCARD(val) DIV LONGCARD(steps-1);
  END Pot;

  PROCEDURE Body(steps:CARDINAL): CARDINAL;
  BEGIN
    RETURN maxBody DIV steps
  END Body;

  PROCEDURE CreateRequester(le,to,wi,he, relLe,relTo: INTEGER;
                            border:   BorderPtr;
                            reqTxt:   IntuiTextPtr;
                            reqFlags: RequesterFlagSet;
                            backPen:  CARDINAL): RequesterPtr;
    VAR req: RequesterPtr;
  BEGIN
    ALLOCATE(req,SIZE(Requester));
    InitRequester(req);
    WITH req^ DO
      leftEdge := le;
      topEdge := to;
      width := wi;
      height := he;
      relLeft := relLe;
      relTop := relTo;
      IF border=autoBorder THEN
        reqBorder := CreateAutoBorder(0,0,wi-1,he-1);
      ELSE
        reqBorder := border;
      END;
      reqText := reqTxt;
      flags := reqFlags;
      backFill := backPen;
    END;
    RETURN req
  END CreateRequester;

  PROCEDURE AddReqGadget(req:RequesterPtr; gadg:GadgetPtr);
  BEGIN
    gadg^.gadgetType :=
      CAST(CARDINAL,CAST(BITSET,gadg^.gadgetType) + CAST(BITSET,reqGadget));
    gadg^.nextGadget := req^.reqGadget;
    req^.reqGadget := gadg;
  END AddReqGadget;

  PROCEDURE DefMenu(name:ADDRESS; le,wi:INTEGER; enable:BOOLEAN);
    VAR menu: MenuPtr;
  BEGIN
    ALLOCATE(menu,SIZE(Menu));
    WITH menu^ DO
      leftEdge:=le; topEdge:=0; width:=wi; height:=10;
      IF enable THEN flags:={menuEnabled} ELSE flags:={} END;
      menuName:=name;
    END;
    IF lastMenu=NIL THEN firstMenu:=menu ELSE lastMenu^.nextMenu:=menu END;
    lastMenu:=menu; lastItem:=NIL;
  END DefMenu;

  PROCEDURE DefItem(fill:ADDRESS; le,to,wi,he:INTEGER; pen:CARDINAL;
                    mutEx:LONGSET; cmd: CHAR; iFlags:MenuItemFlagSet);
    VAR item:MenuItemPtr;
  BEGIN
    ALLOCATE(item,SIZE(MenuItem));
    WITH item^ DO
      leftEdge:=le; topEdge:=to; width:=wi; height:=he;
      flags:=iFlags;
      mutualExclude:=mutEx;
      IF (itemText IN iFlags) & NOT IsIText(fill) THEN
        itemFill:=CreateIntuiText(pen,0,jam1,0,1,topaz80,fill,NIL);
      ELSE
        itemFill:=fill
      END;
      selectFill:=NIL; command:=cmd;
    END;
    IF lastItem=NIL THEN
      lastMenu^.firstItem:=item
    ELSE
      lastItem^.nextItem:=item
    END;
    lastItem:=item; lastSub:=NIL;
  END DefItem;

  PROCEDURE DefSub(fill:ADDRESS;le,to,wi,he:INTEGER; pen:CARDINAL;
                   mutEx:LONGSET; cmd:CHAR; iFlags:MenuItemFlagSet);
    VAR item:MenuItemPtr;
  BEGIN
    ALLOCATE(item,SIZE(MenuItem));
    WITH item^ DO
      leftEdge:=le; topEdge:=to; width:=wi; height:=he;
      flags:=iFlags;
      mutualExclude:=mutEx;
      IF (itemText IN iFlags) & NOT IsIText(fill) THEN
        itemFill:=CreateIntuiText(pen,0,jam1,0,1,topaz80,fill,NIL);
      ELSE
        itemFill:=fill
      END;
      selectFill:=NIL; command:=cmd; subItem:=NIL;
    END;
    IF lastSub=NIL THEN
      lastItem^.subItem:=item
    ELSE
      lastSub^.nextItem:=item
    END;
    lastSub:=item;
  END DefSub;

  PROCEDURE InstallMenuStrip(win:WindowPtr): MenuPtr;
  BEGIN
    IF NOT SetMenuStrip(win,firstMenu) THEN RETURN NIL END;
    lastMenu:=NIL;
    RETURN firstMenu
  END InstallMenuStrip;

BEGIN
  lastMenu := NIL;
  WITH topaz80attr DO
    name  := ADR("topaz.font");
    ySize := 8;
    style := normalFont;
    flags := FontFlagSet{romFont}
  END;
  topaz80 := ADR(topaz80attr);
END IntuiSupport.mod
