(*-------------------------------------------------------------------------
:Program.       Planet.mod
:Contents.      Berechnung eines fraktalen Planeten
:Author.        Christian Stiens
:Address.       Heustiege 2, D-4710 Ldinghausen
:Phone.         02591/6299
:Copyright.     PD
:Language.      Modula-2
:Translator.    M2Amiga V3.3d,  AMSoft
:History.       V1.0, 26-Jul-90
:Imports.       IntuiSupport, IFFSupport [fbs]
-------------------------------------------------------------------------*)

MODULE Planet;

  FROM MathTrans IMPORT
    Sqrt,Pow;
  FROM IntuiSupport IMPORT
    CreateScreen,CreateBoolGadget,CreatePropGadget,GetIMes,VertPot,HorizPot,
    CreateWindow,CreateStrGadget,stdActv,stdGadg,ValToPot,GadgetText,stdWin;
  FROM Exec IMPORT
    WaitPort,CopyMem;
  FROM Graphics IMPORT
    RastPortPtr,WritePixel,SetAPen,RectFill,SetRGB4,ViewPortPtr,ViewModes,
    ViewModeSet,Text,Move,SetDrMd,jam1,SetRast;
  FROM Intuition IMPORT
    CloseScreen,ScreenPtr,stdScreenHeight,CloseWindow,WindowPtr,ScreenToFront,
    GadgetFlags,GadgetFlagSet,ActivationFlags,ActivationFlagSet,PropInfoFlags,
    PropInfoFlagSet,IDCMPFlags,IDCMPFlagSet,PropInfoPtr,WindowFlags,GadgetPtr,
    WindowFlagSet,StringInfoPtr,AddGadget,ActivateGadget,RefreshGList,
    RefreshGadgets,NewModifyProp,RemoveGadget,DisplayBeep;
  FROM SYSTEM IMPORT
    FFP,ADR,ADDRESS;
  FROM Arts IMPORT
    TermProcedure,Assert,Terminate;
  FROM IFFSupport IMPORT
    WriteILBMScreen;
  FROM RandomNumber IMPORT
    RND;
  FROM FFPConversions IMPORT
    StrToReal,RealToStr;

  CONST
    right  = ActivationFlagSet{stringRight};
    center = ActivationFlagSet{stringCenter};
    pf     = PropInfoFlagSet{freeVert,autoKnob};
    wf     = stdWin - WindowFlagSet{windowSizing};
    if     = IDCMPFlagSet{closeWindow,gadgetUp};

    aspectRatio = 0.5;

  VAR
    a0,b0,rd: FFP;
    lx,ly,lz: FFP;
    fa,fd,fs,e: FFP;
    R,G,B: ARRAY [1..3] OF [0..15];
    win: WindowPtr;
    rpw: RastPortPtr;
    scr: ScreenPtr;
    rps: RastPortPtr;
    vp:  ViewPortPtr;
    gad: ARRAY [1..19] OF GadgetPtr;
    gadStop: GadgetPtr;
    choosedGad: GadgetPtr;
    class: IDCMPFlagSet;
    code: CARDINAL;
    adr: ADDRESS;
    attr: [1..3];

  PROCEDURE CleanUp;
  BEGIN
    IF scr#NIL THEN CloseScreen(scr) END;
    IF win#NIL THEN CloseWindow(win) END
  END CleanUp;

  PROCEDURE SetParameters;
  BEGIN
    R[1] := 0;  G[1] := 5; B[1] := 10;
    R[2] := 10; G[2] := 10;B[2] := 10;
    R[3] := 10; G[3] := 5; B[3] := 0;
    a0 := 0.0; b0 := 0.3;
    rd := 1.0;
    lx := 0.0; ly := 0.0; lz := -1.0;
    fa := 10.0; fd := 70.0; fs := 20.0;
    e  := 10.0;
  END SetParameters;

  PROCEDURE Normierung():BOOLEAN;
    VAR temp: FFP;
  BEGIN
    temp := Sqrt(lx*lx+ly*ly+lz*lz);
    IF temp=0.0 THEN RETURN FALSE END;
    lx:= lx/temp;ly:=ly/temp;lz:=lz/temp;
    temp := fa+fd+fs;
    IF temp=0.0 THEN RETURN FALSE END;
    fa:= fa/temp; fd := fd/temp; fs := fs/temp;
    RETURN TRUE
  END Normierung;

  PROCEDURE CalcColors;
    VAR n: CARDINAL;
  BEGIN
    IF scr#NIL THEN
      FOR n:=1 TO 10 DO
        SetRGB4(vp,n+1,  R[1]*n/10 , G[1]*n/10 , B[1]*n/10);
        SetRGB4(vp,n+11, R[2]*n/10 , G[2]*n/10 , B[2]*n/10);
        SetRGB4(vp,n+21, R[3]*n/10 , G[3]*n/10 , B[3]*n/10);
      END;
    END
  END CalcColors;

  PROCEDURE Iterate(a,b: FFP): INTEGER;
    VAR aOld,bOld: FFP;
        c,d,e,f,g,h,i: FFP;
  BEGIN
    REPEAT
      aOld:= a; bOld := b;
      c := a*a-b*b; d := a*b*2.0;
      e := a*c-b*d+a*0.25-1.25; f := a*d+b*c+b*0.25;
      g := c*3.0+0.25;  h:=d*3.0+0.25;
      i := g*g+h*h;   a := a-(e*g+f*h)/i;  b := b-(f*g-e*h)/i;
    UNTIL (ABS(a-aOld) < 0.01) & (ABS(b-bOld) < 0.01);
    IF b>+0.5 THEN
      RETURN 1
    ELSIF b<-0.5 THEN
      RETURN 2
    ELSE
      RETURN 0
    END
  END Iterate;

  PROCEDURE CalcPlanet;
    VAR
      r,d: INTEGER;
      r2,y2: FFP;
      ix,iy: INTEGER;
      x,y,z,a,b,xy: FFP;
      attr,h: INTEGER;
      f,cosTheta,cosAlpha: FFP;
  BEGIN
    IF scr=NIL THEN
      scr := CreateScreen(NIL,320,stdScreenHeight,5,ViewModeSet{lace});
      Assert(scr # NIL, ADR("Can't Open Screen"));
      rps := ADR(scr^.rastPort); vp := ADR(scr^.viewPort);
    ELSE
      ScreenToFront(scr)
    END;
    SetRGB4(vp,0,0,0,0);
    SetRGB4(vp,1,12,12,12);
    SetRast(rps,0);
    CalcColors;
    r:=(scr^.height-1) DIV 2;
    r2:=FFP(r)*FFP(r)*1.005;
    FOR iy:=-r TO r DO
      y2 := FFP(iy)*FFP(iy);
      d  := TRUNC(Sqrt(r2-y2)*aspectRatio);
      FOR ix:=-d TO d DO
        GetIMes(win,class,code,adr);
        IF gadgetUp IN class THEN RETURN END;
        IF closeWindow IN class THEN Terminate(0) END;
        x := FFP(ix)/(FFP(r)*aspectRatio); y := FFP(iy)/FFP(r);
        xy:=1.0-(x*x+y*y); IF xy<0.0 THEN xy:=0.0 END;
        z := -Sqrt(xy);
        a := a0 + rd * x/(1.0-z); b := b0 + rd * y/(1.0-z);
        attr := Iterate(a,b);
        cosTheta := x*lx+y*ly+z*lz;
        cosAlpha := -(2.0*cosTheta*z-lz);
        IF cosAlpha<0.0 THEN cosAlpha:=0.0 END;
        IF cosTheta<0.0 THEN
          f:=fa
        ELSE
          f:=fa+fd*cosTheta+fs*Pow(cosAlpha,e)
        END;
        IF f<0.0 THEN f:=0.0 ELSE f:=f*10.0 END;
        h := INTEGER(f);
        IF (FFP(RND(10000))/10000.0) < (f-FFP(h)) THEN INC(h) END;
        IF h>10 THEN h:=10 END;
        IF h=0 THEN SetAPen(rps,0) ELSE SetAPen(rps,1+attr*10+h) END;
        IF WritePixel(rps,160+ix,r-iy) THEN END;
      END
    END
  END CalcPlanet;

  PROCEDURE Print(rp:RastPortPtr; x,y:INTEGER; text:ARRAY OF CHAR);
  BEGIN
    Move(rp,x,y);
    Text(rp,ADR(text),HIGH(text)+1)
  END Print;

  PROCEDURE ReadStrGad(gad:GadgetPtr; VAR val:FFP);
    VAR err: BOOLEAN;
        strPtr: POINTER TO ARRAY [0..5] OF CHAR;
  BEGIN
    strPtr := GadgetText(gad);
    StrToReal(strPtr^,val,err);
    IF err THEN val := 0.0 END;
  END ReadStrGad;

  PROCEDURE WriteStrGad(gad:GadgetPtr; val:FFP);
    VAR err: BOOLEAN;
        strPtr: POINTER TO ARRAY [0..5] OF CHAR;
  BEGIN
    strPtr := GadgetText(gad);
    RealToStr(val,strPtr^,5,4,FALSE,err)
  END WriteStrGad;

  PROCEDURE DrawFrame(rp: RastPortPtr; x,y,a,b,dx,dy: INTEGER; pen: CARDINAL);
  BEGIN
    SetAPen(rp,pen);
    RectFill(rp,x,y-dy,x+a-1,y-1);
    RectFill(rp,x+a-dx,y,x+a-1,y+b-1);
    RectFill(rp,x-dx,y+b-dy,x+a-dx-1,y+b-1);
    RectFill(rp,x-dx,y-dy,x-1,y+b-dy-1);
  END DrawFrame;

  PROCEDURE WriteWindow;
  BEGIN
    Print(rpw,30,10,"Zentrum:"); Print(rpw,30,40,"Radius:");
    Print(rpw,55,22,"x"); Print(rpw,135,22,"y");
    Print(rpw,10,80,"Lichtvektor:");
    Print(rpw,25,95,"x"); Print(rpw,25,110,"y"); Print(rpw,25,125,"z");
    Print(rpw,150,80,"Reflektion:");
    Print(rpw,130,96,"umgeb."); Print(rpw,130,111,"gespieg.");
    Print(rpw,130,126,"diffus"); Print(rpw,160,141,"Spot");
    Print(rpw,258,96,"%"); Print(rpw,258,111,"%"); Print(rpw,258,126,"%");
    Print(rpw,310,10,"Frbung:"); Print(rpw,397,16,"R  G  B");
    Print(rpw,300,129,"Dateiname:");
    DrawFrame(rpw,298,20*attr-1,78,16,4,2,1);
  END WriteWindow;

  PROCEDURE Main;
    VAR strPtr: POINTER TO ARRAY [0..63] OF CHAR;
        propInfo: PropInfoPtr;
        dummy: BOOLEAN;
        i: CARDINAL;
  BEGIN
    win := CreateWindow(ADR("Planet"),0,15,500,170,NIL,wf,if);
    Assert(win # NIL, ADR("Can't open window"));
    rpw := win^.rPort;
    SetAPen(rpw,1); SetDrMd(rpw,jam1);
    SetParameters;
    gad[1]  := CreateStrGadget(1,70,17,50,8,6,stdGadg,stdActv+right);
    gad[2]  := CreateStrGadget(2,150,17,50,8,6,stdGadg,stdActv+right);
    gad[3]  := CreateStrGadget(3,70,47,50,8,6,stdGadg,stdActv+right);
    gad[4]  := CreateStrGadget(4,40,90,50,8,6,stdGadg,stdActv+right);
    gad[5]  := CreateStrGadget(5,40,105,50,8,6,stdGadg,stdActv+right);
    gad[6]  := CreateStrGadget(6,40,120,50,8,6,stdGadg,stdActv+right);
    gad[7]  := CreateStrGadget(7,200,90,50,8,6,stdGadg,stdActv+right);
    gad[8]  := CreateStrGadget(8,200,105,50,8,6,stdGadg,stdActv+right);
    gad[9]  := CreateStrGadget(9,200,120,50,8,6,stdGadg,stdActv+right);
    gad[10] := CreateStrGadget(10,200,135,50,8,6,stdGadg,stdActv+right);
    gad[11] := CreateBoolGadget(11,300,20,70,12,ADR("Attr. 1"),stdGadg,stdActv);
    gad[12] := CreateBoolGadget(12,300,40,70,12,ADR("Attr. 2"),stdGadg,stdActv);
    gad[13] := CreateBoolGadget(13,300,60,70,12,ADR("Attr. 3"),stdGadg,stdActv);
    gad[14] := CreatePropGadget(14,390,19,20,54,0,16,NIL,stdGadg,stdActv,pf);
    gad[15] := CreatePropGadget(15,414,19,20,54,0,16,NIL,stdGadg,stdActv,pf);
    gad[16] := CreatePropGadget(16,438,19,20,54,0,16,NIL,stdGadg,stdActv,pf);
    gad[17] := CreateBoolGadget(17,320,90,40,20,ADR("GO"),stdGadg,stdActv);
    gad[18] := CreateBoolGadget(18,400,90,40,20,ADR("Save"),stdGadg,stdActv);
    gad[19] := CreateStrGadget(19,280,135,200,8,64,stdGadg,stdActv+center);
    gadStop := CreateBoolGadget(20,220,60,40,20,ADR("STOP"),stdGadg,stdActv);
    WriteStrGad(gad[1],a0); WriteStrGad(gad[2],b0); WriteStrGad(gad[3],rd);
    WriteStrGad(gad[4],lx); WriteStrGad(gad[5],ly); WriteStrGad(gad[6],lz);
    WriteStrGad(gad[7],fa); WriteStrGad(gad[8],fs); WriteStrGad(gad[9],fd);
    WriteStrGad(gad[10],e);
    CopyMem(ADR("RAM:Planet.iff"),GadgetText(gad[19]),15);
    propInfo:=gad[14]^.specialInfo; propInfo^.vertPot:=ValToPot(15-R[1],16);
    propInfo:=gad[15]^.specialInfo; propInfo^.vertPot:=ValToPot(15-G[1],16);
    propInfo:=gad[16]^.specialInfo; propInfo^.vertPot:=ValToPot(15-B[1],16);
    FOR i := 1 TO 19 DO
      IF AddGadget(win,gad[i],-1)=0 THEN END;
    END;
    RefreshGadgets(gad[1],win,NIL);
    attr := 1;
    WriteWindow;
    LOOP
      WaitPort(win^.userPort);
      GetIMes(win,class,code,adr);
      IF closeWindow IN class THEN Terminate(0) END;
      IF gadgetUp IN class THEN
        choosedGad:=adr;
        CASE choosedGad^.gadgetID OF
        1:  ReadStrGad(gad[1],a0);
            WriteStrGad(gad[1],a0);
            RefreshGList(gad[1],win,NIL,1);
            dummy := ActivateGadget(gad[2],win,NIL) |
        2:  ReadStrGad(gad[2],b0);
            WriteStrGad(gad[2],b0);
            RefreshGList(gad[2],win,NIL,1);
            dummy := ActivateGadget(gad[3],win,NIL) |
        3:  ReadStrGad(gad[3],rd);
            WriteStrGad(gad[3],rd);
            RefreshGList(gad[3],win,NIL,1);
            dummy := ActivateGadget(gad[4],win,NIL) |
        4:  ReadStrGad(gad[4],lx);
            WriteStrGad(gad[4],lx);
            RefreshGList(gad[4],win,NIL,1);
            dummy := ActivateGadget(gad[5],win,NIL) |
        5:  ReadStrGad(gad[5],ly);
            WriteStrGad(gad[5],ly);
            RefreshGList(gad[5],win,NIL,1);
            dummy := ActivateGadget(gad[6],win,NIL) |
        6:  ReadStrGad(gad[6],lz);
            WriteStrGad(gad[6],lz);
            RefreshGList(gad[6],win,NIL,1);
            dummy := ActivateGadget(gad[7],win,NIL) |
        7:  ReadStrGad(gad[7],fa);
            WriteStrGad(gad[7],fa);
            RefreshGList(gad[7],win,NIL,1);
            dummy := ActivateGadget(gad[8],win,NIL) |
        8:  ReadStrGad(gad[8],fs);
            WriteStrGad(gad[8],fs);
            RefreshGList(gad[8],win,NIL,1);
            dummy := ActivateGadget(gad[9],win,NIL) |
        9:  ReadStrGad(gad[9],fd);
            WriteStrGad(gad[9],fd);
            RefreshGList(gad[9],win,NIL,1);
            dummy := ActivateGadget(gad[10],win,NIL) |
        10: ReadStrGad(gad[10],e);
            WriteStrGad(gad[10],e);
            RefreshGList(gad[10],win,NIL,1) |
        11..13:
            DrawFrame(rpw,298,20*attr-1,78,16,4,2,0);
            attr := choosedGad^.gadgetID-10;
            DrawFrame(rpw,298,20*attr-1,78,16,4,2,1);
            NewModifyProp(gad[14],win,NIL,pf,0,ValToPot(15-R[attr],16),0,4096,1);
            NewModifyProp(gad[15],win,NIL,pf,0,ValToPot(15-G[attr],16),0,4096,1);
            NewModifyProp(gad[16],win,NIL,pf,0,ValToPot(15-B[attr],16),0,4096,1)|
        14: R[attr] := 15-VertPot(gad[14],16); CalcColors |
        15: G[attr] := 15-VertPot(gad[15],16); CalcColors |
        16: B[attr] := 15-VertPot(gad[16],16); CalcColors |
        17: IF Normierung() THEN
              FOR i := 19 TO 1 BY -1 DO
                IF RemoveGadget(win,gad[i])=0 THEN END;
              END;
              SetRast(rpw,0);
              IF AddGadget(win,gadStop,-1)=0 THEN END;
              RefreshGadgets(gadStop,win,NIL);
              CalcPlanet;
              IF RemoveGadget(win,gadStop)=0 THEN END;
              SetRast(rpw,0);
              FOR i := 1 TO 19 DO
                IF AddGadget(win,gad[i],-1)=0 THEN END;
              END;
              RefreshGadgets(gad[1],win,NIL);
              WriteWindow
            ELSE
              DisplayBeep(NIL)
            END |
        18: strPtr := GadgetText(gad[19]);
            IF scr#NIL THEN
              IF NOT WriteILBMScreen(strPtr^,scr,NIL,TRUE) THEN
                DisplayBeep(NIL)
              END
            ELSE
              DisplayBeep(NIL)
            END
        ELSE
        END
      END
    END;
  END Main;

BEGIN
  scr := NIL; win := NIL;
  TermProcedure(CleanUp);
  Main;
END Planet.
