(*
 * -------------------------------------------------------------------------
 *
 *	:Program.	Drucke
 *	:Contents.	Ein Druckprogramm.
 *	:Author.	Reiner Nix
 *	:Address.	Geranienhof 2, 5000 Kln 71 Seeberg
 *	:Copyright.	Public Domain
 *	:Language.	Modula-2
 *	:Translator.	M2Amiga A-L V3.3d
 *	:History.	V1.0	21.11.90
 *	:Imports.	ConTools,		siehe diese Diskette
 *	:Imports.	FileOut,		siehe diese Diskette
 *
 * -------------------------------------------------------------------------
 *)
MODULE Drucke;


FROM	SYSTEM		IMPORT	ADR;
FROM	Arts		IMPORT	Assert;
FROM	Arguments	IMPORT	NumArgs, GetArg;
FROM	Intuition	IMPORT	pica, elite, fine, draft, letter, single,
				Preferences,
				GetPrefs;
FROM	Dos		IMPORT	Date, DateStamp;
FROM	Workbench	IMPORT	WBObjectType,
				DiskObjectPtr;
FROM	Icon		IMPORT	GetDiskObject, FreeDiskObject,
				FindToolType, MatchToolValue;
FROM	ASCII		IMPORT	sp, cr, eol, lf, ff, ht, csi, esc;
FROM	DateConversions	IMPORT	DateInfo, FromDos, DateToStr;
FROM	Heap		IMPORT	Allocate, Deallocate;
FROM	Conversions	IMPORT	StrToVal;
FROM	Str		IMPORT	Copy, Concat, FirstPos, Compare, Length;
FROM	Strings		IMPORT	Delete, Occurs;
FROM	FileSystem	IMPORT	File, Response,
				Lookup, Close, ReadChar;
FROM	FileMessage	IMPORT	StrPtr,
				ResponseText;
FROM	ConTools	IMPORT	TBereich, TZeichen, TZeichenMenge, TStatus,
				Kommentar, Status,
				BenutzeBereich, LoescheAusgabe, SetzePosition,
				WriteString, WriteCard, Write, WriteLn,
				ReadString, Read, TasteGedrueckt, ZeichenWahl,
				Meldung, DefiniereBereich;
IMPORT	FileOut;


CONST	maxDateiname	=34;

TYPE	TZustand	=(Text, Satz1, Satz2, Bemerkung);

	TDateiname	=ARRAY [0..maxDateiname] OF CHAR;
	TQualitaet	=(Entwurf, Brief);
	TBreite		=(Pica, Elite, Fine);
	TFormat		=(Frei, Modula);

	TEinstellung	=RECORD Kopf, Einzelblatt,
				Vorschub, Nummerierung	:BOOLEAN;
				Ziel			:TDateiname;
				Tabulator,
				RandOben, RandUnten,
				RandLinks, RandRechts,
				BlattLaenge		:CARDINAL;
				Qualitaet		:TQualitaet;
				Breite			:TBreite;
				Format			:TFormat
				END;


VAR	Einstellung		:TEinstellung;
	Weiter,
	ParameterFehler,
	WarteBeiFehler		:BOOLEAN;
	i, j, Laenge		:INTEGER;
	Eingabe			:TDateiname;
	BildBereich		:TBereich;
	Zeichen			:CHAR;


PROCEDURE BenutzeStandard	(VAR Einstellung	:TEinstellung);

BEGIN
WITH Einstellung DO
  Kopf		:= TRUE;
  Einzelblatt	:= FALSE;
  Vorschub	:= TRUE;
  Nummerierung	:= TRUE;
  Ziel		:= "Prt:";
  Tabulator	:= 8;
  RandOben	:= 0;		RandUnten	:= 0;
  RandLinks	:= 1;		RandRechts	:= 92;
  BlattLaenge	:= 72;
  Qualitaet	:= Entwurf;
  Breite	:= Elite;
  Format	:= Modula
  END
END BenutzeStandard;


PROCEDURE BenutzePreferences	(VAR Einstellung	:TEinstellung);

VAR	Prefs	:Preferences;

BEGIN
GetPrefs (ADR (Prefs), SIZE (Preferences));
WITH Einstellung DO
  WITH Prefs DO
    IF paperType = single THEN
      Einzelblatt := TRUE END;
    RandLinks := printLeftMargin;
    RandRechts := printRightMargin;
    BlattLaenge := paperLength;
    IF    printQuality = draft THEN
      Qualitaet := Entwurf
    ELSIF printQuality = letter THEN
      Qualitaet := Brief
      END;
    IF    printPitch = pica THEN
      Breite := Pica
    ELSIF printPitch = elite THEN
      Breite := Elite
    ELSIF printPitch = fine THEN
      Breite := Fine
      END
    END
  END
END BenutzePreferences;


PROCEDURE BenutzeIcon		(VAR Einstellung	:TEinstellung;
				 VAR ErstesArgument	:INTEGER);

VAR	Programmicon	:DiskObjectPtr;
	Programmname	:TDateiname;
	Laenge		:INTEGER;
	ToolType	:POINTER TO ARRAY [0..128] OF CHAR;


  PROCEDURE CheckBooleanTool	(VAR Tool		:BOOLEAN;
  				     Toolname		:ARRAY OF CHAR);


  BEGIN
  ToolType := FindToolType (Programmicon^.toolTypes, ADR (Toolname));
  IF ToolType # NIL THEN
    IF    MatchToolValue (ToolType, ADR ("Ja")) OR
          MatchToolValue (ToolType, ADR ("ja")) OR
          MatchToolValue (ToolType, ADR ("JA")) THEN
      Tool := TRUE
    ELSIF MatchToolValue (ToolType, ADR ("Nein")) OR
          MatchToolValue (ToolType, ADR ("nein")) OR
          MatchToolValue (ToolType, ADR ("NEIN")) THEN
      Tool := FALSE
      END
    END
  END CheckBooleanTool;


  PROCEDURE CheckCardinalTool	(VAR Tool		:CARDINAL;
				     Toolname		:ARRAY OF CHAR);

  VAR	Negativ, Fehler		:BOOLEAN;
  	Zahl			:LONGINT;

  BEGIN
  ToolType := FindToolType (Programmicon^.toolTypes, ADR (Toolname));
  IF ToolType # NIL THEN
    StrToVal (ToolType^, Zahl, Negativ, 10, Fehler);
    IF NOT (Fehler) AND (0 <= Zahl) AND (Zahl <= 1000) THEN
      Tool := CARDINAL (Zahl)
      END
    END
  END CheckCardinalTool;


(* BenutzeIcon *)
BEGIN
WITH Einstellung DO
  GetArg (0, Programmname, Laenge);
  Programmicon := GetDiskObject (ADR (Programmname));
  IF (Programmicon # NIL) AND (Programmicon^.type = project) THEN
    ToolType := Programmicon^.defaultTool;
    Copy (Programmname, ToolType^);
    FreeDiskObject (Programmicon);
    Programmicon := GetDiskObject (ADR (Programmname));
    ErstesArgument := 0
  ELSE
    ErstesArgument := 1
    END;
  IF Programmicon # NIL THEN
    CheckBooleanTool (Kopf, "KOPF");
    CheckBooleanTool (Einzelblatt, "EINZELBLATT");
    CheckBooleanTool (Vorschub, "VORSCHUB");
    CheckBooleanTool (Nummerierung, "NUMMERIERUNG");
    ToolType := FindToolType (Programmicon^.toolTypes, ADR ("ZIEL"));
    IF ToolType # NIL THEN
      Copy (Ziel, ToolType^)
      END;
    CheckCardinalTool (Tabulator, "TABULATOR");
    CheckCardinalTool (RandOben, "RANDOBEN");
    CheckCardinalTool (RandUnten, "RANDUNTEN");
    CheckCardinalTool (RandLinks, "RANDLINKS");
    CheckCardinalTool (RandRechts, "RANDRECHTS");
    CheckCardinalTool (BlattLaenge, "BLATTLNGE");
    ToolType := FindToolType (Programmicon^.toolTypes, ADR ("QUALITT"));
    IF ToolType # NIL THEN
      IF    MatchToolValue (ToolType, ADR ("Entwurf")) OR
            MatchToolValue (ToolType, ADR ("ENTWURF")) THEN
        Qualitaet := Entwurf
      ELSIF MatchToolValue (ToolType, ADR ("Brief")) OR
            MatchToolValue (ToolType, ADR ("BRIEF")) THEN
        Qualitaet := Brief
        END
      END;
    ToolType := FindToolType (Programmicon^.toolTypes, ADR ("BREITE"));
    IF ToolType # NIL THEN
      IF    MatchToolValue (ToolType, ADR ("Pica")) OR
            MatchToolValue (ToolType, ADR ("PICA")) THEN
        Breite := Pica
      ELSIF MatchToolValue (ToolType, ADR ("Elite")) OR
            MatchToolValue (ToolType, ADR ("ELITE")) THEN
        Breite := Elite
      ELSIF MatchToolValue (ToolType, ADR ("Fine")) OR
            MatchToolValue (ToolType, ADR ("FINE")) THEN
        Breite := Fine
        END
      END;
    ToolType := FindToolType (Programmicon^.toolTypes, ADR ("FORMAT"));
    IF ToolType # NIL THEN
      IF    MatchToolValue (ToolType, ADR ("Frei")) OR
            MatchToolValue (ToolType, ADR ("FREI")) THEN
        Format := Frei
      ELSIF MatchToolValue (ToolType, ADR ("Modula")) OR
            MatchToolValue (ToolType, ADR ("MODULA")) THEN
        Format := Modula
        END
      END;
    FreeDiskObject (Programmicon)
    END
  END
END BenutzeIcon;


PROCEDURE WriteBool		(    Boolean		:BOOLEAN);

BEGIN
IF Boolean THEN
  WriteString (" Ja ")
ELSE
  WriteString ("Nein")
  END
END WriteBool;


PROCEDURE AusgabeMaske;

BEGIN
BenutzeBereich (BildBereich); LoescheAusgabe;
SetzePosition (5, 2); WriteString ("Kopfzeile ausgeben                KOPF = ");
SetzePosition (5, 3); WriteString ("Einzelbltter bedrucken    EINZELBLATT = ");
SetzePosition (5, 4); WriteString ("Blattvorschub                 VORSCHUB = ");
SetzePosition (5, 5); WriteString ("Zeilen nummerieren        NUMMERIERUNG = ");
SetzePosition (5, 6); WriteString ("Ausgabe Ziel                      ZIEL = ");
SetzePosition (5, 7); WriteString ("Tabulatorabstand             TABULATOR = ");
SetzePosition (5, 8); WriteString ("oberer Rand                   RANDOBEN = ");
SetzePosition (5, 9); WriteString ("unterer Rand                 RANDUNTEN = ");
SetzePosition (5,10); WriteString ("linker Rand                  RANDLINKS = ");
SetzePosition (5,11); WriteString ("rechter Rand                RANDRECHTS = ");
SetzePosition (5,12); WriteString ("Blattlnge                  BLATTLNGE = ");
SetzePosition (5,13); WriteString ("Ausgabequalitt               QUALITT = ");
SetzePosition (5,14); WriteString ("Zeichenbreite                   BREITE = ");
SetzePosition (5,15); WriteString ("Formatierung                    FORMAT = ");
WITH Einstellung DO
  SetzePosition (45, 2); WriteBool (Kopf);
  SetzePosition (45, 3); WriteBool (Einzelblatt);
  SetzePosition (45, 4); WriteBool (Vorschub);
  SetzePosition (45, 5); WriteBool (Nummerierung);
  SetzePosition (45, 6); Write (csi); Write (113C); WriteString (Ziel);
  SetzePosition (45, 7); WriteCard (Tabulator, 4);
  SetzePosition (45, 8); WriteCard (RandOben, 4);
  SetzePosition (45, 9); WriteCard (RandUnten, 4);
  SetzePosition (45,10); WriteCard (RandLinks, 4);
  SetzePosition (45,11); WriteCard (RandRechts, 4);
  SetzePosition (45,12); WriteCard (BlattLaenge, 4);
  SetzePosition (45,13);
  CASE Qualitaet OF
  | Entwurf	:WriteString ("Entwurf")
  | Brief	:WriteString ("Brief  ")
    END;
  SetzePosition (45,14);
  CASE Breite OF
  | Pica	:WriteString ("Pica ")
  | Elite	:WriteString ("Elite")
  | Fine	:WriteString ("Fine ")
    END;
  SetzePosition (45,15);
  CASE Format OF
  | Frei	:WriteString ("Frei  ")
  | Modula	:WriteString ("Modula")
    END
  END
END AusgabeMaske;


PROCEDURE UpString		(VAR Satz		:ARRAY OF CHAR);

VAR	i	:CARDINAL;

BEGIN
i := 0;
WHILE (i < CARDINAL (HIGH (Satz))) AND (Satz[i] # 0C) DO
  CASE Satz[i] OF
  | 'a'..'z': Satz[i] := CAP (Satz[i]);
  | ''     : Satz[i] := ""
  ELSE
    END;
  INC (i)
  END
END UpString;


PROCEDURE BenutzeParameter	(    Eingabe		:ARRAY OF CHAR;
				     Warte		:BOOLEAN;
				 VAR ParameterFehler	:BOOLEAN);

CONST	G1 	="KOPF EINZELBLATT VORSCHUB NUMMERIERUNG ZIEL ";
	G2 	="TABULATOR RANDOBEN RANDUNTEN RANDLINKS RANDRECHTS ";
	G3	="BLATTLNGE QUALITT BREITE FORMAT ";

VAR	i			:CARDINAL;
	Parameter		:TDateiname;
	Gesamt			:ARRAY [0..130] OF CHAR;

  PROCEDURE TesteBoolean	(VAR Boolean	:BOOLEAN;
  				     Satz	:ARRAY OF CHAR);

  BEGIN
  UpString (Satz);
  IF    Compare (Satz, "JA") = 0 THEN
    Boolean := TRUE;
  ELSIF Compare (Satz, "NEIN") = 0 THEN
    Boolean := FALSE;
  ELSE
    ParameterFehler := TRUE;
    Meldung ("Parameter kein Wahrheitswert!", Warte)
    END
  END TesteBoolean;


  PROCEDURE TesteCardinal	(VAR Zahl	:CARDINAL;
  				     Satz	:ARRAY OF CHAR);

  VAR	Negativ, Fehler		:BOOLEAN;
  	iZahl			:LONGINT;

  BEGIN
  StrToVal (Satz, iZahl, Negativ, 10, Fehler);
  IF NOT (Fehler) AND (0 <= iZahl) AND (iZahl <= 1000) THEN
    Zahl := CARDINAL (iZahl);
  ELSE
    ParameterFehler := TRUE;
    Meldung ("Parameter ist keine gltige Zahl!", Warte)
    END
  END TesteCardinal;


(* BenutzeParameter *)
BEGIN
ParameterFehler := FALSE;
WITH Einstellung DO
  Copy (Gesamt, G1); Concat (Gesamt, G2); Concat (Gesamt, G3);
  i := 0;
  WHILE (i < CARDINAL (HIGH (Eingabe))) AND (Eingabe[i] # 0C) DO
    IF Eingabe[i] = sp THEN
      Delete (Eingabe, i, 1)
    ELSE
      INC (i)
      END
    END;
  i := CARDINAL (FirstPos (Eingabe, 0, "="));
  Copy (Parameter, Eingabe);
  Parameter[i] := sp; Parameter[i+1] := 0C;
  UpString (Parameter);
  Delete (Eingabe, 0, i+1);
  BenutzeBereich (BildBereich);
  CASE Occurs (Gesamt, 0, Parameter, FALSE) OF
  |   0:
    TesteBoolean (Kopf, Eingabe);
    SetzePosition (45, 2); WriteBool (Kopf)
  |   5:
    TesteBoolean (Einzelblatt, Eingabe);
    SetzePosition (45, 3); WriteBool (Einzelblatt)
  |  17:
    TesteBoolean (Vorschub, Eingabe);
    SetzePosition (45, 4); WriteBool (Vorschub)
  |  26:
    TesteBoolean (Nummerierung, Eingabe);
    SetzePosition (45, 5); WriteBool (Nummerierung)
  |  39:
    Copy (Ziel, Eingabe);
    SetzePosition (45, 6); Write (csi); Write (113C); WriteString (Ziel)
  |  44:
    TesteCardinal (Tabulator, Eingabe);
    SetzePosition (45, 7); WriteCard (Tabulator, 4)
  |  54:
    TesteCardinal (RandOben, Eingabe);
    SetzePosition (45, 8); WriteCard (RandOben, 4)
  |  63:
    TesteCardinal (RandUnten, Eingabe);
    SetzePosition (45, 9); WriteCard (RandUnten, 4)
  |  73:
    TesteCardinal (RandLinks, Eingabe);
    SetzePosition (45,10); WriteCard (RandLinks, 4)
  |  83:
    TesteCardinal (RandRechts, Eingabe);
    SetzePosition (45,11); WriteCard (RandRechts, 4)
  |  94:
    TesteCardinal (BlattLaenge, Eingabe);
    SetzePosition (45,12); WriteCard (BlattLaenge, 4)
  | 105:
    UpString (Eingabe);
    SetzePosition (45,13);
    IF    Compare (Eingabe, "ENTWURF") = 0 THEN
      Qualitaet := Entwurf;
      WriteString ("Entwurf");
    ELSIF Compare (Eingabe, "BRIEF") = 0 THEN
      Qualitaet := Brief;
      WriteString ("Brief  ")
    ELSE
      ParameterFehler := TRUE;
      Meldung ("Der Pameter ist keine gltige Eingabe fr die Qualitt!",Warte)
      END
  | 114:
    UpString (Eingabe);
    SetzePosition (45,14);
    IF    Compare (Eingabe, "PICA") = 0 THEN
      Breite := Pica;
      WriteString ("Pica ")
    ELSIF Compare (Eingabe, "ELITE") = 0 THEN
      Breite := Elite;
      WriteString ("Elite")
    ELSIF Compare (Eingabe, "FINE") = 0 THEN
      Breite := Fine;
      WriteString ("Fine ")
    ELSE
      ParameterFehler := TRUE;
      Meldung ("Der Parameter ist keine gltige Eingabe fr die Breite!",Warte)
      END
  | 121:
    UpString (Eingabe);
    SetzePosition (45,15);
    IF    Compare (Eingabe, "FREI") = 0 THEN
      Format := Frei;
      WriteString ("Frei  ")
    ELSIF Compare (Eingabe, "MODULA") = 0 THEN
      Format := Modula;
      WriteString ("Modula")
    ELSE
      ParameterFehler := TRUE;
      Meldung ("Der Parameter ist keine gltige Eingabe fr das Format!",Warte)
      END
  ELSE
    ParameterFehler := TRUE;
    Meldung ("Die Eingabe ist kein bekannter Parameter!", Warte)
    END
  END
END BenutzeParameter;


PROCEDURE DruckeDatei		(    Quelle		:ARRAY OF CHAR);

CONST	maxWort	=13;
  	R1	="AND ARRAY BEGIN BY CASE CONST DEFINITION DIV DO ELSE ";
  	R2	="ELSIF END EXIT EXPORT FOR FROM IF IMPLEMENTATION IMPORT ";
  	R3	="IN LOOP MOD MODULE NOT OF OR POINTER PROCEDURE QUALIFIED ";
  	R4	="RECORD REPEAT RETURN SET THEN TO TYPE UNTIL VAR WHILE WITH ";
	Monate	="Januar|Februar|Mrz|April|Mai|Juni|Juli|August|September|Oktober|November|Dezember";

VAR	QuellDatei, ZielDatei	:File;
	Fehlermeldung		:ARRAY [0..80] OF CHAR;
	FehlerText		:StrPtr;
	Zustand			:TZustand;
	BemerkungsTiefe,
	SeitenNr, ZeilenNr,
	ZeilenPos, SpaltenPos	:LONGCARD;
	i			:CARDINAL;
	Zeichen, AltesZeichen	:CHAR;
	Abbruch, Wiederholen	:BOOLEAN;
  	Reserviert		:ARRAY [0..300] OF CHAR;


  PROCEDURE KursivAn;

  BEGIN
  FileOut.Write		(ZielDatei, csi);
  FileOut.WriteString	(ZielDatei, "3m")
  END KursivAn;


  PROCEDURE KursivAus;

  BEGIN
  FileOut.Write		(ZielDatei, csi);
  FileOut.WriteString	(ZielDatei, "0m")
  END KursivAus;


  PROCEDURE KleinschriftAn;

  BEGIN
  FileOut.Write		(ZielDatei, csi);
  FileOut.WriteString	(ZielDatei, "4v")
  END KleinschriftAn;


  PROCEDURE KleinschriftAus;

  BEGIN
  FileOut.Write		(ZielDatei, csi);
  FileOut.WriteString	(ZielDatei, "3v")
  END KleinschriftAus;


  PROCEDURE FettAn;

  BEGIN
  FileOut.Write		(ZielDatei, csi);
  FileOut.WriteString	(ZielDatei, "1m")
  END FettAn;


  PROCEDURE FettAus;

  BEGIN
  FileOut.Write		(ZielDatei, csi);
  FileOut.WriteString	(ZielDatei, "0m")
  END FettAus;


  PROCEDURE druckeKopf;

  VAR	Datum			:Date;
  	DatumsInfo		:DateInfo;
  	DatumsText		:ARRAY [0..20] OF CHAR;
  	i			:CARDINAL;

  BEGIN
  WITH Einstellung DO
    IF Kopf THEN
      IF Zustand = Bemerkung THEN
        KursivAus END;
      FOR i := 1 TO RandOben DO
        FileOut.WriteLn		(ZielDatei);
        INC (ZeilenPos)
        END;
      FileOut.WriteString	(ZielDatei, "Seite: ");
      FileOut.WriteCard		(ZielDatei, SeitenNr,3);
      FileOut.WriteString	(ZielDatei, "  '");
      FileOut.WriteString	(ZielDatei, Quelle);
      FileOut.Write	(ZielDatei, "'");
      FOR i := Length (Quelle)+1 TO maxDateiname DO
        FileOut.Write (ZielDatei, sp)
        END;
      DateStamp (ADR (Datum));
      FromDos (Datum, DatumsInfo);
      DateToStr (DatumsInfo,"  %d.%t.%Y  %H:%M",Monate,DatumsText);
      FileOut.WriteString	(ZielDatei, DatumsText);
      FileOut.WriteLn		(ZielDatei);
      FileOut.WriteLn		(ZielDatei);
      INC (ZeilenPos, 2);
      SpaltenPos := RandLinks;
      INC (SeitenNr);
      IF Zustand = Bemerkung THEN
        KursivAn
        END
      END
    END
  END druckeKopf;


  PROCEDURE startDrucken;

  BEGIN
  WITH Einstellung DO
    FileOut.Write 	(ZielDatei, esc);		(* initPrinter *)
    FileOut.WriteString	(ZielDatei, "#1");

    FileOut.Write	(ZielDatei, csi);		(* Left & Right *)
    FileOut.WriteCard	(ZielDatei, RandLinks, 1);
    FileOut.Write	(ZielDatei, ";");
    FileOut.WriteCard	(ZielDatei, RandRechts, 1);
    FileOut.Write	(ZielDatei, "s");

    FileOut.Write	(ZielDatei, csi);		(* NLQ *)
    CASE Qualitaet OF
    | Entwurf	:FileOut.WriteString	(ZielDatei, '1"z')
    | Brief	:FileOut.WriteString	(ZielDatei, '2"z')
    ELSE
      END;

    FileOut.Write	(ZielDatei, csi);		(* Pitch *)
    CASE Breite OF
    | Pica	:FileOut.WriteString	(ZielDatei, '0w')
    | Elite	:FileOut.WriteString	(ZielDatei, '2w')
    | Fine	:FileOut.WriteString	(ZielDatei, '4w')
    ELSE
      END;

    SeitenNr := 1;
    ZeilenNr := 1;
    ZeilenPos := 1;
    SpaltenPos := RandLinks;
    druckeKopf;
    IF Nummerierung THEN
      KleinschriftAn;
      FileOut.WriteCard	(ZielDatei, ZeilenNr, 5);
      FileOut.Write		(ZielDatei, ":");
      KleinschriftAus;
      INC (ZeilenNr)
      END
    END
  END startDrucken;


  PROCEDURE stopDrucken;

  BEGIN
  WITH Einstellung DO
    IF Vorschub THEN
      FileOut.Write	(ZielDatei, ff)
      END;
    FileOut.Write 	(ZielDatei, esc);		(* initPrinter *)
    FileOut.WriteString	(ZielDatei, "#1")
    END
  END stopDrucken;


  PROCEDURE neueSeite;

  BEGIN
  WITH Einstellung DO
    FileOut.Write	(ZielDatei, ff);
    ZeilenPos := 1;
    IF Einzelblatt THEN
      Meldung ("Seitenende erreicht.", TRUE);
      LoescheAusgabe
      END;
    druckeKopf
    END
  END neueSeite;


  PROCEDURE neueZeile;

  BEGIN
  WITH Einstellung DO
    IF ZeilenPos < BlattLaenge-RandUnten THEN
      FileOut.WriteLn	(ZielDatei);
      INC (ZeilenPos)
    ELSE
      neueSeite
      END;
    SpaltenPos := RandLinks;
    END;
  SetzePosition ( 62, 1);
  WriteString ("Zeile: "); WriteCard (ZeilenNr,5);
  IF TasteGedrueckt () THEN
    Read (Zeichen);
    ZeichenWahl (2,2, "Soll der Druckvorgang abgebrochen werden (J/N) ?",
                 TZeichenMenge {Escape, Return, JaNein}, Zeichen);
    SetzePosition (2,2);
    WriteString ("                                                  ");
    IF (CAP (Zeichen) = "J") OR (Zeichen = esc) THEN
      Abbruch := TRUE
      END
    END
  END neueZeile;


  PROCEDURE schreibeZeichen		(    Zeichen		:CHAR);

  BEGIN
  WITH Einstellung DO
    IF SpaltenPos >= RandRechts THEN
      neueZeile
    ELSE
      INC (SpaltenPos)
      END;
    FileOut.Write	(ZielDatei, Zeichen)
    END
  END schreibeZeichen;


  PROCEDURE TesteModulaWort;

  VAR	i,j			:CARDINAL;
  	Puffer			:ARRAY [0..maxWort+2] OF CHAR;

  BEGIN
  IF (Zustand # Text) OR (Zeichen < "A") OR ("Z" < Zeichen) THEN
    schreibeZeichen (Zeichen);
    RETURN
    END;
  i := 0;
  WHILE (i < maxWort) AND ("A" <= Zeichen) AND (Zeichen <= "Z") DO
    Puffer[i] := Zeichen; INC (i);
    ReadChar (QuellDatei, Zeichen)
    END;
  AltesZeichen := Zeichen; Wiederholen := TRUE;
  IF i < maxWort THEN
    Puffer[i] := sp;
    Puffer[i+1] := 0C;
    IF (i > 1) AND (Occurs (Reserviert, 0, Puffer, FALSE) # -1) THEN
      FettAn;
      j := 0;
      WHILE j < i DO
        schreibeZeichen (Puffer[j]); INC (j)
        END;
      FettAus
    ELSE
      j := 0;
      WHILE j < i DO
        schreibeZeichen (Puffer[j]); INC (j)
        END
      END
  ELSE
    j := 0;
    WHILE j < i DO
      schreibeZeichen (Puffer[j]); INC (j)
      END;
    WHILE ("A" <= Zeichen) AND (Zeichen <= "Z") DO
      schreibeZeichen (Zeichen);
      ReadChar (QuellDatei, Zeichen)
      END
    END
  END TesteModulaWort;


(* DruckeDatei *)
BEGIN
Copy (Reserviert, R1); Concat (Reserviert, R2);
Concat (Reserviert, R3); Concat (Reserviert, R4);
Copy (Fehlermeldung, "Ausgabe der Datei: "); Concat (Fehlermeldung, Quelle);
Meldung (Fehlermeldung, FALSE);
Lookup (QuellDatei, Quelle, 512, FALSE);
IF QuellDatei.res # done THEN
  ResponseText (QuellDatei.res, FehlerText);
  Close (QuellDatei);
  Copy (Fehlermeldung, "Fehler beim ffnen der Quelldatei: ");
  Concat (Fehlermeldung, FehlerText^);
  Meldung (Fehlermeldung, TRUE);
  RETURN
  END;
Lookup (ZielDatei, Einstellung.Ziel, 0, TRUE);
IF ZielDatei.res # done THEN
  ResponseText (ZielDatei.res, FehlerText);
  Close (ZielDatei);
  Copy (Fehlermeldung, "Fehler beim ffnen der Zieldatei: ");
  Concat (Fehlermeldung, FehlerText^);
  Meldung (Fehlermeldung, TRUE);
  RETURN
  END;

Zustand := Text;
Abbruch := FALSE;
Wiederholen := FALSE;
BemerkungsTiefe:= 0;
WITH Einstellung DO
  startDrucken;
  REPEAT
    IF Wiederholen THEN
      Zeichen := AltesZeichen;
      Wiederholen := FALSE
    ELSE
      ReadChar (QuellDatei, Zeichen)
      END;
    IF Zeichen = ff THEN
      neueSeite
    ELSIF Zeichen = lf THEN
      IF (Zustand = Satz1) OR (Zustand = Satz2) THEN
        Zustand := Text
        END;
      neueZeile;
      IF Nummerierung THEN
        KleinschriftAn;
        IF Zustand = Bemerkung THEN
          KursivAus
          END;
        FileOut.WriteCard	(ZielDatei, ZeilenNr, 5);
        FileOut.Write		(ZielDatei, ":");
        KleinschriftAus;
        INC (SpaltenPos, 6);
        IF Zustand = Bemerkung THEN
          KursivAn
          END
        END;
      INC (ZeilenNr)
    ELSIF (Zeichen = csi) OR (Zeichen = esc) THEN
        REPEAT
        FileOut.Write	(ZielDatei, Zeichen);
        ReadChar (QuellDatei, Zeichen)
        UNTIL (("A" <= CAP (Zeichen)) AND (CAP (Zeichen) <= "Z"))
    ELSIF Zeichen = "'" THEN
      IF Format = Modula THEN
        CASE Zustand OF
        | Text	:Zustand := Satz1
        | Satz1	:Zustand := Text
        ELSE
          END
        END;
      schreibeZeichen (Zeichen)
    ELSIF Zeichen = '"' THEN
      IF Format = Modula THEN
        CASE Zustand OF
        | Text	:Zustand := Satz2
        | Satz2	:Zustand := Text
        ELSE
          END
        END;
      schreibeZeichen (Zeichen)
    ELSIF Zeichen = "(" THEN
      IF (Format = Modula) AND ((Zustand = Text) OR (Zustand = Bemerkung)) THEN
        ReadChar (QuellDatei, Zeichen);
        IF Zeichen # "*" THEN
          schreibeZeichen ("(");
          AltesZeichen := Zeichen; Wiederholen := TRUE
        ELSE
          INC (BemerkungsTiefe);
          IF Zustand = Text THEN
            Zustand := Bemerkung;
            KursivAn
            END;
          schreibeZeichen ("("); schreibeZeichen ("*")
          END
      ELSE
        schreibeZeichen (Zeichen)
        END
    ELSIF Zeichen = "*" THEN
      IF (Format = Modula) AND (Zustand = Bemerkung) THEN
        schreibeZeichen (Zeichen);
        ReadChar (QuellDatei, Zeichen);
        IF Zeichen = ")" THEN
          schreibeZeichen (Zeichen);
          DEC (BemerkungsTiefe);
          IF BemerkungsTiefe = 0 THEN
            Zustand := Text;
            KursivAus
            END
        ELSE
          AltesZeichen := Zeichen; Wiederholen := TRUE
          END
      ELSE
        schreibeZeichen (Zeichen)
        END
    ELSIF Zeichen = ht THEN
      IF Tabulator = 0 THEN
        FileOut.Write	(ZielDatei, Zeichen)
      ELSE
        schreibeZeichen (" ");
        IF Nummerierung THEN
          WHILE (SpaltenPos-RandLinks-6) MOD Tabulator # 0 DO
            schreibeZeichen (" ")
            END
        ELSE
          WHILE (SpaltenPos-RandLinks) MOD Tabulator # 0 DO
            schreibeZeichen (" ")
            END
          END
        END
    ELSE (* CASE *)
      IF Format = Modula THEN
        TesteModulaWort
      ELSE
        schreibeZeichen (Zeichen)
        END
      END
    UNTIL Abbruch OR QuellDatei.eof OR (QuellDatei.res # done)
  END;
stopDrucken;
Close (ZielDatei);
Close (QuellDatei)
END DruckeDatei;


(* MODULE Drucke *)
BEGIN
DefiniereBereich (BildBereich, 3,2,73,23, 1,2);
BenutzeStandard (Einstellung);
BenutzePreferences (Einstellung);
BenutzeIcon (Einstellung, i);
BenutzeBereich (BildBereich); LoescheAusgabe;
AusgabeMaske;

Weiter := TRUE;
WHILE Weiter DO
  BenutzeBereich (BildBereich);
  FOR j := 0 TO 5 DO
    SetzePosition ( 5,17+j); Write (csi); Write (113C);
    SetzePosition ( 5,17+j);
    IF i+j <= NumArgs () THEN
      GetArg (i+j, Eingabe, Laenge);
      WriteCard (i+j, 4);
      WriteString ("   '"); WriteString (Eingabe); Write ("'")
      END
    END;
  IF i > NumArgs () THEN
    BenutzeBereich (Kommentar); LoescheAusgabe;
    SetzePosition (2,2); WriteString ("Eingabe:");
    IF NOT (ParameterFehler) THEN
      Eingabe[0] := 0C
      END;
    ReadString (12,2, maxDateiname, TZeichenMenge {Return, Escape, alleZeichen},
                Eingabe);
    Weiter := (Eingabe[0] # 0C) AND (Status # Zurueck);
    WarteBeiFehler := TRUE;
  ELSE
    GetArg (i, Eingabe, Laenge); INC (i);
    WarteBeiFehler := FALSE;
    END;
  IF Eingabe[0] # 0C THEN
    IF FirstPos (Eingabe, 0, "=") # -1 THEN
      BenutzeParameter (Eingabe, WarteBeiFehler, ParameterFehler)
    ELSE
      IF Occurs (Eingabe, 0, ".obj", FALSE) # -1 THEN
        BenutzeBereich (Kommentar); LoescheAusgabe;
        SetzePosition (2,1); WriteString (Eingabe);
        ZeichenWahl (2,2, "Soll die Datei gedruckt werden (J/N)? ",
                     TZeichenMenge {Return, Escape, JaNein}, Zeichen);
        IF (Zeichen = cr) OR (CAP (Zeichen) = "J") THEN
          DruckeDatei (Eingabe)
          END
      ELSE
        DruckeDatei (Eingabe)
        END
      END
    END
  END
END Drucke.
