(**********************************************************************

    :Program.    CED.mod
    :Contents.   Communication with CED
    :Author.     Achim Siebert; M2-Version by Steffen Reith;
    :Address.    Nobileweg 67 , 7000 Stuttgart 40
    :Copyright.  Shareware
    :Language.   Oberon
    :Translator. AmigaOberon A+L
    :Imports.    none
    :History.    Dec-1990; V1.1;

**********************************************************************)

MODULE CED;

IMPORT ExecSupport,
       Exec,
       SYSTEM;

TYPE StringType = POINTER TO ARRAY MAX(INTEGER)-1 OF CHAR;
     longintptr  = POINTER TO LONGINT;
          
VAR 
    Status* :  StringType;  (* Pointer to String returned by CED *)
    Result*  : LONGINT;     (* Number returned by CED            *)
    Result1* : LONGINT;     (* Error returned by CED             *)
    lengthptr: longintptr;
    CEDMsgPort,Reply:Exec.MsgPortPtr;
    
    
CONST CEDPortName* = 'rexx_ced';
      ResultWanted = 020000H;   (* 1L<<17 *)

TYPE CEDMsg*=STRUCT
                cmNode*:Exec.Message;
                rfu1*:LONGINT;
                rfu2*:LONGINT;
                action*:LONGINT;
                result1*:LONGINT;
                result2*:Exec.ADDRESS;
                args*:ARRAY 16 OF Exec.ADDRESS;
                rfu7*:LONGINT;
                rfu8*:LONGINT;
                rfu9*:LONGINT;
                rfu10*:LONGINT;
                rfu11*:LONGINT;
                rfu12*:LONGINT
             END;

     CEDMsgPtr*= POINTER TO CEDMsg;

CONST CEDMsgSize*= SYSTEM.SIZE(CEDMsg);

VAR Mes:CEDMsgPtr;
    
PROCEDURE PutMsg*(Command:ARRAY OF CHAR):BOOLEAN;

BEGIN

 CEDMsgPort:= Exec.FindPort(CEDPortName);
 IF CEDMsgPort=NIL THEN
    RETURN FALSE
 END;

 Reply:=ExecSupport.CreatePort("",0);
 IF Reply=NIL THEN
    RETURN FALSE
 END;

 Mes.args[0]:=SYSTEM.ADR(Command);
 Mes.action:=0;
 Mes.result2:=0;
 Mes.cmNode.replyPort:=Reply;

 Exec.PutMsg(CEDMsgPort,Mes);
 Exec.WaitPort(Reply);
 ExecSupport.DeletePort(Reply);
 RETURN TRUE
 
END PutMsg;

PROCEDURE GetString*(Command:ARRAY OF CHAR; VAR ResultString:ARRAY OF CHAR);

BEGIN

 CEDMsgPort:=Exec.FindPort(CEDPortName);
 IF CEDMsgPort=NIL THEN
    RETURN
 END;

 Reply:=ExecSupport.CreatePort("",0);
 IF Reply=NIL THEN
    RETURN
 END;

 Mes.args[0]:=SYSTEM.ADR(Command);
 Mes.action:=ResultWanted;
 Mes.result2:=0;
 Mes.cmNode.replyPort:=Reply;

 Exec.PutMsg(CEDMsgPort,Mes);
 Exec.WaitPort(Reply);
 ExecSupport.DeletePort(Reply);
 Status:= Mes.result2;
 Result := Mes.result2;
 Result1 := Mes.result1;
 ResultString := "";
 IF (Status # NIL) THEN
    COPY(Status^,ResultString);
    lengthptr := SYSTEM.VAL(longintptr,Status);
    DEC(lengthptr,SYSTEM.SIZE(LONGINT));
    IF LEN(ResultString) > lengthptr^ THEN
       ResultString[lengthptr^] := "\x00";
    END;
    Exec.FreeMem(lengthptr,lengthptr^+4);
 END;

END GetString;

PROCEDURE GetNumber*(Command:ARRAY OF CHAR):LONGINT;

BEGIN

 CEDMsgPort:=Exec.FindPort(CEDPortName);
 IF CEDMsgPort=NIL THEN
    RETURN -1
 END;

 Reply:=ExecSupport.CreatePort("",0);
 IF Reply=NIL THEN
    RETURN -1
 END;

 Mes.args[0]:=SYSTEM.ADR(Command);
 Mes.action:=ResultWanted;
 Mes.result2:=0;
 Mes.cmNode.replyPort:=Reply;

 Exec.PutMsg(CEDMsgPort,Mes);
 Exec.WaitPort(Reply);
 ExecSupport.DeletePort(Reply);
 Result := Mes.result2;
 Result1 := Mes.result1;
 RETURN Result; 
 
END GetNumber;

BEGIN

NEW(Mes);
IF Mes = NIL THEN HALT(20) END;
Mes.cmNode.node.type:=Exec.message;
Mes.cmNode.length:=CEDMsgSize;

CLOSE

IF Mes # NIL THEN DISPOSE(Mes) END;

END CED.
