(* Req.mod        ported to Oberon 1990 by Achim Siebert           *)
(* reqlibrary.h  1988/1989 reserved by Colin Fox and Bruce Dawson *)

MODULE Req;

IMPORT E: Exec,
       I: Intuition,
       G: Graphics,
          Dos,
          SYSTEM;
CONST 
      reqName    * = "req.library";
      reqVersion * = 1;
      
      numpairs   * = 10;

TYPE GadgetBlockPtr * = POINTER TO GadgetBlock;

TYPE GadgetBlock * = STRUCT
     gadget       *  : I.Gadget;
     border       *  : I.Border;
     pairs        *  : ARRAY numpairs OF INTEGER;
     text    *  : I.IntuiText;
END;

TYPE StringBlockPtr * = POINTER TO StringBlock;

TYPE StringBlock * = STRUCT
   gadget       *  : I.Gadget;
   info         *  : I.StringInfo;
   border       *  : I.Border;
   pairs        *  : ARRAY numpairs OF INTEGER;
END;

TYPE PropBlockPtr * = POINTER TO PropBlock;

TYPE PropBlock * = STRUCT
   gadget       *  : I.Gadget;
   info         *  : I.PropInfo;
   image *  : I.Image;
END;

TYPE ScrollBlockPtr * = POINTER TO ScrollBlock;

TYPE ScrollBlock * = STRUCT
   arrowUpLt    *  : I.Gadget;
   imageUpLt    *  : I.Image;
   arrowDnRt    *  : I.Gadget;
   imageDnRt    *  : I.Image;
   prop         *  : I.PropInfo;
END;

TYPE TwoImageGadgetPtr * = POINTER TO TwoImageGadget;

TYPE TwoImageGadget * = STRUCT
   gadget       *  : I.Gadget;
   image1       *  : I.Image;
   image2       *  : I.Image;
END;

CONST  
       VERTSLIDER  * = 16;   (* Default: Horizontal Slider *)


(* This structure is used with the TextRequester function. *)

TYPE TRStructurePtr * = POINTER TO TRStructure;

TYPE TRStructure * = STRUCT
   text         *  : E.ADDRESS; (*  This is the message text, including printf() style formatting if desired. *)
   controls     *  : E.ADDRESS;   (*  This is the address of the parameter list, if printf() style formatting is used. *)
   window       *  : I.WindowPtr; (*  This is an optional (zero if not used) pointer to a window on the screen you*)
                                  (*  would like the requester to show up on. *)
   middleText   *  : E.ADDRESS;   (*  If non-zero, this is the text for the gadget in the lower middle (returns 2). *)
   positiveText *  : E.ADDRESS;   (*  If non-zero, this is the text for the gadget in the lower left hand corner (returns 1). *)
   negativeText *  : E.ADDRESS;   (*  If non-zero, this is the text for the gadget in the lower right (returns 0). *)
   title        *  : E.ADDRESS;   (*  This is the title for the window. *)
   keyMask      *  : SET;         (*  This is the qualifier mask for the keyboard shortcuts. *)
                                  (*  Use -1 to allow any qualifiers (or none). *)
                                  (*  Zero means that no keyboard shortcuts are allowed. *)
   textcolor    *  : INTEGER;     (*  Color of the text.  Uses color 1 if no color specified. *)
   detailcolor  *  : INTEGER;     (*  Detail and block color, as in a NewWindow structure.  If *)
   blockcolor   *  : INTEGER;     (*  both are left zero, block pen will be set to 1. *)
   versionnumber*  : INTEGER;     (*  Make SURE this is set to zero. *)
   rfu1   *  : LONGINT;     (*  Make SURE you leave these two zeroed also. *)
   rfu2   *  : LONGINT;     (*  Make SURE you leave these two zeroed also. *)
END;

(* NOTE:
    The  control  values  mentioned above are used if you choose to insert
printf  style directives in your strings and should contain the address of
a list of control parameters, usually on the stack.
    *)

(*          Remember,   if  you  don't  want  to  go  through  the  hassle  of *)
(*  initializing a ExtendedColorRequester structure, you can always just call  *)
(*  ColorRequester  (as opposed to ExtendedColorRequester).  ColorRequester    *)
(*  just  takes  a  single  parameter, in D0, the color that should start out  *)
(*  being highlit.  It returns a single value, the color that was selected at  *)
(*  the end. *)

(*          This structure is for use with the ExtendedColorRequester (_not_,  *)
(*  the ColorRequester) function. *)

TYPE ExtendedColorRequesterPtr * = POINTER TO ExtendedColorRequester;

TYPE ExtendedColorRequester * = STRUCT
   defcolor     *  : LONGINT;     (* The color that is initially highlit.    *)
   window       *  : I.WindowPtr; (* The window the 'requester' opens up in. *)
   rfu1         *  : LONGINT;     (* Who knows what these will be used for,  *)
   rfu2         *  : LONGINT;     (* but I'm sure we'll think of something.  *)
   rfu3         *  : LONGINT;     (* Until then, just keep these zeroed.     *)
   rfu4         *  : LONGINT;     (* Okay? *)
   rfu5         *  : LONGINT;
END;

(*          This structure is for use with the GetLong function. *)

CONST glnodefault * = 0;(* Set this bit in the flags if you don't want a default*)
                        (* value to show up in the get long string gadget.  For *)
                        (* some things this is much better than having a zero   *)
                        (* show up. *)

TYPE GetLongStructPtr * = POINTER TO GetLongStruct;

TYPE GetLongStruct * = STRUCT
   titlebar     *  : E.ADDRESS;
   defaultval   *  : LONGINT;
   minlimit     *  : LONGINT;
   maxlimit     *  : LONGINT;
   result       *  : LONGINT;
   window       *  : I.WindowPtr;
   versionnumber*  : INTEGER; (*  Make SURE this is set to zero. *)
   flags        *  : LONGSET; (*  Some, uh flags.  See above for bit definitions. *)
   rfu2         *  : LONGINT; (*  Make SURE you leave these two zeroed also. *)
END;

TYPE GetStringStructPtr * = POINTER TO GetStringStruct;

TYPE GetStringStruct * = STRUCT
   titlebar     *  : E.ADDRESS;
   stringbuffer *  : E.ADDRESS;
   window       *  : I.WindowPtr;
   stringsize   *  : INTEGER; (* how many characters in the buffer *)
   visiblesize  *  : INTEGER; (* how many characters show on screen *)
   versionnumber*  : INTEGER;
   flags    *   : LONGSET;
   rfu1     *   : LONGINT;
   rfu2     *   : LONGINT;
   rfu3     *   : LONGINT;
END;

CONST dsize      * = 130;
      fchars     * =  30;
      wildlength * =  30;

TYPE DirString      * = ARRAY dsize+1 OF CHAR;
     DirStringPtr   * = POINTER TO DirString;
     FileString     * = ARRAY fchars+1 OF CHAR;
     FileStringPtr  * = POINTER TO FileString;
     PathString     * = ARRAY dsize+fchars+2 OF CHAR;
     PathStringPtr  * = POINTER TO PathString;
     ShowHideString * = ARRAY wildlength + 2 OF CHAR;
(*
   These  are  the  bit  numbers  of various bits that you can set in the
Flags  field of the file requester structure.  The actual values that come
out of these bit numbers are defined in the set of includes below with the
same  names  but  with  an  M  (for  mask) ending instead of a B (for bit)
ending.
   *)

CONST
     showinfo   * = 0;  (* Set this in Flags if you want .info files to show.  They default to hidden. *)
     extselect  * = 1;  (* Set this in Flags if you want extended select.  Default is not. *)
     caching    * = 2;  (* Set this in Flags if you want directory caching.  Default is not. *)
     getfonts   * = 3;  (* Set this in Flags if you want a font requester rather than a file requester. *)
     infogadget * = 4;  (* Set this in Flags if you want a hide-info files gadget. *)
     hidewilds  * = 5;  (* Set this in Flags if you DON'T want 'show' and 'hide' string gadgets. *)
     absolutexy * = 6;  (* Use absolute x,y positions rather than centering on mouse. *)
     cachepurge * = 7;  (* Purge the cache whenever the directory date stamp changes if this is set. *)
     nohalfcache* = 8;  (* Don't cache a directory unless it is completely read in when this is set. *)
     nosort     * = 9;  (* Set this in Flags if you DON'T want sorted directories. *)
     nodrag     * = 10; (* Set this in Flags if you DON'T want a drag bar and depth gadgets. *)
     saving     * = 11; (* Set this bit if you are selecting a file to save to. *)
     loading    * = 12; (* Set this bit if you are selecting a file(s) to load from. *)
                        (* These two bits (save and load) aren't currently used for*)
                        (* anything, but they may be in the future, so you should*)
                        (* remember to set them.  Also, these bits make it easier if*)
                        (* somebody wants to customize the file requester for their*)
                        (* machine.  They can make it behave differently for loading*)
                        (* vs saving. *)
     dironly    * = 13; (* Allow the user to select a directory, rather than a file. *)

TYPE ESStructurePtr  * = POINTER TO ESStructure;

TYPE ESStructure * = STRUCT
   nextFile     *  : ESStructurePtr;
   nameLength   *  : INTEGER; (* Length of the filename field, not including the terminating zero. *)
   pad          *  : INTEGER;
   node         *  : E.ADDRESS;     (* For internal use only. *)
   thefilename  *  : ARRAY 256 OF CHAR; (* This is (was) a variable size field. *)
END;

TYPE FileRequesterPtr *  = POINTER TO FileRequester;

TYPE FileRequester * = STRUCT
   versionNumber  *  : INTEGER;  (* Make sure this is zeroed for now. *)
   title        *  : E.ADDRESS;  (* Hailing text *)
   dir          *  : DirStringPtr;  (* Directory array (DSIZE+1) *)
   file         *  : FileStringPtr; (* Filename array (FCHARS+1) *)
   pathName     *  : PathStringPtr; (* Complete path name array (DSIZE+FCHARS+2) *)
   window       *  : I.WindowPtr;   (* Window requesting or NULL *)

   maxExtendedSelect *  : INTEGER;  (* Zero implies a maximum of 65535, as long as FRQEXTSELECT is set. *)
   numlines          *  : INTEGER;  (* Number of lines in file window. *)
   numcolumns        *  : INTEGER;  (* Number of columns in file window. *)
   devcolumns        *  : INTEGER;
   flags             *  : LONGSET;  (* Various - umm - flags.  See above for more info. *)
   dirnamescolor     *  : INTEGER;  (* These five colors will all default *)
   filenamescolor    *  : INTEGER;  (* to color one if you don't specify *)
   devicenamescolor  *  : INTEGER;  (* a color (ie; if you specify color zero). *)
   fontnamescolor    *  : INTEGER;  (* If you want color zero to be used, specify *)
   fontsizescolor    *  : INTEGER;  (* color 32, or some other too large number *)
                                    (* which mods down to zero. *)
   detailcolor       *  : INTEGER;  (* If both of these colors are specified as *)
   blockcolor        *  : INTEGER;  (* zero then the block pen will be set to one. *)

   gadgettextcolor     *  : INTEGER;   (* The color for the text of the five boolean gadgets.  Defaults to 1. *)
   textmessagecolor    *  : INTEGER;   (* The color for the message at the screen top.  Defaults to 1. *)
   stringnamecolor     *  : INTEGER;   (* The color for the words Drawer, File, Hide and Show.  Defaults to 3. *)
   stringgadgetcolor   *  : INTEGER;   (* The color for the borders of the string gadgets.  Defaults to 3. *)
                                       (* Unfortunately it is not possible to specify *)
                                       (* the color of the actual text in an Intuition *)
                                       (* string gadget. *)
   boxbordercolor      *  : INTEGER;   (* The color for the boxes around the file and directory areas.  Defaults to 3. *)
   gadgetboxcolor      *  : INTEGER;   (* The color for the boxes around the five boolean gadgets.  Defaults to 3. *)

   stuff               *  : ARRAY 18 OF INTEGER; (* This area, which is reserved for *)
                                                 (* future use, should all be zero. *)

   dirDateStamp        *  : Dos.Date; (* A copy of the cached directories date stamp. *)
                                      (* There should never be any need to change this. *)

   windowLeftEdge      *  : INTEGER;  (* These two fields are only used when the *)
   windowTopEdge       *  : INTEGER;  (* FRQABSOLUTEXY flag is set.  They specify *)
                                      (* the location of the upper left hand *)
                                      (* corner of the window. *)

   fontYSize        *  : INTEGER;     (* These fields are used to return the selected *)
   fontStyle        *  : INTEGER;     (* font size and style, only applicable when the *)
                                      (* font bit is set. *)

   (*If you set the extended select bit and the user extended selects, the list of filenames will start from here. *)
   extendedSelect       *  : ESStructurePtr;
   hide                 *  : ShowHideString; (* The wildcards text. *)
   show                 *  : ShowHideString; (* More wildcards text. *)
   fileBufferPos        *  : INTEGER; (* Various fields taken from the various *)
   fileDispPos          *  : INTEGER; (* string gadgets so that the cursor *)
   dirBufferPos         *  : INTEGER; (* can be returned to the same position *)
   dirDispPos           *  : INTEGER; (* on subsequent calls. *)
   hideBufferPos        *  : INTEGER;
   hideDispPos          *  : INTEGER;
   showBufferPos        *  : INTEGER;
   showDispPos          *  : INTEGER;

(*      The  following  fields are PRIVATE!  Don't go messing with them or
  wierd  things may/will happen.  If this isn't enough of a warning, go read
  the one in intuition.h, that should scare you off.   *)

   memory  : E.ADDRESS;    (* Memory allocate for dir entries.    *)
   memory2 : E.ADDRESS;    (* More memory, used for hidden files. *)
   lock    : E.ADDRESS;
   privateDirBuffer : ARRAY dsize+2 OF CHAR; (* Used for keeping a record of which *)
                                             (* directory we have file names for. *)
   fileInfoBlock : Dos.FileInfoBlock;
   numEntries        : INTEGER;
   numHiddenEntries  : INTEGER;
   filestartnumber   : INTEGER;
   devicestartnumber : INTEGER;
END;

(*        This is used with the RealTimeScroll function. *)

TYPE ScrollStructPtr * = POINTER TO ScrollStruct;

TYPE ScrollStruct * = STRUCT
   topEntryNumber *  : LONGINT; (* This is the ordinal number of the first*)
                                (* displayed entry. *)
   numEntries     *  : LONGINT; (* This is the total number of entries in*)
                                (* the list. *)
   lineSpacing    *  : INTEGER; (* This is how many pixels high each entry is. *)
   numLines       *  : LONGINT; (* This is how many entries can be displayed simultaneously. *)
   propGadget     *  : I.Gadget;(* This is a pointer to the prop gadget being monitored. *)

   RedrawAll * : PROCEDURE(); (* This routine is used to redraw all of the*)
                              (* entries when the user moves far enough*)
                              (* that scrolling will take too long. *)

   ReadMore  * : PROCEDURE(); (* An optional routine that is called when*)
                              (* the scroll routine is waiting for movement. *)
                              (* This allows reading of new data while real*)
                              (* time scrolling. *)
   ScrollAndDraw *  : PROCEDURE(); (* This routine is called when the data needs*)
                              (* to be scrolled and updated.  This routine is*)
                              (* passed four long parameters (on the stack and*)
                              (* in D0-D3) which are, respectively:*)
                              (* D0 - entry number of first line to be drawn. *)
                              (* D1 - pixel offset to draw first line at. *)
                              (* D2 - amount to scroll before doing any drawing. *)
                              (* D3 - number of lines of data to draw. *)
   versionnumber * : INTEGER; (* Make SURE this is set to zero. *)
   rfu1     *  : LONGINT;     (* Make SURE you leave these two zeroed also. *)
   rfu2     *  : LONGINT;     (* Make SURE you leave these two zeroed also. *)
END;

TYPE ChipstuffPtr *  = POINTER TO Chipstuff;

TYPE Chipstuff * = STRUCT
   arrowUp    *  : ARRAY 10 OF INTEGER; (* 16 pixels wide, 10 pixels high. *)
   arrowDown  *  : ARRAY 10 OF INTEGER; (* 16 pixels wide, 10 pixels high. *)
   arrowLeft  *  : ARRAY  9 OF INTEGER; (* 16 pixels wide, 9 pixels high.  *)
   arrowRight *  : ARRAY  9 OF INTEGER; (* 16 pixels wide, 9 pixels high.  *)
   letterR    *  : ARRAY 10 OF INTEGER; (* 16 pixels wide, 10 pixels high. *)
   letterG    *  : ARRAY 10 OF INTEGER; (* 16 pixels wide, 10 pixels high. *)
   letterB    *  : ARRAY 10 OF INTEGER; (* 16 pixels wide, 10 pixels high. *)
   letterH    *  : ARRAY 10 OF INTEGER; (* 16 pixels wide, 10 pixels high. *)
   letterS    *  : ARRAY 10 OF INTEGER; (* 16 pixels wide, 10 pixels high. *)
   letterV    *  : ARRAY 10 OF INTEGER; (* 16 pixels wide, 10 pixels high. *)
END;

TYPE ReqLibPtr *  = POINTER TO  ReqLib;

TYPE ReqLib * = STRUCT
   rLib          *  : E.LibraryPtr;
   sysLib        *  : E.ExecBasePtr;
   dosLib        *  : Dos.DosLibraryPtr;
   intuiLib      *  : I.IntuitionBasePtr;
   gfxLib        *  : G.GfxBasePtr;
   segList       *  : E.BPTR;
   images        *  : ChipstuffPtr;
   flags         *  : SET;
   pad           *  : SET;
   consoleDev    *  : E.ADDRESS;   (* Not really a char, but it should work *)
   consoleHandle *  : E.IOStdReqPtr;
END;

VAR   req * : ReqLibPtr;

PROCEDURE Center* {req,-30}(VAR nw{8}:I.NewWindow;x{0},y{1}:INTEGER);

PROCEDURE SetSize* {req,-36}(maxValue{0},viewSize{1}:LONGINT): INTEGER;

PROCEDURE SetLocation* {req,-42}(maxValue{0},viewSize{1},value{2}:LONGINT): INTEGER;

PROCEDURE ReadLocation* {req,-48}(maxValue{0},viewSize{1}:LONGINT;potValue{2}:INTEGER): LONGINT;

PROCEDURE Format* {req,-54}(buffer{10},string{8}: ARRAY OF CHAR;VAR values{9}:E.ADDRESS);

PROCEDURE FileRequest* {req,-84}(filereq{8} :FileRequesterPtr): BOOLEAN;

PROCEDURE ColorRequester* {req,-90}(desiredColor{0}:LONGINT): LONGINT;

PROCEDURE DrawBox* {req,-96}(rp{9}: G.RastPortPtr;minX{0},minY{1},maxX{2},maxY{3}:LONGINT);

PROCEDURE MakeButton* {req,-102}(buffer{8}:TwoImageGadgetPtr;image{9},image2{10}:E.ADDRESS;width{0},heigth{1},depth{2}:LONGINT);

PROCEDURE MakeScrollBar* {req,-108}(buffer{8}:ScrollBlockPtr;flags{0}:LONGSET;size{1},x{2},y{3}:LONGINT);

PROCEDURE PurgeFiles* {req,-114}(filereq{8}:FileRequesterPtr);

PROCEDURE MakeGadget* {req,-126}(buffer{8}:GadgetBlockPtr;string{9}: ARRAY OF CHAR;x{0},y{1}:LONGINT);

PROCEDURE MakeString* {req,-132}(buffer{8}:StringBlockPtr;stringbuff{9},undobuff{10}:ARRAY OF CHAR;maxwidthbits{0},maxnumchars{1},x{2},y{3}:LONGINT);

PROCEDURE MakeProp* {req,-138}(buffer{8}:PropBlockPtr;width{0},height{1}:LONGINT;flags{2}:LONGSET);

PROCEDURE LinkGadget* {req,-144}(buffer{8}:GadgetBlockPtr;string{9}: ARRAY OF CHAR;VAR nw{11}:I.NewWindow;x{0},y{1}:LONGINT);

PROCEDURE LinkStringGadget* {req,-150}(buffer{8}:StringBlockPtr;stringbuff{9},undobuff{10}:ARRAY OF CHAR;VAR nw{11}:I.NewWindow;widthbits{0},
                           numchars{1},x{2},y{3}:LONGINT);
PROCEDURE LinkPropGadget* {req,-156}(buffer{8}:PropBlockPtr;VAR nw{11}:I.NewWindow;width{0},height{1}:LONGINT;flags{2}:LONGSET;leftEdge{3},topEdge{4}:LONGINT);

PROCEDURE GetString* {req,-162}(buffer{8}:ARRAY OF CHAR;title{9}:ARRAY OF CHAR;window{10}:I.WindowPtr;visibelchars{0},maxchars{1}:LONGINT):BOOLEAN;

PROCEDURE RealTimeScroll* {req,-168}(scrollstruct{8}:ScrollStructPtr);

PROCEDURE TextRequest* {req,-174}(trs{8}:TRStructurePtr):INTEGER;

PROCEDURE GetLong* {req,-180}(gl{8}:GetLongStructPtr):BOOLEAN;

PROCEDURE RawKeyToAscii* {req,-186}(code{0},qualifier{1}:INTEGER;iaddress{8}:E.ADDRESS);

PROCEDURE ExtendedColorRequest* {req,-192}(extcolreq{8}:ExtendedColorRequesterPtr):LONGINT;

PROCEDURE NewGetString* {req,-198}(ngetstr{8}:GetStringStructPtr):BOOLEAN;

PROCEDURE SimpleRequest* (string:ARRAY OF CHAR;parameterlist:E.ADDRESS);

VAR textRPtr:TRStructurePtr;

BEGIN

   NEW(textRPtr);
   textRPtr.text := SYSTEM.ADR(string);
   textRPtr.controls := parameterlist;
   textRPtr.window := NIL;
   textRPtr.middleText := NIL;
   textRPtr.positiveText := NIL;
   textRPtr.negativeText := SYSTEM.ADR("Resume");
   textRPtr.title := SYSTEM.ADR("okay?");
   textRPtr.keyMask := {0..15};
   textRPtr.textcolor := 1;
   textRPtr.detailcolor := 0;
   textRPtr.blockcolor := 0;
   textRPtr.versionnumber := 0;
   textRPtr.rfu1 := 0;
   textRPtr.rfu2 := 0;
   IF TextRequest(textRPtr)=0 THEN END;
   DISPOSE(textRPtr);
   
END SimpleRequest;

PROCEDURE TwoGadRequest* (string:ARRAY OF CHAR;parameterlist:E.ADDRESS):INTEGER;

VAR textRPtr:TRStructurePtr;
    result:INTEGER; 
BEGIN

   NEW(textRPtr);
   textRPtr.text := SYSTEM.ADR(string);
   textRPtr.controls := parameterlist;
   textRPtr.window := NIL;
   textRPtr.middleText := NIL;
   textRPtr.positiveText := SYSTEM.ADR("  Ok  ");
   textRPtr.negativeText := SYSTEM.ADR("Cancel");
   textRPtr.title := SYSTEM.ADR("okay?");
   textRPtr.keyMask := {0..15};
   textRPtr.textcolor := 1;
   textRPtr.detailcolor := 0;
   textRPtr.blockcolor := 0;
   textRPtr.versionnumber := 0;
   textRPtr.rfu1 := 0;
   textRPtr.rfu2 := 0;
   result := TextRequest(textRPtr);
   DISPOSE(textRPtr);
   RETURN result;
   
END TwoGadRequest;

(* $OvflChk- $RangeChk- $StackChk- $NilChk- $ReturnChk- $CaseChk- *)

BEGIN

 req :=  SYSTEM.VAL(ReqLibPtr,E.OpenLibrary(reqName,reqVersion));
 IF req = NIL THEN
   SYSTEM.SETREG(0,I.DisplayAlert(0,"\x00\x84\x14Missing req.library\o\o",50));
   HALT(0);
 END;

CLOSE

 IF req#NIL THEN E.CloseLibrary(SYSTEM.VAL(E.LibraryPtr,req)) END;
 
END Req.
