(* ------------------------------------------------------------------------
  :Program.     PowerPacker
  :Contents.    Interface to "powerpacker.library"
  :Author.      Kai Bolay [kai]
  :Address.     Hoffmannstrae 168
  :Address.     D-7250 Leonberg
  :History.     v1.0 [kai] 03-Jan-91 Now it works :-)
  :Copyright.   PD
  :Language.    Oberon
  :Translator.  AMIGA OBERON v1.17.1
------------------------------------------------------------------------ *)
MODULE PowerPacker;

(*
**      Filename: libraries/ppbase.h
**      Release: 1.0
**
**      (C) Copyright 1990 Nico Franois
**          All Rights Reserved
*)

IMPORT
   s: SYSTEM, e: Exec, I: Intuition;
CONST
   PPName* = "powerpacker.library";
   PPVersion* = 33;
   (* decrunch colors for LoadData and DecrunchBuffer *)
   DecrCol0*  = 0;
   DecrCol1*  = 1;
   DecrPoint* = 2;
   DecrScrol* = 3;
   DecrNone*  = 4;
   (* error codes returned by LoadData *)
   LoadOk*    =  0;
   OpenErr*   = -1;
   ReadErr*   = -2;
   NoMemory*  = -3;
   Crypted*   = -4;
   PassErr*   = -5;
   UnknownPP* = -6;
TYPE
   PPBase* = STRUCT
                LibNode* : e.Library;
                flags* : SHORTSET;
                pad* : SHORTINT;
                segList* : LONGINT; (* BPTR *)
             END; (* STRUCT *)
   PPBasePtr* = POINTER TO PPBase;
   PassFunc* = PROCEDURE (checksum: INTEGER; password: e.ADDRESS): BOOLEAN;
   (** Must be of type "DeallocPars" and "SaveAllRegs" !! **)
VAR
   pp* : PPBasePtr;

PROCEDURE LoadData* {pp, -30} (filename{8}: ARRAY OF CHAR; col{0}: LONGINT;
                               memtype{1}: LONGSET; VAR buffer{9}: e.ADDRESS;
                               VAR len{10}: LONGINT; func{11}: PassFunc) : LONGINT;
PROCEDURE DecrunchBuffer* {pp, -36} (endcrun{8}, decrbuf{9}, effptr{10}: e.ADDRESS;
                                     col{0}: LONGINT);
PROCEDURE CalcCheckSum* {pp, -42} (string{8}: ARRAY OF CHAR): INTEGER;
PROCEDURE CalcPasskey* {pp, -48} (password{8}: ARRAY OF CHAR) : LONGINT;
PROCEDURE Decrypt* {pp, -54} (buffer{8}: e.ADDRESS; len{0}, key{1}: LONGINT);
PROCEDURE GetPassword* {pp, -60} (screen{8}: I.ScreenPtr; buffer{9}: e.ADDRESS;
                                  maxchars{0}: LONGINT; checksum{1}: INTEGER)
                                  : BOOLEAN;
PROCEDURE GetString* {pp, -66} (screen{8}: I.ScreenPtr; buffer{9}: e.ADDRESS;
                                maxchar{0}: LONGINT; title{10}: ARRAY OF CHAR)
                                : BOOLEAN;
PROCEDURE GetLong* {pp, -72} (screen{8}: I.ScreenPtr; VAR longvar{9}: LONGINT;
                              title{10}: ARRAY OF CHAR) : BOOLEAN;
BEGIN
   pp := s.VAL (PPBasePtr, e.OpenLibrary (PPName, PPVersion));
   IF pp = NIL THEN
      IF I.DisplayAlert (0, "\x00\x64\x14missing powerpacker.library V33\o\o",
                         50) THEN END;
      HALT (20)
   END; (* IF *)
CLOSE
   IF pp # NIL THEN
      e.CloseLibrary (s.VAL (e.LibraryPtr, pp))
   END; (* IF *)
END PowerPacker.
