(**********************************************************************

:Program.    SortLibTest.mod
:Contens.    Ein Programm zum Testen des Moduls SortLib.
:Author.     Bernd Braun
:Address.    Lippestr. 11, D-3300 Braunschweig
:Phone.      0531/845498
:Copyright.  Public Domain
:Language.   Modula-2
:Translator. M2Amiga A+L V3.32d
:Imports.    NewInOut, SortLib
:History.    V1.0 24.Nov.1990

***********************************************************************)

MODULE SortLibTest;

(* $R- $S- $V- $F- *)

   FROM ASCII IMPORT
      nul;
   FROM Dos IMPORT
      DatePtr , DateStamp;
   FROM NewInOut IMPORT
      WriteString, WriteLn, WriteInt;
   FROM RandomNumber IMPORT
      RND, PutSeed;
   FROM SortLib IMPORT
      HeapSort, QuickSort;
   FROM Str IMPORT
      Compare, Copy;
   FROM SYSTEM IMPORT
      ADR, ADDRESS, BYTE, CAST, TSIZE;

   CONST
      max      = 1000;
      maxstr   = 10;

   TYPE
      String   = ARRAY [ 0 .. maxstr - 1 ] OF CHAR;

   VAR
      Feld1,
      Feld2    : ARRAY [ 0 .. max - 1 ] OF INTEGER;
      Strings1,
      Strings2 : ARRAY [ 0 .. max - 1 ] OF String;
      n, m, x  : INTEGER;
      Str      : String;
      c        : CHAR;
      Ticks    : LONGINT;

   (* Rckgabe der seit Mitternacht vergangenen Sekunden. *)
   PROCEDURE GetTime () : LONGINT;
      VAR
         time : ARRAY [ 0 .. 2 ] OF LONGINT;
   BEGIN
      DateStamp ( CAST ( DatePtr , ADR ( time ) ) );
      (* In time [ 0 ] steht die Anzahl der seit 1.Jan 1977
         vergangenen Tage. *)
      RETURN 60 * time [ 1 ] + time [ 2 ] DIV 50;
   END GetTime;

   (* Kopiert Integer sour nach Integer dest. *)
   PROCEDURE CopyInt ( dest, sour : ADDRESS );
      VAR
         a, b : POINTER TO INTEGER;
   BEGIN
      a := sour;
      b := dest;
      b ^ := a ^;
   END CopyInt;

   (* Kopiert String sour nach String dest. *)
   PROCEDURE CopyStr ( dest, sour : ADDRESS );
      VAR
         n    : LONGINT;
         x, y : POINTER TO String;
   BEGIN
      x := sour;
      y := dest;
      Copy ( y ^, x ^ );
   END CopyStr;

   (* Vergleicht zwei Integer-Zahlen. *)
   PROCEDURE CompInt ( a, b : ADDRESS ) : INTEGER;
      VAR
         x, y : POINTER TO INTEGER;
   BEGIN
      x := a;
      y := b;
      RETURN x ^ - y ^;
   END CompInt;

   (* Vergleicht zwei Strings. *)
   PROCEDURE CompStr ( a, b : ADDRESS ) : INTEGER;
      VAR
         x, y : POINTER TO String;
   BEGIN
      x := a;
      y := b;
      RETURN Compare ( x^, y^ );
   END CompStr;

BEGIN
   WriteString ( "Testprogramm zum Modul SortLib." );
   WriteLn;
   WriteLn;
   WriteString ( "Erzeuge " ); WriteInt ( max, 1 );
   WriteString ( " zufllige Integerzahlen." );
   WriteLn;
   WriteLn;
   PutSeed ( GetTime () );
   FOR n := 0 TO max - 1 DO
      x := RND ( max ) - max DIV 2;
      Feld1 [ n ] := x;
      Feld2 [ n ] := x;
      WriteInt ( x, 11 );
   END;
   WriteLn;
   WriteLn;
   WriteString ( "Sortiere mit HeapSort." );
   WriteLn;
   Ticks := GetTime ();

   HeapSort ( Feld1, CompInt, CopyInt, max, TSIZE ( INTEGER ) );

   WriteString ( "Fertig! " );
   WriteInt ( GetTime () - Ticks, 1 );
   WriteString ( " sec" );
   WriteLn;
   WriteLn;
   FOR n := 0 TO max - 1 DO
      WriteInt ( Feld1 [ n ], 11 );
   END;
   WriteLn;
   WriteLn;
   WriteString ( "Sortiere mit QuickSort." );
   WriteLn;
   Ticks := GetTime ();

   QuickSort ( Feld2, CompInt, CopyInt, max, TSIZE ( INTEGER ) );

   WriteString ( "Fertig! " );
   WriteInt ( GetTime () - Ticks, 1 );
   WriteString ( " sec" );
   WriteLn;
   WriteLn;
   FOR n := 0 TO max - 1 DO
      WriteInt ( Feld2 [ n ], 11 );
   END;
   WriteLn;
   WriteLn;
   WriteString ( "Erzeuge " ); WriteInt ( max, 1 );
   WriteString ( " zufllige Strings." );
   WriteLn;
   WriteLn;
   FOR n := 0 TO max - 1 DO
      x := RND ( maxstr - 4 ) + 4;
      FOR m := 0 TO x - 1 DO
         c := CHR ( RND ( 26 ) + ORD ( "A" ) );
         Strings1 [ n, m ] := c;
         Strings2 [ n, m ] := c;
      END;
      Strings1 [ n, x ] := nul;
      Strings2 [ n, x ] := nul;
      WriteString ( Strings1 [ n ] );
      WriteString ( " " );
   END;
   WriteLn;
   WriteLn;
   WriteString ( "Sortiere mit HeapSort." );
   WriteLn;
   Ticks := GetTime ();

   HeapSort ( Strings1, CompStr, CopyStr, max, maxstr );

   WriteString ( "Fertig! " );
   WriteInt ( GetTime () - Ticks, 1 );
   WriteString ( " sec" );
   WriteLn;
   WriteLn;
   FOR n := 0 TO max - 1 DO
      WriteString ( Strings1 [ n ] );
      WriteString ( " " );
   END;
   WriteLn;
   WriteLn;
   WriteString ( "Sortiere mit QuickSort." );
   WriteLn;
   Ticks := GetTime ();

   QuickSort ( Strings2, CompStr, CopyStr, max, maxstr );

   WriteString ( "Fertig! " );
   WriteInt ( GetTime () - Ticks, 1 );
   WriteString ( " sec" );
   WriteLn;
   WriteLn;
   FOR n := 0 TO max - 1 DO
      WriteString ( Strings2 [ n ] );
      WriteString ( " " );
   END;
   WriteLn;
END SortLibTest.
