(*---------------------------------------------------------------------------
    :Program.    SaveIFF.mod
    :Author.     Fridtjof Siebert
    :Address.    Nobileweg 67, D-7-Stgt-40
    :Copyright.  PD
    :Language.   Oberon
    :Translator. Amiga Oberon
    :Imports.    IFFSupport 1.6
    :History.    V1.0, 26-Jun-88: original M2 Version by [fbs]
    :History.    V1.1, 25-Jan-91: ported this to Oberon  [fbs]
    :History.    V1.2, 25-Jan-91: 2.0-like design        [fbs]
    :Contents.   Speichert Screens und Windows als IFF-Files.
    :Remark.     Demonstartion fr IFFSupport 1.6.
---------------------------------------------------------------------------*)

MODULE SaveIFF;

IMPORT rq : Requests,
       m  : Mouse,
       e  : Exec,
       g  : Graphics,
       I  : Intuition,
       str: Strings,
       iff: IFFSupport,
       y  : SYSTEM;

CONST
(* Gadgets *)
  scrn0    = 0;
  scrn1    = 1;
  scrn2    = 2;
  scrn3    = 3;
  scrn4    = 4;
  scrn5    = 5;
  scrn6    = 6;
  scrn7    = 7;
  scrn8    = 8;
  scrn9    = 9;
  wind0    = 10;
  wind1    = 11;
  wind2    = 12;
  wind3    = 13;
  wind4    = 14;
  wind5    = 15;
  wind6    = 16;
  wind7    = 17;
  wind8    = 18;
  wind9    = 19;
  name     = 20;
  savescrn = 21;
  savewind = 22;
  savegzz  = 23;
  showiff  = 24;
  dummy    = 25;
  numGadgs = 26;

VAR
  NuWindow: I.NewWindow;
  Window: I.WindowPtr;                       (* SaveIFF's Window           *)
  RP: g.RastPortPtr;                         (* It's RastPort              *)
  Gadgs: ARRAY numGadgs OF I.Gadget;         (* It's Gadgets               *)
  NameInfo: I.StringInfo;                    (* IFF-Name's Gadget's Info   *)
  Name: ARRAY 80 OF CHAR;                    (* IFF-Name                   *)
  IDCount: INTEGER;                          (* Counting Gadgets           *)
  ChosenScreen, ChosenWindow: INTEGER;       (* User-Selected Screnn&Window*)
  Screen: I.ScreenPtr;                       (* Screen for Loaded IFF-File *)
  DummyWind: I.WindowPtr;                    (* only a Dummy               *)
  Screens: ARRAY 10 OF I.ScreenPtr;          (* ScreenPtrs                 *)
  Windows: ARRAY 10 OF I.WindowPtr;          (* WindowPtrs                 *)
  NumScreens, NumWindows: INTEGER;           (* How many are in that List? *)
  gadget: I.GadgetPtr;                       (* Gadget causing a Message   *)
  Msg: I.IntuiMessagePtr;                    (* Receives Messages          *)
  Rect: g.Rectangle;                         (* Rectangle for Windows      *)
  font: g.TextFontPtr;
  textattr: g.TextAttr;
  top,left: INTEGER;

(*-----------------------  Small Procedures:  -----------------------------*)

(*------  Set a Bool-Gadget:  ------*)

PROCEDURE SetBool(VAR Gadg: I.Gadget; x,y,w,h: INTEGER);

BEGIN
  Gadg.nextGadget := NIL;
  Gadg.leftEdge := x;  Gadg.topEdge := y;
  Gadg.width    := w;  Gadg.height  := h;
  Gadg.flags    := {};
  Gadg.activation   := {I.gadgImmediate,I.toggleSelect};
  Gadg.gadgetType   := I.boolGadget;
  Gadg.gadgetRender := NIL;
  Gadg.selectRender := NIL;
  Gadg.gadgetText   := NIL;
  Gadg.mutualExclude:= LONGSET{};
  Gadg.specialInfo  := NIL;
  Gadg.gadgetID     := 0;
  Gadg.userData     := NIL;
END SetBool;

(*------  Draw A Box:  ------*)

PROCEDURE Box(rp: g.RastPortPtr; x,y,X,Y: INTEGER);
BEGIN
  DEC(X,x); DEC(Y,y);
  g.SetAPen(rp,2); g.SetDrMd(rp,g.jam1);
  g.Move(rp,x+X-1,y);
  g.Draw(rp,x,y);
  g.Draw(rp,x,y+Y);
  g.Move(rp,x+1,y);
  g.Draw(rp,x+1,y+Y-1);
  g.SetAPen(rp,1);
  g.Move(rp,x+1,y+Y);
  g.Draw(rp,x+X,y+Y);
  g.Draw(rp,x+X,y);
  g.Move(rp,x+X-1,y+1);
  g.Draw(rp,x+X-1,y+Y);
END Box;


(*------  Type Text:  ------*)

PROCEDURE Type(rp: g.RastPortPtr; x,y: INTEGER; text: ARRAY OF CHAR); (* $CopyArrays- *)

BEGIN
 g.Move(rp,x,y); g.Text(rp,text,str.Length(text));
END Type;

(*-------------------------------------------------------------------------*)
(*                                                                         *)
(*                      Refresh Display: (Gadgets & Names)                 *)
(*                                                                         *)
(*-------------------------------------------------------------------------*)

PROCEDURE Refresh(Display: BOOLEAN);
(* IF NOT(Display) THEN Don't make anything affecting the display          *)

VAR
  SearchScreen: I.ScreenPtr;
  SearchWindow: I.WindowPtr;
  SearchName: ARRAY 80 OF CHAR;
  NamePtr: POINTER TO ARRAY 256 OF CHAR;

BEGIN

(*------  Delete highlighted Gadgets:  ------*)

  IF Display THEN I.RefreshGadgets(y.ADR(Gadgs),Window,NIL) END;

(*------  Get ScreenNames:  ------*)

  IF Display THEN
    g.SetAPen(RP,0); g.SetDrMd(RP,g.jam1); g.RectFill(RP,left+7,top+17,left+139,top+99);
    g.SetAPen(RP,1); g.SetBPen(RP,0); g.SetDrMd(RP,g.jam2);
  END;
  IDCount := scrn0;
  e.Forbid();
  SearchScreen := I.int.firstScreen;
  WHILE (SearchScreen#NIL) AND (IDCount<=scrn9) DO
    Screens[IDCount-scrn0] := SearchScreen;
    IF SearchScreen.title=NIL THEN
      SearchName := "Unnamed";
    ELSE
      NamePtr := SearchScreen.title;
      str.Cut(NamePtr^,0,16,SearchName);
    END;
    IF Display THEN Type(RP,left+9,top+25+8*IDCount,SearchName) END;
    INC(IDCount);
    SearchScreen := SearchScreen.nextScreen;
  END;
  e.Permit();
  NumScreens := IDCount;
  IF ChosenScreen>=NumScreens THEN ChosenScreen := scrn0 END;

(*------  Get WindowNames:  ------*)

  IF Display THEN
    g.SetAPen(RP,0); g.SetDrMd(RP,g.jam1); g.RectFill(RP,left+151,top+17,left+283,top+99);
    g.SetAPen(RP,1); g.SetBPen(RP,0); g.SetDrMd(RP,g.jam2);
  END;
  IDCount := wind0;
  e.Forbid();
  SearchWindow := Screens[ChosenScreen-scrn0].firstWindow;
  WHILE (SearchWindow#NIL) AND (IDCount<=wind9) DO
    Windows[IDCount-wind0] := SearchWindow;
    IF SearchWindow.title=NIL THEN
      SearchName := "Unnamed";
    ELSE
      NamePtr := SearchWindow.title;
      str.Cut(NamePtr^,0,16,SearchName);
    END;
    IF Display THEN Type(RP,left+153,top+8*IDCount-55,SearchName) END;
    INC(IDCount);
    SearchWindow := SearchWindow.nextWindow;
  END;
  e.Permit();
  NumWindows := IDCount;
  IF ChosenWindow>=NumWindows THEN
    IF NumWindows=wind0 THEN
      ChosenWindow := dummy;
    ELSE
      ChosenWindow := wind0;
    END;
  END;

(*------  Set Gadgets:  ------*)

  IF Display THEN
    IDCount := scrn0;
    WHILE IDCount<=showiff DO
      Gadgs[IDCount].flags := Gadgs[IDCount].flags - {I.selected};
      INC(IDCount);
    END;
    INCL(Gadgs[ChosenScreen].flags,I.selected);
    INCL(Gadgs[ChosenWindow].flags,I.selected);
  END;

(*------  Refresh:  ------*)

  IF Display THEN I.RefreshGadgets(y.ADR(Gadgs),Window,NIL) END;

END Refresh;

(*-------------------------------------------------------------------------*)
(*                                                                         *)
(*                             M A I N :                                   *)
(*                                                                         *)
(*-------------------------------------------------------------------------*)

BEGIN

(*------  Init Variables:  ------*)

  Window := NIL;
  Screen := NIL;
  Name := "df0:Pic.iff";
  ChosenScreen := scrn0;
  ChosenWindow := wind0;

(*------------------------  Build up Display:  ----------------------------*)

(*------  Window:  ------*)


  NuWindow.detailPen  := 0;    NuWindow.blockPen  := 1;
  NuWindow.idcmpFlags := LONGSET{I.gadgetDown,I.closeWindow};
  NuWindow.flags      := LONGSET{I.windowDrag,I.windowDepth,I.windowClose,I.activate,
                              I.noCareRefresh};
  NuWindow.firstGadget:= NIL;
  NuWindow.checkMark  := NIL;
  NuWindow.title      := y.ADR("SaveIFF -  F. Siebert");
  NuWindow.screen     := NIL;
  NuWindow.bitMap     := NIL;
  NuWindow.type       := {I.wbenchScreen};
  NuWindow.leftEdge   := 172;  NuWindow.topEdge   := 36;
  NuWindow.width      := 298;  NuWindow.height    := 165;

  LOOP
    Window := I.OpenWindow(NuWindow);
    rq.Assert(Window#NIL,"SaveIFF: Can't open Window");
    IF (Window.height-Window.borderTop -Window.borderBottom>=152) AND
       (Window.width -Window.borderLeft-Window.borderRight >=290) THEN EXIT END;
    NuWindow.height := 152+Window.borderTop +Window.borderBottom;
    NuWindow.width  := 290+Window.borderLeft+Window.borderRight;
    I.CloseWindow(Window);
  END;
  RP := Window.rPort;

  textattr.name := y.ADR("topaz.font");
  textattr.ySize := 8;
  textattr.style := SHORTSET{};
  textattr.flags := SHORTSET{g.romFont};
  font := g.OpenFont(y.ADR(textattr));
  IF font#NIL THEN g.SetFont(RP,font) END;

  top := Window.borderTop;
  left := Window.borderLeft;

(*------  Gadgets: ------*)

  IDCount:=scrn0;
  WHILE IDCount <= scrn9 DO
    SetBool(Gadgs[IDCount]      ,left+  8,top+19+8*IDCount,132,8);
    SetBool(Gadgs[IDCount+wind0],left+152,top+19+8*IDCount,132,8);
    INC(IDCount);
  END;
  SetBool(Gadgs[name    ], left+57,top+106,224, 8);
  Gadgs[name].activation  := {I.stringCenter};
  Gadgs[name].gadgetType  := I.strGadget;
  Gadgs[name].specialInfo := y.ADR(NameInfo);
  NameInfo.buffer := y.ADR(Name);
  NameInfo.undoBuffer := NIL;
  NameInfo.bufferPos  := 0;
  NameInfo.maxChars   := 80;
  NameInfo.dispPos    := 0;
  NameInfo.numChars   := str.Length(Name);
  SetBool(Gadgs[savescrn],left+  5,top+120,137,13);
  SetBool(Gadgs[savewind],left+149,top+120,137,13);
  SetBool(Gadgs[savegzz ],left+  5,top+136,137,13);
  SetBool(Gadgs[showiff ],left+149,top+136,137,13);

(*------  Link Gadgets:  ------*)

  IDCount := scrn0;
  WHILE IDCount <= savegzz DO
    Gadgs[IDCount].nextGadget := y.ADR(Gadgs[IDCount+1]);
    Gadgs[IDCount].gadgetID   := IDCount;
    INC(IDCount);
  END;
  Gadgs[showiff].nextGadget := NIL;
  Gadgs[showiff].gadgetID := showiff;
  IF I.AddGList(Window,y.ADR(Gadgs),-1,numGadgs,NIL)=0 THEN END;

(*------  Draw into Window:  ------*)

  Box(RP,left+  5,top+ 16,left+142,top+100); Box(RP,left+149,top+ 16,left+286,top+100);
  Box(RP,left+ 53,top+104,left+285,top+116); Box(RP,left+  5,top+120,left+141,top+132);
  Box(RP,left+149,top+120,left+285,top+132); Box(RP,left+  5,top+136,left+141,top+148);
  Box(RP,left+149,top+136,left+285,top+148);

(*------  Type Text into Window:  ------*)

  g.SetAPen(RP,1);
  Type(RP,left+  8,top+ 11,"Screens:");
  Type(RP,left+152,top+ 11,"Windows:");
  Type(RP,left+  8,top+113,"Name:");
  Type(RP,left+ 36,top+129,"Save Screen");
  Type(RP,left+180,top+129,"Save Window");
  Type(RP,left+ 28,top+145,"Save GimmeZZ");
  Type(RP,left+188,top+145,"Show IFF");

(*------  Initialize Display:  ------*)

  Refresh(TRUE);

(*----------------------------  Get Messages:  ----------------------------*)

  LOOP

    e.WaitPort(Window.userPort);
    Msg := e.GetMsg(Window.userPort);
    IF I.closeWindow IN Msg.class THEN
      e.ReplyMsg(Msg);
      EXIT;
    END;
    gadget := Msg.iAddress;
    e.ReplyMsg(Msg);

    IDCount := gadget.gadgetID;

    CASE IDCount OF

(*------  Screen-Gadget:  ------*)

    scrn0..scrn9:
      IF IDCount<NumScreens THEN
        ChosenScreen := IDCount;
      ELSE
        I.DisplayBeep(NIL);
      END; |

(*------  Window-Gadget:  ------*)

    wind0..wind9:
      IF IDCount<NumWindows THEN
        ChosenWindow := IDCount;
      ELSE
        I.DisplayBeep(NIL);
      END; |

(*------  SaveScrn:  ------*)

    savescrn:
      Refresh(FALSE);
      IF NOT(iff.WriteILBM(Name,
                y.ADR(Screens[ChosenScreen-scrn0].rastPort),
                y.ADR(Screens[ChosenScreen-scrn0].viewPort),
                NIL,TRUE)) THEN
        I.DisplayBeep(NIL);
      END; |

(*------  savewind:  ------*)

    savewind:
      Refresh(FALSE);
      IF ChosenWindow#dummy THEN
        Rect.minX := Windows[ChosenWindow-wind0].leftEdge;
        Rect.minY := Windows[ChosenWindow-wind0].topEdge;
        Rect.maxX := Rect.minX + Windows[ChosenWindow-wind0].width - 1;
        Rect.maxY := Rect.minY + Windows[ChosenWindow-wind0].height - 1;
        IF NOT(iff.WriteILBM(Name,
                  Windows[ChosenWindow-wind0].rPort,
                  y.ADR(Windows[ChosenWindow-wind0].wScreen.viewPort),
                  y.ADR(Rect),TRUE)) THEN
          I.DisplayBeep(NIL);
        END;
      ELSE
        I.DisplayBeep(NIL);
      END; |

(*------  Save GimmeZeroZero:  ------*)

    savegzz:
      Refresh(FALSE);
      IF ChosenWindow#dummy THEN
        Rect.minX := Windows[ChosenWindow-wind0].leftEdge + Windows[ChosenWindow-wind0].borderLeft;
        Rect.minY := Windows[ChosenWindow-wind0].topEdge  + Windows[ChosenWindow-wind0].borderTop;
        Rect.maxX := Rect.minX + Windows[ChosenWindow-wind0].gzzWidth - 1;
        Rect.maxY := Rect.minY + Windows[ChosenWindow-wind0].gzzHeight - 1;
        IF NOT(iff.WriteILBM(Name,
                  Windows[ChosenWindow-wind0].rPort,
                  y.ADR(Windows[ChosenWindow-wind0].wScreen.viewPort),
                  y.ADR(Rect),TRUE)) THEN
          I.DisplayBeep(NIL);
        END;
      ELSE
        I.DisplayBeep(NIL);
      END; |

(*------  ShowIFF:  ------*)

    showiff:
      IF iff.ReadILBM(Name,{iff.front,iff.visible},Screen,DummyWind) THEN
         m.WaitLMB;
         I.CloseScreen(Screen);
         Screen := NIL;
      ELSE
        I.DisplayBeep(NIL);
      END; |

    ELSE
    END;

    Refresh(TRUE);

  END;   (* LOOP *)

CLOSE

  IF Window#NIL THEN I.CloseWindow(Window) END;
  IF Screen#NIL THEN I.CloseScreen(Screen) END;

END SaveIFF.
