(**************************************************************************

:Program.       Lists.mod
:Copyright.     Copyright  1990 by Fridtjof Siebert
:Language.      Oberon
:Translator.    Oberon Compiler V1.17.1
:History.       V1.0, 17-Jun-90 Fridtjof Siebert
:History.       V1.1, 10-Jan-91 Hartmut Goebel AddSection...,GoForw./Backw.
:Date.          24 Feb 1991 16:48:09

**************************************************************************)

MODULE Lists;

TYPE
  NodePtr* = POINTER TO Node;
  Node* = RECORD
            next,prev: NodePtr;
          END;
  List* = RECORD
            head* : NodePtr;
            tail* : NodePtr;
            remallowed: INTEGER;
          END;

  DoProc * = PROCEDURE(n: NodePtr);

(* Die DoProc darf Remove(), RemHead() und RemTail() nicht benutzen. *)


PROCEDURE Init*(VAR list: List);
BEGIN
  list.head := NIL;
  list.tail := NIL;
  list.remallowed := 0;
END Init;


PROCEDURE AddHead*(VAR list: List; n: NodePtr);
BEGIN
  n.next := list.head;
  n.prev := NIL;
  IF n.next=NIL THEN list.tail   := n;
                ELSE n.next.prev := n END;
  list.head := n;
END AddHead;


PROCEDURE AddTail*(VAR list: List; n: NodePtr);
BEGIN
  n.prev := list.tail;
  n.next := NIL;
  IF n.prev=NIL THEN list.head   := n;
                ELSE n.prev.next := n END;
  list.tail := n;
END AddTail;


PROCEDURE Remove*(VAR list: List; n: NodePtr);
BEGIN
  IF n#NIL THEN
    IF list.remallowed # 0 THEN HALT(20) END;
    IF n.next#NIL THEN n.next.prev := n.prev ELSE list.tail := n.prev END;
    IF n.prev#NIL THEN n.prev.next := n.next ELSE list.head := n.next END;
  END;
END Remove;


PROCEDURE RemHead*(VAR list: List): NodePtr;
VAR n: NodePtr;
BEGIN
  n := list.head; Remove(list,n); RETURN n;
END RemHead;


PROCEDURE RemTail*(VAR list: List): NodePtr;
VAR n: NodePtr;
BEGIN
  n := list.tail; Remove(list,n); RETURN n;
END RemTail;


PROCEDURE AddBefore*(VAR list: List;
                         n,x: NodePtr);
(* fgt n vor x in die Liste ein *)

BEGIN
  n.prev := x.prev;
  n.next := x;
  x .prev := n;
  IF n.prev=NIL THEN list.head   := n
                ELSE n.prev.next := n END;
END AddBefore;


PROCEDURE AddBehind*(VAR list: List;
                         n,x: NodePtr);
(* fgt n hinter x in die Liste ein *)

BEGIN
  n.next := x.next;
  n.prev := x;
  x .next := n;
  IF n.next=NIL THEN list.tail   := n
                ELSE n.next.prev := n END;
END AddBehind;


PROCEDURE Empty*(VAR list: List): BOOLEAN;
BEGIN
  RETURN list.head=NIL
END Empty;


PROCEDURE CountElements*(VAR list: List): LONGINT;
VAR
  i: LONGINT;
  n: NodePtr;
BEGIN
  i := 0;
  n := list.head;
  WHILE n#NIL DO n := n.next; INC(i) END;
  RETURN i;
END CountElements;


PROCEDURE DoForward*(VAR list: List; proc: DoProc);
VAR n: NodePtr;
BEGIN
  INC(list.remallowed);
  n := list.head; WHILE n#NIL DO proc(n); n := n.next END;
  DEC(list.remallowed);
END DoForward;


PROCEDURE DoBackward*(VAR list: List; proc: DoProc);
VAR n: NodePtr;
BEGIN
  INC(list.remallowed);
  n := list.tail; WHILE n#NIL DO proc(n); n := n.prev END;
  DEC(list.remallowed);
END DoBackward;


PROCEDURE Next*(VAR n: NodePtr): BOOLEAN;
BEGIN
  n := n.next;
  RETURN n#NIL;
END Next;


PROCEDURE Previous*(VAR n: NodePtr): BOOLEAN;
BEGIN
  n := n.prev;
  RETURN n#NIL;
END Previous;


PROCEDURE Succ*(VAR n: NodePtr);
BEGIN
  n := n.next;
END Succ;


PROCEDURE Pred*(VAR n: NodePtr);
BEGIN
  n := n.prev;
END Pred;


PROCEDURE Head*(VAR list: List): NodePtr;
BEGIN
  RETURN list.head;
END Head;


PROCEDURE Tail*(VAR list: List): NodePtr;
BEGIN
  RETURN list.tail;
END Tail;


PROCEDURE AddSectionBefore*(VAR list: List;
                         b,e,x: NodePtr);
(* fgt Nodes zwischen b und e vor x in die Liste ein *)

BEGIN
  b.prev := x.prev;
  e.next := x;
  x.prev := e;
  IF b.prev=NIL THEN list.head   := b
                ELSE b.prev.next := b END;
END AddSectionBefore;


PROCEDURE AddSectionBehind*(VAR list: List; b,e,x: NodePtr);
(* fgt Nodes zwischen b und e hinter x in die Liste ein *)

BEGIN
  e.next := x.next;
  b.prev := x;
  x.next := b;
  IF e.next=NIL THEN list.tail   := e
                ELSE e.next.prev := e END;
END AddSectionBehind;


PROCEDURE AddSectionHead*(VAR list: List; b,e: NodePtr);
BEGIN
  e.next := list.head;
  b.prev := NIL;
  IF e.next=NIL THEN list.tail   := e;
                ELSE e.next.prev := e END;
  list.head := b;
END AddSectionHead;


PROCEDURE AddSectionTail*(VAR list: List; b,e: NodePtr);
BEGIN
  b.prev := list.tail;
  e.next := NIL;
  IF b.prev=NIL THEN list.head   := b;
                ELSE b.prev.next := b END;
  list.tail := e;
END AddSectionTail;


PROCEDURE RemoveSection*(VAR list: List; b,e: NodePtr);
BEGIN
  IF (b#NIL) AND (e#NIL)THEN
    IF list.remallowed # 0 THEN HALT(20) END;
    IF e.next#NIL THEN e.next.prev := b.prev ELSE list.tail := b.prev END;
    IF b.prev#NIL THEN b.prev.next := e.next ELSE list.head := e.next END;
  END;
END RemoveSection;


PROCEDURE GoForward*(list: List; VAR n: NodePtr; num: LONGINT);
BEGIN
  WHILE (num>0) AND (n#NIL) DO
    n := n.next;
    DEC(num);
  END;
  IF n=NIL THEN n:=list.tail; END;
END GoForward;


PROCEDURE GoBackward*(list: List; VAR n: NodePtr; num: LONGINT);
BEGIN
  WHILE (num>0) AND (n#NIL) DO
    n := n.prev;
    DEC(num);
  END;
  IF n=NIL THEN n:=list.head; END;
END GoBackward;

END Lists.

