
(**********************************************************************

    :Program.    TimerTest
    :Contents.   Simpel test of module TimerSupport
    :Author.     Stefan Salewski
    :Address.    Stefan Salewski, Stolper Weg 3, D-2160 Stade
    :Copyright.  Public Domain
    :Language.   Oberon
    :Translator. Amiga-Oberon-Compiler V1.17.1
    :History.    V1.0    18-JAN-1991

**********************************************************************)

MODULE TimerTest;
  IMPORT io,
         NoGuru,
         MathTrans,
         MathIEEEDoubTrans,
         Break,
         TimerSupport;

  CONST
    ZT=10000;

  VAR
    month,day,year:LONGINT;
    min,sec,hour:LONGINT;
    x,y,z:REAL;
    xl,yl,zl:LONGREAL;
    i,j:LONGINT;
    m,m1,m2:LONGINT;
    n,n1,n2:INTEGER;
    time,time1:TimerSupport.TimeVal;
    trPtr:TimerSupport.TimeRequestPtr;

  PROCEDURE TodayIs(t:TimerSupport.TimeVal);
  BEGIN
    TimerSupport.SysToNormalTime(t,year,month,day,hour,min,sec);
    io.WriteString('Date: ');
    io.WriteInt(day,2);
    io.Write('.');
    io.WriteInt(month,2);
    io.Write('.');
    io.WriteInt(year,4);
    io.WriteString(' Time: ');
    io.WriteInt(hour,2);
    io.Write(':');
    io.WriteInt(min,2);
    io.Write(':');
    io.WriteInt(sec,2);
    io.WriteLn;
  END TodayIs;

BEGIN
  io.WriteString('SystemTime:');
  io.WriteLn;
  TimerSupport.GetSysTime(time);
  TodayIs(time);
  io.WriteString('Now we set SystemTime to one hour later');
  io.WriteLn;
  time1.secs:=60*60;
  time1.micro:=0;
  TimerSupport.AddTime(time,time1);
  TimerSupport.SetSysTime(time);
  io.WriteString('SystemTime:');
  io.WriteLn;
  TodayIs(time);
  io.WriteLn;

  trPtr:=TimerSupport.OpenTimer(TimerSupport.vBlank);
  IF trPtr=NIL THEN HALT(0) END;
  io.WriteString("Let's wait for 5 seconds");
  io.WriteLn;
  TimerSupport.Delay(trPtr,5,0);
  io.WriteString("You have 3 seconds to read this text");
  io.WriteLn;
  io.WriteString("In this time I calculate the sum of the numbers 1 to 999");
  io.WriteLn;
  TimerSupport.SendIO(trPtr,3,0);
  i:=0;j:=0;
  REPEAT
    INC(i);
    INC(j,i);
  UNTIL i=999;
  TimerSupport.WaitIO(trPtr);
  io.WriteString("The sum is:");
  io.WriteInt(j,8);
  io.WriteLn;

  io.WriteString('Now we test the time needed for arithmetic operations');
  io.WriteLn;
  io.WriteString('INTEGER: Zehntausend mal INC(n); x:=n;           dauert: ');
  TimerSupport.StartClock;
  n:=0;
  i:=0;
  REPEAT
    INC(n);
    x:=n;
    INC(i)
  UNTIL i=ZT;
  TimerSupport.ReadClock(time);
  TimerSupport.WriteTime(time);

  io.WriteString('LONGINT: Zehntausend mal x:=x+1.2; m:=ENTIER(x); dauert: ');
  TimerSupport.StartClock;
  x:=123.45678;
  i:=0;
  REPEAT
    x:=x+1.2;
    m:=ENTIER(x);
    INC(i)
  UNTIL i=ZT;
  TimerSupport.ReadClock(time);
  TimerSupport.WriteTime(time);

  io.WriteString('INTEGER: Zehntausend mal INC(n); n2:=n1+n;       dauert: ');
  TimerSupport.StartClock;
  n:=0;
  n1:=13;
  i:=0;
  REPEAT
    INC(n);
    n2:=n1+n;
    INC(i)
  UNTIL i=ZT;
  TimerSupport.ReadClock(time);
  TimerSupport.WriteTime(time);

  io.WriteString('INTEGER: Zehntausend mal INC(n); n2:=n1*n;       dauert: ');
  TimerSupport.StartClock;
  n:=0;
  n1:=3;
  i:=0;
  REPEAT
    INC(n);
    n2:=n1*n;
    INC(i)
  UNTIL i=ZT;
  TimerSupport.ReadClock(time);
  TimerSupport.WriteTime(time);

  io.WriteString('INTEGER: Zehntausend mal INC(n); n2:=n DIV n1;   dauert: ');
  TimerSupport.StartClock;
  n:=0;
  n1:=13;
  i:=0;
  REPEAT
    INC(n);
    n2:=n DIV n1;
    INC(i)
  UNTIL i=ZT;
  TimerSupport.ReadClock(time);
  TimerSupport.WriteTime(time);

  io.WriteString('INTEGER: Zehntausend mal INC(n); n2:=n MOD n1;   dauert: ');
  TimerSupport.StartClock;
  n:=0;
  n1:=13;
  i:=0;
  REPEAT
    INC(n);
    n2:=n MOD n1;
    INC(i)
  UNTIL i=ZT;
  TimerSupport.ReadClock(time);
  TimerSupport.WriteTime(time);

  io.WriteString('LONGINT: Zehntausend mal INC(m); m2:=m1+m;       dauert: ');
  TimerSupport.StartClock;
  m:=0;
  m1:=13;
  i:=0;
  REPEAT
    INC(m);
    m2:=m1+m;
    INC(i)
  UNTIL i=ZT;
  TimerSupport.ReadClock(time);
  TimerSupport.WriteTime(time);

  io.WriteString('LONGINT: Zehntausend mal INC(m); m2:=m1*m;       dauert: ');
  TimerSupport.StartClock;
  m:=0;
  m1:=13;
  i:=0;
  REPEAT
    INC(m);
    m2:=m1*m;
    INC(i)
  UNTIL i=ZT;
  TimerSupport.ReadClock(time);
  TimerSupport.WriteTime(time);

  io.WriteString('LONGINT: Zehntausend mal INC(m); m2:=m DIV m1;   dauert: ');
  TimerSupport.StartClock;
  m:=0;
  m1:=13;
  i:=0;
  REPEAT
    INC(m);
    m2:=m DIV m1;
    INC(i)
  UNTIL i=ZT;
  TimerSupport.ReadClock(time);
  TimerSupport.WriteTime(time);

  io.WriteString('LONGINT: Zehntausend mal INC(m); m2:=m MOD m1;   dauert: ');
  TimerSupport.StartClock;
  m:=0;
  m1:=13;
  i:=0;
  REPEAT
    INC(m);
    m2:=m MOD m1;
    INC(i)
  UNTIL i=ZT;
  TimerSupport.ReadClock(time);
  TimerSupport.WriteTime(time);

  io.WriteString('REAL:    Zehntausend mal x:=x+x; z:=x+y          dauert: ');
  TimerSupport.StartClock;
  x:=12.345;
  y:=67.89;
  i:=0;
  REPEAT
    x:=x+x;
    z:=x+y;
    INC(i)
  UNTIL i=ZT;
  TimerSupport.ReadClock(time);
  TimerSupport.WriteTime(time);

  io.WriteString('REAL:    Zehntausend mal x:=x+x; z:=x*y          dauert: ');
  TimerSupport.StartClock;
  x:=12.345;
  y:=67.89;
  i:=0;
  REPEAT
    x:=x+x;
    z:=x*y;
    INC(i)
  UNTIL i=ZT;
  TimerSupport.ReadClock(time);
  TimerSupport.WriteTime(time);

  io.WriteString('REAL:    Zehntausend mal x:=x+x; z:=x/y          dauert: ');
  TimerSupport.StartClock;
  x:=12.345;
  y:=67.89;
  i:=0;
  REPEAT
    x:=x+x;
    z:=x/y;
    INC(i)
  UNTIL i=ZT;
  TimerSupport.ReadClock(time);
  TimerSupport.WriteTime(time);

  io.WriteString('REAL:    Zehntausend mal x:=x+1.2; z:=sin(x)     dauert: ');
  TimerSupport.StartClock;
  x:=12.345;
  i:=0;
  REPEAT
    x:=x+1.2;
    z:=MathTrans.Sin(x);
    INC(i)
  UNTIL i=ZT;
  TimerSupport.ReadClock(time);
  TimerSupport.WriteTime(time);

  io.WriteString('REAL:    Zehntausend mal x:=x+1.2; z:=Atan(x)    dauert: ');
  TimerSupport.StartClock;
  x:=12.345;
  i:=0;
  REPEAT
    x:=x+1.2;
    z:=MathTrans.Atan(x);
    INC(i)
  UNTIL i=ZT;
  TimerSupport.ReadClock(time);
  TimerSupport.WriteTime(time);

  io.WriteString('LONGREAL:Zehntausend mal x:=x+x; z:=x+y          dauert: ');
  TimerSupport.StartClock;
  xl:=12.345;
  yl:=67.89;
  i:=0;
  REPEAT
    xl:=xl+xl;
    zl:=xl+yl;
    INC(i)
  UNTIL i=ZT;
  TimerSupport.ReadClock(time);
  TimerSupport.WriteTime(time);

  io.WriteString('LONGREAL:Zehntausend mal x:=x+x; z:=x*y          dauert: ');
  TimerSupport.StartClock;
  xl:=12.345;
  yl:=67.89;
  i:=0;
  REPEAT
    xl:=xl+xl;
    zl:=xl*yl;
    INC(i)
  UNTIL i=ZT;
  TimerSupport.ReadClock(time);
  TimerSupport.WriteTime(time);

  io.WriteString('LONGREAL:Zehntausend mal x:=x+x; z:=x/y          dauert: ');
  TimerSupport.StartClock;
  xl:=12.345;
  yl:=67.89;
  i:=0;
  REPEAT
    xl:=xl+xl;
    zl:=xl/yl;
    INC(i)
  UNTIL i=ZT;
  TimerSupport.ReadClock(time);
  TimerSupport.WriteTime(time);

  io.WriteString('LONGREAL:Zehntausend mal x:=x+1.2; z:=sin(x)     dauert: ');
  TimerSupport.StartClock;
  xl:=12.345;
  i:=0;
  REPEAT
    xl:=xl+1.2;
    zl:=MathIEEEDoubTrans.Sin(xl);
    INC(i)
  UNTIL i=ZT;
  TimerSupport.ReadClock(time);
  TimerSupport.WriteTime(time);

  io.WriteString('LONGREAL:Zehntausend mal x:=x+1.2; z:=Atan(x)    dauert: ');
  TimerSupport.StartClock;
  xl:=12.345;
  i:=0;
  REPEAT
    xl:=xl+1.2;
    zl:=MathIEEEDoubTrans.Atan(xl);
    INC(i)
  UNTIL i=ZT;
  TimerSupport.ReadClock(time);
  TimerSupport.WriteTime(time);

  TimerSupport.Delay(trPtr,5,0);
  TimerSupport.CloseTimer(trPtr);
END TimerTest.



